;+ 
;NAME: 
; thm_ui_data_range__define
;
;PURPOSE:  
; data range object 
;
;CALLING SEQUENCE:
; dataRange = Obj_New("THM_UI_DATA_RANGE")
;
;INPUT:
; none
;
;KEYWORDS:
; startData  start data 
; endData    end data        
;
;OUTPUT:
; data range object reference
;
;METHODS:
; SetProperty  procedure to set keywords 
; GetProperty  procedure to get keywords
; GetStartData returns the start data (default format is double)
; GetStopData  returns the stop data (default format is double)
; GetDuration  returns duration in seconds 
;
;HISTORY:
;
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-21 12:57:40 -0700 (Tue, 21 Oct 2008) $
;$LastChangedRevision: 3772 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_data_range__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_DATA_RANGE::Copy
   out = Obj_New("THM_UI_DATA_RANGE")
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out
   RETURN, out
END ;--------------------------------------------------------------------------------



PRO THM_UI_DATA_RANGE::SetProperty,   $ ; standard property set method 
      StartData=startdata,            $ ; value at start of data 
      EndData=enddata                   ; value at end of data           
   
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(startdata) NE 0 THEN self.startData = startdata
   IF N_Elements(enddata) NE 0 THEN self.endData = enddata

RETURN
END ;--------------------------------------------------------------------------------



PRO THM_UI_DATA_RANGE::GetProperty,   $
      StartData=startdata,            $ ; value at start of data 
      EndData=enddata                   ; value at end of data           
   
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

      ; Return only what the user asked for

   IF Arg_Present(startdata) THEN startdata = self.startData  
   IF Arg_Present(enddata) THEN enddata = self.endData   

RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_DATA_RANGE::Init, $
      StartData=startdata,        $ ; value at start of data 
      EndData=enddata,            $ ; value at end of data           
      Debug=debug                   ; flag to debug

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF
   
      ; Check for parameters
      
   IF N_Elements(startdata) EQ 0 THEN startdata = 0.0D
   IF N_Elements(enddata) EQ 0 THEN enddata = 0.0D
   
      ; Set all data range object attributes

   self.startData = startdata
   self.endData = enddata
                 
RETURN, 1
END ;--------------------------------------------------------------------------------



PRO THM_UI_DATA_RANGE__DEFINE

   struct = { THM_UI_DATA_RANGE,   $

      startData : 0.0D,    $ ; value where data starts  
      endData : 0.0D       $ ; value where data ends 
 
}

END
