;+ 
;NAME:
; thm_ui_marker__define
;
;PURPOSE:  
; Marker object, displayed whenever user ctrl-click-drags to highlight an area
;
;CALLING SEQUENCE:
; marker = Obj_New("THM_UI_MARKER")
;
;INPUT:
; none
;
;KEYWORDS:
; name              name for this marker
; range             start and stop value of marker (data coords)
; settings          property settings of the marker
; isSelected        flag set if marker is currently selected
; filename          filename, if saved
;
;OUTPUT:
; marker object reference
;
;METHODS:
; SetProperty   procedure to set keywords 
; GetProperty   procedure to get keywords 
; GetAll        returns the entire structure
;
;HISTORY:
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2008-12-17 14:25:17 -0800 (Wed, 17 Dec 2008) $
;$LastChangedRevision: 4233 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_marker__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER::Copy
   out = Obj_New('THM_UI_MARKER')
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out
   newSettings=Obj_New("THM_UI_MARKER_SETTINGS")
   IF Obj_Valid(self.Settings) THEN newSettings=self.settings->Copy() ELSE $
      newSettings=Obj_New()
   out->SetProperty, Settings=newSettings
;   newName=Obj_New("THM_UI_TEXT")
;   IF Obj_Valid(self.name) THEN newName=self.name->Copy() ELSE $
;      newName=Obj_New()
;   out->SetProperty, Name=newName
   RETURN, out
END ;--------------------------------------------------------------------------------
 
  
   
PRO THM_UI_MARKER::SetProperty,  $ ; standard property set method 
      Name=name,                 $ ; name for this marker
      Range=range,               $ ; start and end value of marker (in normalized coordinates)
      Settings=settings,         $ ; property settings of the marker
      IsSelected=isselected,     $ ; flag set if marker is selected
      FileName=filename            ; filename, if marker is saved
    
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(name) NE 0 THEN self.name = name
   IF N_Elements(range) NE 0 THEN self.range = range
   IF Obj_Valid(settings) THEN self.settings = settings
   IF N_Elements(isselected) NE 0 THEN self.isSelected = isselected 
   IF N_Elements(filename) NE 0 THEN self.filename = filename 

RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER::GetAll
RETURN, self
END ;--------------------------------------------------------------------------------



PRO THM_UI_MARKER::GetProperty,           $
      Name=name,                          $ ; name for this marker
      Range=range,                        $ ; start and end time of marker
      Settings=settings,                  $ ; property settings of the marker
      IsSelected=isselected,              $ ; flag set if marker is selected
      FileName=filename                     ; filename if marker is saved
    
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

      ; Return only what the user asked for

   IF Arg_Present(name) THEN name = self.name
   IF Arg_Present(range) THEN range = self.range   
   IF Arg_Present(settings) THEN settings = self.settings   
   IF Arg_Present(isselected) THEN isselected = self.isSelected   
   IF Arg_Present(filename) THEN filename = self.filename   

RETURN
END ;--------------------------------------------------------------------------------



PRO THM_UI_MARKER::Cleanup
;Obj_Destroy, self.settings
RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER::Init,     $
      Range=range,                $ ; start and end value of marker (in normalized coordinates) 
      Settings=settings,          $ ; property settings of the marker
      Name=name,                  $ ; name for this marker
      IsSelected=isselected,      $ ; 
      FileName=filename,          $ ; filename if marker is saved
      Debug=debug                   ; flag to debug
      
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF

      ; Check that all parameters have values
   
   IF N_Elements(range) EQ 0 THEN range = [0D,0D]
   IF NOT Obj_Valid(settings) THEN settings = Obj_New("THM_UI_MARKER_SETTINGS")
   IF N_Elements(name) EQ 0 THEN name = ''
   IF N_Elements(isselected) EQ 0 THEN isselected = 0
   IF N_Elements(filename) EQ 0 THEN filename = ''  

      ; Set all parameters

   self.name = name
   self.range = range
   self.settings = settings
   self.isSelected = isselected
   self.filename = filename
                  
RETURN, 1
END ;--------------------------------------------------------------------------------

PRO THM_UI_MARKER__DEFINE

   struct = { THM_UI_MARKER,  $

        name:'',              $ ; name for this marker
        range:[0D,0D],        $ ; start and end of marker in data coordinates
        settings:Obj_New(),   $ ; properties of this marker
        isSelected: 0,        $ ; flag set if this marker is selected
        filename:'',          $ ; filename, if saved
        INHERITS THM_UI_READWRITE $ ; generalized read/write methods

}

END ;--------------------------------------------------------------------------------

