;+ 
;NAME: 
; thm_ui_marker_title__define
;
;PURPOSE:  
; Marker object, displayed whenever user ctrl-click-drags to highlight an area
;
;CALLING SEQUENCE:
; markerTitle = Obj_New("THM_UI_MARKER_TITLE")
;
;INPUT:
; none
;
;KEYWORDS:
; name             name for this marker
; useDefault       flag set if using default name
; defaultName      default name for marker
; cancelled        flag set if window cancelled
; doNotAsk         flag set if user doesn't want to be asked each time
; 
;
;OUTPUT:
; marker object reference
;
;METHODS:
; SetProperty   procedure to set keywords 
; GetProperty   procedure to get keywords 
; GetAll        returns the entire structure
;
;HISTORY:
;
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-12-16 12:24:35 -0800 (Tue, 16 Dec 2008) $
;$LastChangedRevision: 4212 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_marker_title__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER_TITLE::Copy
   out = Obj_New("THM_UI_MARKER_TITLE")
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out
   RETURN, out
END ;--------------------------------------------------------------------------------
 
  
   
PRO THM_UI_MARKER_TITLE::SetProperty, $ ; standard property set method 
      Name=name,                      $ ; name for this marker
      UseDefault=usedefault,          $ ; flag set if user want to use default name
      DefaultName=defaultname,        $ ; default name of marker 
      Cancelled=cancelled,            $ ; flag set if window was cancelled 
      DoNotAsk=donotask                 ; flag set is user does not want to be asked every time
    
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(name) NE 0 THEN self.name = name
   IF N_Elements(usedefault) NE 0 THEN self.useDefault = usedefault
   IF N_Elements(defaultname) NE 0 THEN self.defaultName = defaultname
   IF N_Elements(cancelled) NE 0 THEN self.cancelled = cancelled
   IF N_Elements(donotask) NE 0 THEN self.doNotAsk = donotask 

RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER_TITLE::GetAll
RETURN, self
END ;--------------------------------------------------------------------------------



PRO THM_UI_MARKER_TITLE::GetProperty,   $
      Name=name,                        $ ; name for this marker
      UseDefault=usedefault,            $ ; flag set if user want to use default name
      DefaultName=defaultname,          $ ; default name of marker 
      Cancelled=cancelled,              $ ; flag set if window was cancelled 
      DoNotAsk=donotask                   ; flag set is user does not want to be asked every time
    
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

      ; Return only what the user asked for

   IF Arg_Present(name) THEN name = self.name
   IF Arg_Present(usedefault) THEN usedefault = self.useDefault
   IF Arg_Present(defaultname) THEN defaultname = self.defaultName   
   IF Arg_Present(cancelled) THEN cancelled = self.cancelled   
   IF Arg_Present(donotask) THEN donotask = self.doNotAsk  

RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MARKER_TITLE::Init,     $
      Name=name,                  $ ; name for this marker
      UseDefault=usedefault,      $ ; flag set if user want to use default name
      DefaultName=defaultname,    $ ; default name of marker 
      DoNotAsk=donotask,          $ ; flag set is user does not want to be asked every time
      Cancelled=cancelled,        $ ; flag set is cancelled the page
      Debug=debug                   ; flag to debug
      
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF

      ; Check that all parameters have values
   
   IF N_Elements(name) EQ 0 THEN name = ''
   IF N_Elements(usedefault) EQ 0 THEN useDefault = 0
   IF N_Elements(defaultname) EQ 0 THEN defaultname = ''
   IF N_Elements(cancelled) EQ 0 THEN cancelled = 1
   IF N_Elements(donotask) EQ 0 THEN donotask = 0  

      ; Set all parameters

   self.name = name
   self.useDefault = usedefault
   self.defaultName = defaultname
   self.cancelled = cancelled
   self.doNotAsk = donotask

                  
RETURN, 1
END ;--------------------------------------------------------------------------------



PRO THM_UI_MARKER_TITLE__DEFINE

   struct = { THM_UI_MARKER_TITLE,  $

        name:'',              $ ; name for this marker
        useDefault: 0,        $ ; flag set if user want to use default name
        defaultName: '',      $ ; default name of marker
        cancelled: 0,            $ ; flag to indicate whether the window was canceled 
        doNotAsk: 0           $ ; flag set is user does not want to be asked every time

}

END ;--------------------------------------------------------------------------------
