;+ 
;NAME:  
;  thm_ui_make_default_specplot
;  
;PURPOSE:
;  Routine that creates a default specplot on particular panel
;  
;CALLING SEQUENCE:
;  thm_ui_make_default_specplot, loadedData panel, xvar, yvar, zvar
;  
;INPUT:
;  loadedData: the loadedData object
;  panel:  the panel on which the plot should be placed.
;  xvar:  the name of the variable storing the x-data.
;  yvar:  the name of the variable storing the y-data.
;  zvar:  the name of the variable storing the z-data.
;            
;KEYWORDS:
;  none
;        
;OUTPUT:
;  none
;
;--------------------------------------------------------------------------------

pro thm_ui_make_default_specplot, loadedData, panel, xvar, yvar, zvar

compile_opt idl2, hidden

panel->GetProperty, traceSettings=traceSettings, xAxis=xAxis, yAxis=yAxis, $
                    zAxis=zAxis
;ntraces = traceSettings->count()

; check if z-axis object exists, create z-axis and make sure y-axis is log
if ~obj_valid(zAxis) then begin
  
  xAxis->setProperty, annotateLastTick=0

  yAxis->setProperty, rangeOption=0, $
                      rangeMargin=.00, $
                      scaling=0, $
                      lineatzero=0
  
  zAxis = obj_new('thm_ui_zaxis_settings', $
                   colorTable = 6,$
                   fixed = 0,$
                   tickNum = 5, $
                   minorTickNum = 8, $
                   labelmargin = 40, $
                   scaling = 1, $
                   annotationStyle = 4, $
                   margin = 5, $
                   annotateTextObject = obj_new('thm_ui_text',size=8.,font=2,format=3), $
                   annotationOrientation = 0, $
                   ;labelTextObject = obj_new('thm_ui_text',value='FLUX in eV'),$
                   labelOrientation=1, $
                   placement=3)
endif

; check if x-axis object exists, create if not
if ~obj_valid(xAxis) then xAxis = obj_new('thm_ui_axis_settings', $
                                  isTimeAxis=1, $
                                  orientation=0, $
                                  majorLength=10., $
                                  minorLength=5., $
                                  topPlacement=1, $
                                  bottomPlacement=1, $
                                  tickStyle=0, $
                                  majorTickAuto=1, $
                                  numMajorTicks=5, $
                                  minorTickAuto=1, $
                                  numMinorTicks=5, $
                                 ; majorGrid=majorGrid, $
                                 ; minorGrid=minorGrid, $
                                  dateString1="%date", $
                                  dateString2="%time", $
                                  showdate=1,$
                                  annotateAxis=1,$
                                  placeAnnotation=0,$
                                  AnnotateMajorTicks=1,$
                                  annotatefirstTick=0,$
                                  annotateLastTick=0,$
                                  annotateStyle=5,$
                                  annotateTextObject=obj_new('thm_ui_text',font=2,size=8.,color=[0,0,0]),$
                                  scaling=0,$
                                  rangeMargin=0., $
                                  margin=15., $
                                  annotateOrientation=0 $
                                  ;labels=labels,$
                                  ;stacklabels=0 $
                                  ;rangeoption=2,$
                                  )

; check if y-axis object exists, create if not
if ~obj_valid(yAxis) then yAxis = obj_new('thm_ui_axis_settings', $
                                   isTimeAxis=0,$
                                   orientation=1,$
                                   majorLength=10.,$
                                   minorLength=5.,$
                                   topPlacement=1,$
                                   bottomPlacement=1,$
                                   tickStyle=0,$
                                   majorTickAuto=1,$
                                   numMajorTicks=5,$
                                   minorTickAuto=1,$
                                   numMinorTicks=8,$
                                   ;majorGrid=majorGrid,$
                                   ;minorGrid=minorGrid,$
                                   ;/lineatzero,$
                                   annotateAxis=1,$
                                   placeAnnotation=0,$
                                   AnnotateMajorTicks=1,$
                                   annotateLastTick=0,$
                                   annotateFirstTick=0,$
                                   annotateStyle=4,$
                                   annotateTextObject=obj_new('thm_ui_text',font=2,size=8.,color=[0,0,0]),$
                                   rangeOption=0,$
                                   rangeMargin=.00, $
                                   scaling=1, $
                                   annotateOrientation=1, $
                                   margin=45. $
                                   ;labels=labels, $
                                   ;stackLabels=1 $
                                   )

yAxis->getProperty, labels=ylabels
if ~obj_valid(ylabels) then ylabels = obj_new('IDL_Container')
xAxis->getProperty, labels=xlabels
if ~obj_valid(xlabels) then xlabels = obj_new('IDL_Container')

ylabels->add,obj_new('thm_ui_text', value='', font=2, format=3, size=8, $
                    color=[0,0,0],show=0)
                    
xlabels->add,obj_new('thm_ui_text',value ='',font=2,format=3,size=8,$
                    color=[0,0,0],show=0)
                    
traceSettings->add,obj_new('thm_ui_spectra_settings', dataX=xvar, dataY=yvar, dataZ=zvar)

; get data object for x and y quantities
xObj = loadedData->getObjects(name=xvar[0])
yObj = loadedData->getObjects(name=yvar[0])

xObj->getProperty, dlimitptr=dlptr, limitptr=lptr
extract_tags,*dlptr,*lptr
dl = *dlptr

if (size(dl,/type) eq 8) then begin

  ; set y-axis scaling according to limits/dlimits
  if in_set('ylog', strlowcase(tag_names(dl))) then begin
    yAxis->setProperty, scaling=dl.ylog
  endif; else yAxis->setProperty, scaling=0

  ; set y-axis limits and turn on fixed yrange
  if in_set('yrange', strlowcase(tag_names(dl))) then begin
    if dl.yrange[0] ne dl.yrange[1] then begin ; make sure the range isn't zero
      yaxis->getproperty, minfixedrange=minfixedrange, maxfixedrange=maxfixedrange, scaling=scaling
      if (minfixedrange eq 0) AND (maxfixedrange eq 0) then begin
        minfixedrange = dl.yrange[0]
        maxfixedrange = dl.yrange[1]
      endif
      if dl.yrange[0] lt minfixedrange then minfixedrange = dl.yrange[0]
      if dl.yrange[1] gt maxfixedrange then maxfixedrange = dl.yrange[1]
      
      if scaling then begin
        if (dl.yrange[0] le 0) OR (dl.yrange[1] le 0) then yAxis->setProperty, rangeoption=0 $
        else begin
          yAxis->setProperty, minfixedrange=minfixedrange, maxfixedrange=maxfixedrange, $
                              rangeoption=0
        endelse
      endif else begin
        yAxis->setProperty, minfixedrange=minfixedrange, maxfixedrange=maxfixedrange, $
                            rangeoption=0
      endelse
    endif
  endif
  
  ; set z-axis limits
  if in_set('zrange', strlowcase(tag_names(dl))) then begin
    zAxis->setProperty, minrange=dl.zrange[0], maxrange=dl.zrange[1]
  endif
  
  ; set z-axis scale type
  if in_set('zlog', strlowcase(tag_names(dl))) then begin
    zAxis->setProperty, scaling=dl.zlog
    if dl.zlog ne 0 then begin
      zAxis->setProperty, minorTickNum=1
    endif
  endif
  
endif

; check whether or not data is time quantity
xObj->getProperty, istime=xistime
if n_elements(yObj) gt 1 then begin
  yistime=0
endif else begin
  yObj->getProperty, istime=yistime
endelse

; set x and y axis properties
xAxis->SetProperty, labels=xlabels, isTimeAxis=xistime

;Set number of minor ticks(xAxis)
if xistime then begin
  xAxis->setProperty,numMinorTicks = 5
endif else begin
  xAxis->getProperty,scaling=xscale
  if xscale eq 1 then begin
    xAxis->setProperty,numMinorTicks = 8
  endif else if xScale eq 2 then begin
    xAxis->setProperty,numMinorTicks = 0
  endif else begin
    xAxis->setProperty,numMinorTicks = 4
  endelse
endelse

yAxis->SetProperty, labels=ylabels, isTimeAxis=yistime

;Set number of minor ticks(yAxis)
if yistime then begin
  yAxis->setProperty,numMinorTicks = 5
endif else begin
  yAxis->getProperty,scaling=yscale
  if yscale eq 1 then begin
    yAxis->setProperty,numMinorTicks = 8
  endif else if yScale eq 2 then begin
    yAxis->setProperty,numMinorTicks = 0
  endif else begin
    yAxis->setProperty,numMinorTicks = 4
  endelse
endelse

zAxis->getProperty,scaling=zscale
if zscale eq 1 then begin
  zAxis->setProperty,minorTickNum = 8
endif else if zScale eq 2 then begin
  zAxis->setProperty,minorTickNum = 0
endif else begin
  zAxis->setProperty,minorTickNum = 4
endelse

panel->SetProperty, traceSettings=traceSettings, xAxis=xAxis, yAxis=yAxis, $
                    zAxis=zAxis

end