
;+ 
;NAME:
; thm_ui_load_data_file.pro
;
;PURPOSE:
; A widget interface for selecting THEMIS data
;
;CALLING SEQUENCE:
; thm_ui_load_data_file, master_widget_id
;
;INPUT:
; master_widget_id     the id of the widget that calls this routine
;
;OUTPUT:
; not sure yet
;
;HISTORY:
;$LastChangedBy: bckerr $
;$LastChangedDate: 2009-04-20 09:47:20 -0700 (Mon, 20 Apr 2009) $
;$LastChangedRevision: 5693 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/panels/thm_ui_load_data_file/thm_ui_load_data_file.pro $
;
;-

; TODO: When we get around to loading non-themis data, might need a pop-up window for when we don't know the coordsys of incoming data
Pro thm_ui_load_data_file_event, event, info

  Compile_Opt idl2, hidden

      ; get the state structure from the widget

  Widget_Control, event.TOP, Get_UValue=state, /No_Copy

      ; put a catch here to insure that the state remains defined

  err_xxx = 0
  Catch, err_xxx
  IF (err_xxx NE 0) THEN BEGIN
    Catch, /Cancel
    Help, /Last_Message, Output = err_msg
    if is_struct(state) then begin
      FOR j = 0, N_Elements(err_msg)-1 DO state.historywin->update,err_msg[j]
      x=state.gui_id
      histobj=state.historywin
    endif
    Print, 'Error--See history'
    ok=error_message('An unknown error occured and the window must be restarted. See console for details.',$
       /noname, /center, title='Error in Load Data')
    Widget_Control, event.TOP, Set_UValue=state, /No_Copy
    widget_control, event.top,/destroy
    if widget_valid(x) && obj_valid(histobj) then begin 
      thm_gui_error,x,histobj
    endif
    RETURN
  ENDIF

    ;kill request block

  IF (Tag_Names(event, /Structure_Name) EQ 'WIDGET_KILL_REQUEST') THEN BEGIN  


    exit_sequence:
    Print, 'Load Themis Data widget killed.' 
    state.historyWin->Update,'THM_UI_LOAD_DATA_FILE: Widget closed' 
    Widget_Control, event.TOP, Set_UValue=state, /No_Copy
    Widget_Control, event.top, /Destroy
    RETURN 

  ENDIF


      ; get the uval from this event so we know which widget was used 

  Widget_Control, event.id, Get_UValue=uval
  
  state.historywin->update,'THM_UI_LOAD_DATA_FILE: User value: '+uval  ,/dontshow

  CASE uval OF
    'ADD': BEGIN
      ; make sure valid times are set
      widget_control, state.startText, get_value=startval
      widget_control, state.stopText, get_value=stopval
      startt=thm_ui_timefix(startval)
      stopt=thm_ui_timefix(stopval)
      if is_string(startt) AND is_string(stopt) then begin
        ; make sure an observatory is selected
        if ~array_equal(*state.observ, '', /no_typeconv) then begin
          widget_control, /hourglass
          state.statusText->Update, 'Loading data...'
          thm_ui_load_data_file_load, state, event
          IF Obj_Valid(state.loadedData) && state.dataFlag EQ 0 THEN BEGIN
             dataNames=state.loadedData->GetAll()
             IF is_string(dataNames) && state.dataFlag EQ 0 THEN BEGIN
                FOR i=0,N_Elements(state.dataButtons)-1 DO Widget_Control, state.dataButtons[i], sensitive=1
                state.dataFlag=1  
            ENDIF
          ENDIF
          state.loadlist->update
        endif else begin
          if state.instr eq 'none' then begin
            h = 'No instrument selected.  Please select an instrument.'
          endif else begin
            h = 'No probe selected.  Please select one or more probes.'
          endelse
          
          state.statusText->Update, h
          state.historyWin->Update, 'THM_UI_LOAD_DATA_FILE: ' + h
        endelse
      endif else begin
        state.statusText->update, 'Invalid Start and/or Stop Time.  ' + $
                          'Please enter valid time before loading data.'
        break
      endelse
    END
    'CANC': BEGIN
       Print, 'New File widget canceled' 
       state.historyWin->Update, 'THM_UI_LOAD_DATA_FILE: Widget closed.'
       Widget_Control, event.top, Set_UValue=state, /No_Copy
       Widget_Control, event.top, /Destroy
       RETURN
    END
    'CLEAR':BEGIN ; Clear all data (tplot vars)
      widget_control, /hourglass
      val_data = state.loadeddata->getall();/times)
      if ptr_valid(state.data_sel) then ptr_free, state.data_sel
      state.data_sel = ptr_new(val_data)
      thm_ui_load_data_file_del, state
      h = 'All data deleted.'
      state.statusText->Update, h
      state.historyWin->Update, 'THM_UI_LOAD_DATA_FILE: ' + h
      state.loadedData->reset
      state.loadlist->update    ; update tree widget
    END
    'CLEAR_DTYP':Begin
      if ptr_valid(state.dtyp1) then ptr_free, state.dtyp1
      if ptr_valid(state.dtyp2) then ptr_free, state.dtyp2
      if ptr_valid(state.dtyp) then ptr_free, state.dtyp
      widget_control, state.level1List, set_value=*state.dlist1
      widget_control, state.level2List, set_value=*state.dlist2
      h = 'No chosen data types'
      state.statusText->Update, h
    END
    'CLEAR_PRST':Begin
      If(state.instr Eq 'asi' Or state.instr Eq 'ask') Then Begin
        If(ptr_valid(state.astation)) Then ptr_free, state.astation
        h = 'No Chosen Asi_station'
        state.statusText->Update, h
      Endif Else If(state.instr Eq 'gmag') Then Begin
        If(ptr_valid(state.station)) Then ptr_free, state.station
        h = 'No Chosen Gmag_station'
        state.statusText->Update, h
      Endif Else Begin
        If(ptr_valid(state.probe)) Then ptr_free, state.probe      
        h = 'No Chosen Probe'
        state.statusText->Update, h
      Endelse
      widget_control,state.observlist, set_value=*state.validobservlist
    END
    'COORD_DLIST':BEGIN ; Output Coordinates dropdown list
      thm_ui_load_data_file_coord_sel, state    
    END
    'DELDATA':BEGIN ; Clear selected data (tplot vars)
      widget_control, /hourglass
      thm_ui_load_data_file_del, state
      state.loadlist->update
    END
    'DISMISS':BEGIN
       Print, 'New File widget dismissed' 
       Widget_Control, event.top, Set_UValue=state, /No_Copy
       Widget_Control, event.top, /Destroy
       RETURN
    END
    'ITYPE_DLIST':BEGIN ; Instrument Type dropdown list
      thm_ui_load_data_file_itype_sel, state
    END
    'LEVEL1': BEGIN ; Level 1 data list
      thm_ui_load_data_file_l1_sel, state
    END
    'LEVEL2': BEGIN ; Level 2 data list
      thm_ui_load_data_file_l2_sel, state
    END
    'LOADLIST':BEGIN
      thm_ui_load_data_file_loadlist, state, event
    END
    'OBSERV_LIST':BEGIN ; Observatory list (probes, ground stations, etc.)
      thm_ui_load_data_file_obs_sel, state
    END
    'START_TIME':BEGIN ; Start Time entry box
      widget_control,event.id,get_value=value
      t0 = thm_ui_timefix(value)
      If(is_string(t0)) Then Begin
;get both times for limit checking
          state.tr->GetProperty, startTime=st ;st is starttime object (thm_ui_time__define)
          state.tr->GetProperty, endTime=et   ;et is endtime object
;set start time value
          st->SetProperty, tstring = value
;return a warning if the time range is less than zero, or longer than 1 week
          et->getproperty, tdouble=t1
          st->getproperty, tdouble=t0
          If(t1 Le t0) Then state.statustext->update, 'End time less than/equal to Start time.' $
          Else If((t1-t0) Gt 7.0*86400.0d0) Then state.statustext->update, 'Time range is longer than One Week.' $
          Else state.statustext->update, 'Valid Start Time Entered'
      Endif Else state.statustext->update, 'INVALID START TIME ENTERED'
    END
    'STARTCAL':BEGIN;Calendar button
      widget_control, state.startText, get_value= val
      start=thm_ui_timefix(val)

      state.statusText->update, 'Invalid start time, please enter valid time before opening calendar.'

      state.tr->getproperty, starttime=start_obj
      state.tr->getproperty, endtime=end_obj
      thm_ui_calendar_new,'Choose Start Date/Time:',start_obj, state.gui_id
      start_obj->getproperty, tdouble=t0
      end_obj->getproperty, tdouble=t1
      if t1 le t0 then state.statustext->update, 'End time less than/equal to Start time.' $
        else if(t1-t0) gt 7.0*86400.0d0 then state.statustext->update, 'Time range is longer than One Week.' $
        else state.statustext->update, 'Valid Start Time Entered'
      start_obj->getproperty, tstring=start
      widget_control,state.starttext, set_value = start        
    END
    'STOP_TIME':BEGIN ; Stop Time entry box
      widget_control,event.id,get_value=value
      t0 = thm_ui_timefix(value)
      If(is_string(t0)) Then Begin
          state.tr->GetProperty, startTime=st ;st is starttime object (thm_ui_time__define)
          state.tr->GetProperty, endTime=et ;et is endtime object (thm_ui_time__define)
;Set end time value
          et->SetProperty, tstring = value
;return a warning if the time range is less than zero, or longer than 1 week
          et->getproperty, tdouble=t1
          st->getproperty, tdouble=t0
          If(t1 Le t0) Then state.statustext->update, 'End time less than/equal to Start time.' $
          Else If((t1-t0) Gt 7.0*86400.0d0) Then state.statustext->update, 'Time range is longer than One Week.' $
          Else state.statustext->update, 'Valid End Time Entered'
      Endif Else state.statustext->update, 'INVALID END TIME ENTERED'
    END
    'STOPCAL':BEGIN;Calendar button
      widget_control, state.stopText, get_value= val
      endt=thm_ui_timefix(val)

      state.statusText->update, 'Invalid end time, please enter valid time before opening calendar.'

      state.tr->getproperty, starttime=start_obj
      state.tr->getproperty, endtime=end_obj
      thm_ui_calendar_new,'Choose End Date/Time:',end_obj, state.gui_id
      start_obj->getproperty, tdouble=t0
      end_obj->getproperty, tdouble=t1
      if t1 le t0 then state.statustext->update, 'End time less than/equal to Start time.' $
        else if(t1-t0) gt 7.0*86400.0d0 then state.statustext->update, 'Time range is longer than One Week.' $
        else state.statustext->update, 'Valid Start Time Entered'
      end_obj->getproperty, tstring=endt
      widget_control,state.stoptext, set_value = endt     
    END
    ELSE: Print, ''    
  ENDCASE
  
      ; must ALWAYS reset the state value
      
  Widget_Control, event.top, Set_UValue = state, /No_Copy

  RETURN
END

Pro thm_ui_load_data_file, gui_id, windowStorage, loadedData, historyWin, $
                           dataFlag, dataButtons, timerange

  Compile_Opt idl2, hidden
  
  widget_control, /hourglass

  dtyp1='' & dtyp2='' & dtyp10='' & dtyp20='' & dtype='' & observ0='' & dtyp=''
  outCoord0=' DSL' ; default output coordinates
;  astation0 = '*' ; default all-sky station
;  station0 = '*' ; defautl ground mag station
;  probe0 = '*' ; default probe
  astation0 = '' ; default all-sky station
  station0 = '' ; defautl ground mag station
  probe0 = '' ; default probe
  probes = ['*', 'a', 'b', 'c', 'd', 'e', 'f']


  If(ptr_valid(dtyp)) Then ptr_free, dtyp
  dtyp = ptr_new(dtyp)
  If(ptr_valid(dtyp1)) Then ptr_free, dtyp1
  dtyp1 = ptr_new(dtyp1)
  If(ptr_valid(dtyp2)) Then ptr_free, dtyp2
  dtyp2 = ptr_new(dtyp2)

  
  observ_labels=['All-Sky Ground Station','GMAG Ground Station','Probes']
  observ_label=observ_labels[0]+':' ; default instrument label
;  instr_in0 = 'ASI' ; default data type
  instr_in0 = 'ASK' ; default data type
;  thm_load_asi, /valid_names, site=asi_stations
  thm_load_ask, /valid_names, site=asi_stations
  validobservlist = ['* (All)', asi_stations]
  validobserv = validobservlist
  validobserv = ptr_new(validobserv)
  observ = ptr_new(observ0) 

  instr_in=strlowcase(instr_in0)
  instr=instr_in
  
  outCoord = strcompress(strlowcase(outCoord0),/remove_all)
  ;outCoord = 'N/A'
  
;TODO: remember to reset this!  
;  starttime = '1970-01-01/00:00:00.00'
;  stopt = '1970-01-02/00:00:00.00'
  starttime = '2007-03-23/00:00:00'
  stoptime = '2007-03-24/00:00:00'
  
  ; Start/Stop Time objects
  if obj_valid(timerange) && (obj_class(timerange) eq 'THM_UI_TIME_RANGE') then begin
    tr = timerange
    tr->getproperty, startTime=startt, endTime=stopt
    startt->getproperty, tdouble=stdouble
    if stdouble ne 0 then begin
      startt->getproperty,tstring=starttime
      stopt->getproperty,tstring=stoptime
    endif else begin
      startt->setproperty,tstring=starttime
      stopt->setproperty,tstring=stoptime
    endelse
  endif else begin
    tr = Obj_New("THM_UI_TIME_RANGE", startTime=startt, endTime=stopt)
    startt->setproperty,tstring=starttime
    stopt->setproperty,tstring=stoptime
    timerange=tr
  endelse

;Get valid datatypes, probes, etc for different data types  
  dlist = thm_ui_new_valid_datatype(instr_in, ilist, llist)
  dlist1_all = ['*', dlist]
  dlist2_all = 'None'
  dlist1 = ptr_new(dlist1_all) & dlist2 = ptr_new(dlist2_all)
  ;dlist1 = dlist1_all & dlist2 = dlist2_all
  
      ;master widget
  
  tlb = Widget_Base(/Col, Title = 'THEMIS: Load Data ', Group_Leader = gui_id, $
                    /Modal, /Floating, /TLB_KILL_REQUEST_EVENTS)
                    
      ;base widgets
 
  topBase = Widget_Base(tlb, /Row, /Align_Top, /Align_Left, YPad=1) 
  buttonBase = Widget_Base(tlb, /Row, /Align_Center, YPad=8) 
  statusBase = Widget_Base(tlb, /Row)
    dataBase = Widget_Base(topBase, /Col, /Align_Left, YPad=1)
    ldBase = Widget_Base(topBase, /Col)
      toploadBase= Widget_Base(ldBase, /Row)
      bottomloadBase = Widget_Base(ldBase, /Row, YPad=6, /Align_Center)
      addBase = Widget_Base(toploadBase, /Col, /Align_Left, YPad=1, XPad=5)
      loadBase = Widget_Base(toploadBase, /Col, /Align_Left, YPad=1)
    data_label = Widget_Label(dataBase, Value='Data Selection:', /Align_Left)  
    dlistBase = Widget_Base(dataBase, /Col, XPad=2, Frame=3)
      top1Base = Widget_Base(dlistBase, /Col)
        droplistBase = Widget_Base(top1Base, /Row)
          itypeBase = Widget_Base(droplistBase, /Row)
          coordBase = Widget_Base(droplistBase, /Row)
        dbottomBase = Widget_Base(top1Base, /Col)
        observBase = Widget_Base(top1Base, /Row)
          observBase2 = Widget_Base(observBase, /Col)
            o1Base = Widget_Base(observBase2, /Col)
              o1ListBase = Widget_Base(o1Base, /Col)
          o2Base = Widget_Base(observBase, /Col)
            levelBase = Widget_Base(o2Base, /Row)
              llabelBase = Widget_Base(levelBase, /Row)
              level1Base = Widget_Base(levelBase, /Col)
              level2Base = Widget_Base(levelBase, /Col)
      

      ;widgets
    
;  validIType = [' ASI     ', ' ASK', ' ESA', ' EFI', ' FBK', ' FFT', ' FGM', $
;               ' FIT ', ' GMAG', ' MOM', ' SCM', ' SST', ' STATE']
  validIType = [' None    ', ' ASK', ' ESA', ' EFI', ' FBK', ' FFT', ' FGM', $
               ' FIT ', ' GMAG', ' MOM', ' SCM', ' SST', ' STATE']

  itypeDroplistLabel = Widget_Label(itypeBase, Value='Instrument Type:  ')
  itypeDroplist = Widget_ComboBox(itypeBase, Value=validIType, $ ;XSize=160, $
                                 uval='ITYPE_DLIST')
  widget_control, itypeDroplist, set_combobox_select=1
  validProbes = [' * (All)', ' A (P5)', ' B (P1)', ' C (P2)', ' D (P3)', $
                 ' E (P4)', ' F (Flatsat)']
  
  observLabel = Widget_Label(o1ListBase, Value=observ_label)
  observList = Widget_List(o1ListBase, Value=*validobserv, uval='OBSERV_LIST', $
                         /Multiple, XSize=16, YSize=11)
  
  level1Label = Widget_Label(level1Base, Value='Level 1:')
  level1List = Widget_List(level1Base, Value=*dlist1, XSize=19, /Multiple, YSize=11, $
                           Uvalue='LEVEL1')
  
  level2Label = Widget_Label(level2Base, Value='Level 2:')
  level2List = Widget_List(level2Base, Value=*dlist2, /Multiple, XSize=19, YSize=11, $
                           Uvalue='LEVEL2')
                           
  validCoords = [ ' DSL ', ' GSM ', ' SPG  ', ' SSL ',' GSE ', ' GEI ']

  coordDroplistLabel = Widget_Label(coordBase, Value=' Output Coordinates:  ')
  coordDroplist = Widget_ComboBox(coordBase, Value=validCoords, $ ;XSize=165, $
                                  Sensitive=0, uval='COORD_DLIST')
  
  getresourcepath,rpath
  cal = read_bmp(rpath + 'cal.bmp', /rgb)
  thm_ui_match_background, tlb, cal  
  
  ttextBase = Widget_Base(dbottomBase, /Col, tab_mode=1, YPad=3)
  startBase = Widget_Base(ttextBase, /Row)
  stopBase = Widget_Base(ttextBase, /Row)
  formatBase = Widget_Base(ttextBase, /Row)
  startLabel = Widget_Label(startBase, Value='Start Time: ', xsize=105)
  startText = Widget_Text(startBase, /Editable, /All_events, $
                          ;Value='2008-01-01/00:00:00.00  ', $
                          Value=starttime, $
                          UValue='START_TIME')
  startcal = widget_button(startbase, val = cal, /bitmap, tab_mode=0, uval='STARTCAL', uname='startcal', $
                           tooltip='Choose date/time from calendar.', sensitive=1)
  stopLabel = Widget_Label(stopBase, Value='Stop Time: ', xsize=105)
  stopText = Widget_Text(stopBase, /Editable, /All_events, $
                         ;Value='2008-01-02/00:00:00.00  ', $
                         Value=stoptime, $
                         UValue='STOP_TIME')
  stopcal = widget_button(stopbase, val = cal, /bitmap, tab_mode=0, uval='STOPCAL', uname='stopcal', $
                          tooltip='Choose date/time from calendar.', sensitive=1)
  spLabel = Widget_Label(formatBase, $
    Value='  ', xsize=100)
  formatLabel = Widget_Label(formatBase, $
    Value='Format: YYYY-MM-DD/hh:mm:ss')
  
  ;clear buttons
  clearbuts = Widget_Base(dbottomBase, /Row)
  clearbut1 = Widget_Button(observBase2, val = ' Clear Probe/Station ', $
                            uval = 'CLEAR_PRST', /align_center, $
                            ToolTip='Deselect all probes/stations')
  clearbut2 = Widget_Button(o2Base, val = ' Clear Data Type ', $
                            uval = 'CLEAR_DTYP', /align_center, $
                            ToolTip='Deselect all data types')

;A very annoying IDL 'feature'
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')
  spaceLabel = Widget_Label(addBase, Value='     ')

  rightArrow = read_bmp(rpath + 'arrow_000_medium.bmp', /rgb)
  leftArrow = read_bmp(rpath + 'arrow_180_medium.bmp', /rgb)
  
  thm_ui_match_background, tlb, rightArrow 
  thm_ui_match_background, tlb, leftArrow
  
  
  addButton = Widget_Button(addBase, Value=rightArrow, /Bitmap,  UValue='ADD', $
              ToolTip='Load data selection')
  minusButton = Widget_Button(addBase, Value=leftArrow, /Bitmap, $
                Uvalue='DELDATA', $
                ToolTip='Delete data selected in the list of loaded data')

  ; ============== Setup loaded data list ======================================
  if obj_valid(loadedData) then begin
    val_data = loadedData->getall(/times)
    if array_equal(val_data,0,/no_typeconv) then val_data = 'None' $
      else begin
        ndata = n_elements(val_data[0,*])
        val_data_temp = strarr(ndata)
        ; would like to vectorize this but need loaded data to be vectorized.
        lastParent = ''
        for i=0, ndata-1 do begin
          ;child = loadedData->isChild(val_data[0,i])
          
          parent = loadedData->isParent(val_data[0,i])
          
          ;this logic is in-case parent and child have the same name
          if parent && val_data[0,i] ne lastParent then begin
            lastParent =val_data[0,i]
            val_data_temp[i] = val_data[0,i]+':    '+val_data[1,i]+' to '+val_data[2,i]
          endif else begin
            val_data_temp[i] = '  - '+val_data[0,i]+':    '+val_data[1,i]+' to '+val_data[2,i]
          endelse
          
;          if child then begin
;            val_data_temp[i] = '  - '+val_data[0,i]+':    '+val_data[1,i]+' to '+val_data[2,i]
;          endif else begin
;            val_data_temp[i] = val_data[0,i]+':    '+val_data[1,i]+' to '+val_data[2,i]
;          endelse
        endfor
        val_data = val_data_temp
      endelse
  endif else val_data = 'None'

  loadLabel  = Widget_Label(loadBase, Value='Data Loaded: ', /Align_Left)
  loadList = Obj_New('thm_ui_widget_tree', loadBase, 'LOADLIST', loadedData, $
                     XSize=400, YSize=325, mode=0, /multi,/showdatetime)
;  loadList = Widget_List(loadBase, Value=val_data, UValue='LOADLIST', $
;                         XSize=75, YSize=24, /multiple)
  ; ============== End setup loaded data list =================================

;  cancelButton = Widget_Button(bottomloadBase, Value='Cancel', XSize=82, UValue='CANC', $
;    ToolTip='Cancel this operation')
  clearButton = Widget_Button(bottomloadBase, Value='Clear All', XSize=82, UValue='CLEAR', $
    ToolTip='Clears all loaded data')
  okButton = Widget_Button(bottomloadBase, Value='Done', XSize=82, uValue='DISMISS', $
    ToolTip='Dismiss Load Panel')

;  statusText = Widget_Text(statusBase, Value='Status information is displayed here ', $
;    UValue='STAT', XSize=87)
    
  ; Create Status Bar Object
  statusText = Obj_New('THM_UI_MESSAGE_BAR', $
                       Value='Status information is displayed here.', $
                        statusBase, XSize=145, YSize=1)


      ;main structure for this panel (to be filled in as items are needed)
  
  state = {tlb:tlb, gui_id:gui_id, itypeDroplist:itypeDroplist, observBase:observBase, $
           observ_label:observ_label, $
           observ_labels:observ_labels, observLabel:observLabel, observ:observ, $
           observList:observList, validobserv:validobserv, $
           validobservlist:ptr_new(validobservlist), validIType:validIType, $
           level1List:level1List, probe:ptr_new(probe0), probes:probes, $
           validProbes:validProbes, dataFlag:dataFlag, $
           station:ptr_new(station0), astation:ptr_new(astation0), $
           level2List:level2List, dlist1:dlist1, dlist2:dlist2, instr:instr, $
           dtyp10:dtyp10, dtyp20:dtyp20, dtyp1:dtyp1, dtyp2:dtyp2, dtype:dtype, $
           dtyp:dtyp, dtyp_pre:ptr_new(), $
           coordDroplist:coordDroplist, validCoords:ptr_new(validCoords), $
           outCoord:outCoord, tr:tr, startText:startText, stopText:stopText, $
           startt:startt, stopt:stopt, dataButtons:dataButtons, $
           ;addButton:addButton, minusButton:minusButton, loadlist:state_x.loadList, $
           addButton:addButton, minusButton:minusButton, loadlist:loadList, $
           loadedData:loadedData, historyWin:historyWin, $
           data_sel:ptr_new(), validData:ptr_new(val_data), $
           statusText:statusText, windowStorage:windowStorage}

  CenterTLB, tlb
  Widget_Control, tlb, Set_UValue = state, /No_Copy
  Widget_Control, tlb, /Realize
  XManager, 'thm_ui_load_data_file', tlb, /No_Block

  RETURN
END
