;+ 
;NAME:
; thm_ui_manage_data
;
;PURPOSE:
;  panel which allows user to import and export tplot data
;    
;CALLING SEQUENCE:
; thm_ui_manage_data,gui_id
; 
;INPUT:
; gui_id:  id of top level base widget from calling program
;
;OUTPUT:
; 
;HISTORY:
;$LastChangedBy: bckerr $
;$LastChangedDate: 2009-04-22 17:45:18 -0700 (Wed, 22 Apr 2009) $
;$LastChangedRevision: 5720 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/panels/thm_ui_manage_data.pro $
;
;--------------------------------------------------------------------------------

pro thm_ui_manage_data_event,event

  compile_opt hidden,idl2

  Widget_Control, event.TOP, Get_UValue=state, /No_Copy

  ;Put a catch here to insure that the state remains defined

  err_xxx = 0
  Catch, err_xxx
  IF (err_xxx NE 0) THEN BEGIN
    Catch, /Cancel
    Help, /Last_Message, Output = err_msg
    if is_struct(state) then begin
      FOR j = 0, N_Elements(err_msg)-1 DO state.historywin->update,err_msg[j]
      x=state.gui_id
      histobj=state.historywin
    endif
    Print, 'Error--See history'
    ok=error_message('An unknown error occured and the window must be restarted. See console for details.',$
       /noname, /center, title='Error in Manage Data')
    Widget_Control, event.TOP, Set_UValue=state, /No_Copy
    widget_control, event.top,/destroy
    if widget_valid(x) && obj_valid(histobj) then begin 
      thm_gui_error,x,histobj
    endif
    RETURN
  ENDIF
  
  IF(Tag_Names(event, /Structure_Name) EQ 'WIDGET_KILL_REQUEST') THEN BEGIN  
    Exit_Sequence:
    Print, 'Widget Killed' 
    Widget_Control, event.TOP, Set_UValue=state, /No_Copy    
    Widget_Control, event.top, /Destroy
    RETURN      
  ENDIF

  Widget_Control, event.id, Get_UValue=uval

  state.historywin->update,'THM_UI_MANAGE_DATA: User value: '+uval  ,/dontshow

  CASE uval OF
    'OK': BEGIN
      Widget_Control, event.TOP, Set_UValue=state, /No_Copy
      Widget_Control, event.top, /Destroy
      RETURN
    END
    'EDIT': BEGIN
      select = state.guiTree->getValue()
      if ptr_valid(select[0]) then begin
        for i = 0,n_elements(select)-1 do begin
          if keyword_set(names) then begin
            names = [names,(*select[i]).groupname]
          endif else begin
            names =  [(*select[i]).groupname]
          endelse
        endfor
         thm_ui_verify_data,state.tlb, names, state.loadedData, state.windowStorage, state.historywin, /edit,newnames=newnames,success=success
         
         if success then begin
           state.guiTree->update,selected=newnames
           state.statusBar->update,'Edit Completed'
           state.historyWin->update,'Edit Completed'
         endif else begin
           state.statusBar->update,'Edit Canceled'
           state.historyWin->update,'Edit Canceled'
         endelse
      ENDIF ELSE BEGIN
         state.statusBar->update,'GUI Data must be loaded and selected'
         state.historyWin->update,'Edit selected with no available data'
      ENDELSE
    END
    'TPLOT': BEGIN
      indices=widget_info(event.id, /list_select)
      if indices[0] eq -1 then begin  ;no valid selection
        state.tplotSelect = ptr_new()
      endif else begin  
        tplotNames = (*state.tplotNames)[indices]
        idx = where(tplotNames ne '',c)
        if c eq 0 then begin    ;no valid names
          state.tplotSelect = ptr_new()
        endif else begin
          tplotNames = tplotNames[idx]
          if Ptr_Valid(state.tplotSelect) THEN Ptr_Free, state.tplotSelect
          state.tplotSelect = Ptr_New(tplotNames)
        endelse
      endelse
    END
    'GUI_TREE':BEGIN
      ;do nothing
    END
    'RIGHT': BEGIN
      if ptr_valid(state.tplotselect) then begin 
        names = *state.tplotSelect
        numNames = N_Elements(names)
        existingData=state.loadedData->GetAll(/Parent)
        IF numNames GT 0 THEN BEGIN
           FOR i=0,numNames-1 DO BEGIN
              IF Is_Num(existingData) THEN index=-1 ELSE index = Where(existingData EQ names[i])
              IF index EQ -1 THEN BEGIN
                 result = state.loadedData->Add(names[i])
                 
                 if result eq 0 then begin
                  
                   state.statusBar->update,'Failed to add: ' + names[i] + '. Operation Terminated.' + $
                                           ' No tplot variables were imported.'
                   state.historyWin->update,'Failed to add: ' + names[i] + '. Operation Terminated.' + $
                                            ' No tplot variables were imported.'
                   
                   ;remove everything we were adding, since we cannot verify.
                   for j=0,i-1 do begin
                     tmp = state.loadedData->remove(names[j])
                   endfor
                   Widget_Control, event.TOP, Set_UValue=state, /No_Copy
                   return
                 
                 endif
                 
              ENDIF ELSE BEGIN
                messageString=String('The variable '+names[i]+' already exists in the GUI. Do you want to continue?')
                result=dialog_message(messageString,/QUESTION,/CENTER)
                IF result EQ 'No' THEN BEGIN
                   Widget_Control, event.TOP, Set_UValue=state, /No_Copy
                   RETURN
                ENDIF
                
                result = state.loadedData->Add(names[i])
                 
                if result eq 0 then begin
                 
                  state.statusBar->update,'Failed to add: ' + names[i] + '. Operation Terminated.' + $
                                          ' No tplot variables were imported.'
                  state.historyWin->update,'Failed to add: ' + names[i] + '. Operation Terminated.' + $
                                           ' No tplot variables were imported.'
                  ;remove everything we were adding, since we cannot verify.
                  for j=0,i-1 do begin
                    tmp = state.loadedData->remove(names[j])
                  endfor
                  Widget_Control, event.TOP, Set_UValue=state, /No_Copy
                  return
               
                endif
              ENDELSE
           ENDFOR        
           thm_ui_verify_data,state.tlb, names, state.loadedData, state.windowStorage, state.historywin,newnames=newnames,success=success
           
           if success then begin
             state.guiTree->update,selected=newnames
             state.statusBar->update,'Import Completed'
             state.historyWin->update,'Import Completed'
           endif else begin
             ;The verify panel requires data to have been added,
             ;so we just remove added quantities on failure
             ;The downside of this, is that if the operation was a replace
             ;operation, that data will disappear.
             tmp = state.loadedData->remove(newnames)
             state.guiTree->update
             state.statusBar->update,'Import canceled'
             state.historyWin->update,'Import canceled'
           endelse
        ENDIF 
      endif else begin 
        state.statusBar->update,'No Valid Selection'
        state.historyWin->update,'No valid selection for import'
      endelse
    END
    'LEFT': BEGIN
      select = state.guiTree->getValue()
      IF ptr_valid(select[0]) THEN BEGIN
         FOR i=0,n_elements(select)-1 DO BEGIN
            guiName = state.loadedData->GetTvarData((*select[i]).groupName)
         ENDFOR
;         tplot_names, names=names
	 names=tnames()        ;Does not have limit on length of name like TPLOT_NAMES.
         if ~keyword_set(names) then begin
           names = ['']
         endif
         if ptr_valid(state.tplotNames) then ptr_free, state.tplotNames
         state.tplotNames = ptr_new(names)
         widget_control,state.tplotList,set_value=names
         ptr_free,state.tplotSelect
         state.statusBar->update,'Exported Selection'
         state.historyWin->update,'Exported: ' + strjoin(names,' ')
      ENDIF Else begin
         state.statusBar->update,'GUI Data must be loaded and selected'
         state.historyWin->update,'No valid selection for export'
      endelse
    END
    'TRASH': BEGIN
      result=dialog_message('Are you sure you want to delete the selected data from the GUI?', $
                            /question,/center, title='Manage Data: Delete GUI data?')
      if result eq 'Yes' then begin
        select = state.guiTree->getValue()
        if ptr_valid(select[0]) then begin
          for i = 0,n_elements(select)-1 do begin
            if ~state.loadedData->remove((*select[i]).groupname) then begin
              state.statusBar->update,'Error deleting: ' + (*select[i]).groupname
              state.historyWin->update,'Error deleting: ' + (*select[i]).groupname
              return
            endif
          endfor
          heap_gc
          state.guiTree->update
          state.statusBar->update,'Deleted Selection'
          state.historyWin->update,'Deleted Selection'
        endif else begin
          state.statusBar->update,'GUI Data must be loaded and selected'
          state.historyWin->update,'No valid selection for delete'
        endelse
      endif

    END
    ELSE: state.historyWin->update,'Unimplemented UVAL in manage data'
  ENDCASE
    
  Widget_Control, event.top, Set_UValue=state, /No_Copy

  RETURN

END


pro thm_ui_manage_data, gui_id, loadedData, windowStorage, historywin

  compile_opt idl2
  
  xsize = 300
  ysize = 350
  
  err_xxx = 0
  Catch, err_xxx
  If(err_xxx Ne 0) Then Begin
    Catch, /Cancel
    Help, /Last_Message, output = err_msg
    FOR j = 0, N_Elements(err_msg)-1 DO historywin->update,err_msg[j]
    ;Print, 'Error--See history'
    ok = error_message('An unkown error occured starting Manage Data. See console for details.',$
        /noname, /center, title='Error in Manage Data')
    Widget_control,tlb,/destroy
    thm_gui_error,gui_id,historywin
    RETURN
  EndIf
  
  
  tlb = Widget_Base(/col, Title='Manage Data', Group_Leader=gui_id, /Modal, /Floating,/tlb_kill_request_events)
  
  ; Define Base structure for this window
  mainBase = Widget_Base(tlb, /row)
    tplotDataBase = Widget_Base(mainBase, /col, ypad=10)
    arrowBase = Widget_Base(mainBase, /col, ypad=115, xpad=12)
    guiDataBase = Widget_Base(mainBase, /col)
  buttonBase = widget_base(tlb,/row, /align_center)
  statusBase = widget_base(tlb,/row)
  
  ; Create the Widgets
  guiLabel = Widget_Label(guiDataBase, value='GUI Data: ',/align_left)
  guiTree = Obj_New('thm_ui_widget_tree',guiDataBase,'GUI_TREE',loadedData,xsize=xsize, $
                    ysize=ysize,mode=0,uname='GUI_TREE')       
  
  getresourcepath,rpath
  trashcan = read_bmp(rpath + 'trashcan.bmp',/rgb)
  leftArrow = read_bmp(rpath + 'arrow_180_medium.bmp',/rgb)
  rightArrow = read_bmp(rpath + 'arrow_000_medium.bmp',/rgb)
  zoomIn = read_bmp(rpath + 'magnifier_zoom.bmp',/rgb)
  
  thm_ui_match_background, tlb, trashcan
  thm_ui_match_background, tlb, leftArrow
  thm_ui_match_background, tlb, rightArrow
  thm_ui_match_background, tlb, zoomIn

  rightButton = Widget_Button(arrowBase, Value=rightArrow, /Bitmap, $
                Uvalue='RIGHT', ToolTip='Create a gui variable from tplot data', xsize=27, ysize=27)
  leftButton = Widget_Button(arrowBase, Value=leftArrow, /Bitmap,  UValue='LEFT', $
              ToolTip='Create a tplot variable from gui data', xsize=27, ysize=27)
  trashbutton = Widget_Button(arrowBase, Value=trashcan, /Bitmap,  UValue='TRASH', $
              ToolTip='Delete a Gui Variable', xsize=27, ysize=27) 
  edit_button = widget_button(arrowBase, value=zoomIn, uvalue='EDIT',/Bitmap, $
              Tooltip='Examine a Gui Variable', xsize=27, ysize=27)
  tplotLabel = Widget_Label(tplotDataBase, value='TPLOT Data: ', /align_left)
;  tplot_names, names=tplotListNames
  tplotListNames=tnames()        ;Does not have limit on length of name like TPLOT_NAMES.
  if ~keyword_set(tplotListNames) then begin
    tplotListNames = ['']
  endif
  tplotList = Widget_List(tplotDataBase, value=tplotListNames, xsize=42, $
              ysize=floor(ysize/(!D.Y_CH_SIZE+4)), uvalue='TPLOT', /multiple)
  ok_button = widget_button(buttonBase,value='OK',uvalue='OK', xsize=75)
  
  statusBar = Obj_New("THM_UI_MESSAGE_BAR", statusBase, Xsize=98, YSize=1)
 
  historyWin->update,'Opening manage data panel'
 
  state = {tlb:tlb, gui_id:gui_id, statusBar:statusBar, guiTree:guiTree, tplotSelect:Ptr_New(),$
           tplotList:tplotList,tplotNames:ptr_new(tplotListNames), loadedData:loadedData, $
           windowStorage:windowStorage, historywin:historywin}
            
  Widget_Control, tlb, Set_UValue = state, /No_Copy
  CenterTLB, tlb
  Widget_Control, tlb, /Realize
  
  XManager, 'thm_ui_manage_data', tlb, /No_Block
  
  historyWin->update,'Closing manage data panel'
  RETURN
  
end
