;+
;NAME:
; thm_ui_part_getspec_options
;PURPOSE:
; An options panel for inputs to thm_ui_part_getspec:
;CALLING SEQUENCE:
; thm_ui_part_getspec_options, dproc_id
;INPUT:
; dproc_id = the widget_id of the dproc panel
;OUTPUT:
; No explicit output, new variables are created. For a given data
; type, one or two spectra are created. One is the angular spectrum
; for the full energy range. If the energy spectrum option is chosen,
; then another spectrum is created which is the energy spectrum for
; the full angular range.
;HISTORY:
; 5-jan-2009, jmm, jimm@ssl.berkeley.edu
; 14-jan-2009, jmm, added statusbar object
; 15-jan-2009, jmm, added external_state, so that the active data
;                   widget on the dproc panel can be updated.
; 23-jan-2009, jmm, deletes tplot variables created during processing,
;                   correctly updates active data
;$LastChangedBy: bckerr $
;$LastChangedDate: 2009-04-17 17:36:31 -0700 (Fri, 17 Apr 2009) $
;$LastChangedRevision: 5691 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/panels/thm_ui_part_getspec_options.pro $
;-

; check angle inputs for correct ranges
function thm_ui_part_getspec_angcheck, state


        ; reset angle widgets to remove invalid inputs...has to be a better way
        id=widget_info(state.master, find_by_uname='ANGLE:PHI:MIN')
        widget_control, id, set_value=state.angle_values[0, 0]
        id=widget_info(state.master, find_by_uname='ANGLE:PHI:MAX')
        widget_control, id, set_value=state.angle_values[1, 0]
        id=widget_info(state.master, find_by_uname='ANGLE:THETA:MIN')
        widget_control, id, set_value=state.angle_values[0, 1]
        id=widget_info(state.master, find_by_uname='ANGLE:THETA:MAX')
        widget_control, id, set_value=state.angle_values[1, 1]        
        id=widget_info(state.master, find_by_uname='ANGLE:PA:MIN')
        widget_control, id, set_value=state.angle_values[0, 2]
        id=widget_info(state.master, find_by_uname='ANGLE:PA:MAX')
        widget_control, id, set_value=state.angle_values[1, 2]
        id=widget_info(state.master, find_by_uname='ANGLE:GYRO:MIN')
        widget_control, id, set_value=state.angle_values[0, 3]
        id=widget_info(state.master, find_by_uname='ANGLE:GYRO:MAX')
        widget_control, id, set_value=state.angle_values[1, 3]
        id=widget_info(state.master, find_by_uname='ANGLE:START')
        widget_control, id, set_value=state.start_angle_value
        id=widget_info(state.master, find_by_uname='ENERGY:MIN')
        widget_control, id, set_value=state.erange_values[0]
        id=widget_info(state.master, find_by_uname='ENERGY:MAX')
        widget_control, id, set_value=state.erange_values[1]
        id=widget_info(state.master, find_by_uname='REGRID:M')
        widget_control, id, set_value=state.regrid_values[0]
        id=widget_info(state.master, find_by_uname='REGRID:N')
        widget_control, id, set_value=state.regrid_values[1]        
        id=widget_info(state.master, find_by_uname='DATAGAP:DG')
        widget_control, id, set_value=state.datagap_value
        id=widget_info(state.master, find_by_uname='SST:MASKREMOVE')
        widget_control, id, set_value=state.mask_value
        id=widget_info(state.master, find_by_uname='LIM:VALUE')
        widget_control, id, set_value=state.limvalue


phi = float(state.angle_values[*, 0])
theta = float(state.angle_values[*, 1])
pitch = float(state.angle_values[*, 2])
gyro = float(state.angle_values[*, 3])
;phi = angle_values[*, 0]
;theta = angle_values[*, 1]
;pitch = angle_values[*, 2]
;gyro = angle_values[*, 3]

err_mess = ''

 if phi[0] gt phi[1] then $
   err_mess = 'Error with phi angle limits: Min must be less than Max.  '

 
 if (theta[1] lt theta[0]) OR (theta[0] lt -90) OR (theta[1] gt 90) then begin
 
   if theta[1] lt theta[0] then $
     err_mess = err_mess + 'Error with theta angle limits: Min must be less than Max.  '
   
   if theta[0] lt -90 then $
     err_mess = err_mess + 'Error with theta angle limits: Min must be greater than -90.  '
     
   if theta[1] gt 90 then $
     err_mess = err_mess + 'Error with theta angle limits: Max must be less than 90.  '
 endif


 if (pitch[1] lt pitch[0]) OR (pitch[0] lt 0) OR (pitch[1] gt 180) then begin

   if pitch[1] lt pitch[0] then $
     err_mess = err_mess + 'Error with pitch angle limits: Min must be less than Max.  '
   
   if pitch[0] lt 0 then $
     err_mess = err_mess + 'Error with pitch angle limits: Min must be greater than 0.  '
     
   if pitch[1] gt 180 then $
     err_mess = err_mess + 'Error with pitch angle limits: Max must be less than 180.  '
 endif


 if gyro[0] gt gyro[1] then $
   err_mess = err_mess + 'Error with gyro angle limits: Min must be less than Max.  '

return, err_mess
end


;event handler
Pro thm_ui_part_getspec_options_event, event
;Catch here to insure that the state remains defined
err_xxx = 0
catch, err_xxx
If(err_xxx Ne 0) Then Begin
    catch, /cancel
;Extra Error message
    txt = ['Error in THM_PART GETSPEC. It is possible that the proper data has not been loaded.', $
           'THM_PART_GETSPEC Operates on Level 1 SST and ESA data only. For each datatype, (e.g., ', $
           'peif, peef, psif, psef, etc...) all of the L1 data must be loaded. For ESA data, this ', $
           'is done automatically when a datatype is chosen from the Load THEMIS Data widget. For ', $
           'SST you need to click on all L1 variable choices for each given data type. If creating ',$
           'pitch angle or gyrovelocity spectra, try increasing the pitch/gyro angle ranges, or ', $
           'increase M and/or N. Otherwise, check to see if all of the appropriate data is loaded.']
    ok = error_message(txt, title='Error in Get Particle Spectra.', /center,traceback = 1)
    If(apply) Then Begin
        Goto, delete_tplot_vars ;back into the 'apply' block to delete tplot vars
    Endif Else Begin
        histobj=state.info.historywin
        x=state.dproc_id
        If(is_struct(state)) Then $
          widget_control, event.top, set_uvalue = state, /no_copy
        thm_gui_error,x,histobj
        Return                  ;get out
    Endelse
Endif

widget_control, event.top, get_uvalue = state, /no_copy
;Kill Request block
If(TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_KILL_REQUEST') Then Begin
    If(is_struct(state) && obj_valid(state.ext_statusbar)) Then $
      state.ext_statusbar -> update, 'Part_Getspec Options Panel Closed'
    If(is_struct(state)) Then widget_control, event.top, set_uvalue = state, /no_copy
    widget_control, event.top, /destroy
    Return
Endif
;Options
apply = 0b                      ;local flag for the 'apply' process.
widget_control, event.id, get_uvalue = uval
state.info.historyWin->Update,'THM_UI_PART_GETSPEC_OPTIONS: User value: '+uval,/dontshow
Case uval of
    'OK': Begin
        If(is_struct(state) && obj_valid(state.ext_statusbar)) Then $
          state.ext_statusbar -> update, 'Part_Getspec Options Panel Closed'
        If(is_struct(state)) Then widget_control, event.top, set_uvalue = state, /no_copy
        widget_control, event.top, /destroy
        Return
    End
    'SUFFIX': Begin
        widget_control, event.id, get_value = temp_string
;anything can be a suffix, as long as there are no spaces
        state.suffix = strcompress(/remove_all, temp_string)
        state.statusbar -> update, uval+' = '+state.suffix
    End
    'ANGLE:COMBOBOX': Begin
        temp = strsplit(widget_info(state.angle_combobox, /combobox_gettext),':',/extract)
        state.def_angle = strupcase(strcompress(/remove_all, temp[0]))
        state.statusbar -> update, 'ANGLE = '+state.def_angle
    End
    'ANGLE:PHI:MIN': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[0, 0] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[0, 0] + ' when you hit Apply.'
    End
    'ANGLE:PHI:MAX': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[1, 0] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[1, 0] + ' when you hit Apply.'
    End
    'ANGLE:THETA:MIN': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[0, 1] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[0, 1] + ' when you hit Apply.'
    End
    'ANGLE:THETA:MAX': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[1, 1] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[1, 1] + ' when you hit Apply.'
    End
    'ANGLE:PA:MIN': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[0, 2] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[0, 2] + ' when you hit Apply.'
    End
    'ANGLE:PA:MAX': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[1, 2] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[1, 2] + ' when you hit Apply.'
    End
    'ANGLE:GYRO:MIN': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[0, 3] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[0, 3] + ' when you hit Apply.'
    End
    'ANGLE:GYRO:MAX': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.angle_values[1, 3] = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.angle_values[1, 3] + ' when you hit Apply.'
    End
    'ANGLE:START': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            state.start_angle_value = temp_string
            state.statusbar -> update, uval+' = '+temp_string
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
                   '.  Will be reset to ' + state.start_angle_value + ' when you hit Apply.'
    End
    'ENERGY:MIN': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Ge 0) Then Begin
                state.erange_values[0] = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.erange_values[0] + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.erange_values[0] + ' when you hit Apply.'
    End
    'ENERGY:MAX': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Ge 0) Then Begin
                state.erange_values[1] = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.erange_values[1] + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.erange_values[1] + ' when you hit Apply.'
    End
    'ENERGY:SET': Begin
        state.energy_set = event.select
        If(event.select Eq 1) Then state.statusbar -> update, 'Return Energy Spectrum Option Set' $
        Else state.statusbar -> update, 'Return Energy Spectrum Option Not Set'
    End
    'TIME:MIN': Begin
        widget_control, event.id, get_value = temp_string
        test = thm_ui_timefix(temp_string)
        If(is_string(test)) Then Begin
            state.trange_values[0] = time_string(test)
            state.statusbar -> update, uval+' = '+state.trange_values[0]
        Endif Else state.statusbar -> update, uval+' = '+temp_string+' is invalid'
    End
    'TIME:MAX': Begin
        widget_control, event.id, get_value = temp_string
        test = thm_ui_timefix(temp_string)
        If(is_string(test)) Then Begin
            state.trange_values[1] = time_string(test)
            state.statusbar -> update, uval+' = '+state.trange_values[1]
        Endif Else state.statusbar -> update, uval+' = '+temp_string+' is invalid'
    End
    'NORM:SET': Begin
        state.normalize = event.select
        If(event.select Eq 1) Then state.statusbar -> update, 'Normalize Option Set' $
        Else state.statusbar -> update, 'Normalize Option Not Set'
    End
    'REGRID:M': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Gt 0) Then Begin
                state.regrid_values[0] = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.regrid_values[0] + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.regrid_values[0] + ' when you hit Apply.'
    End
    'REGRID:N': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Gt 0) Then Begin
                state.regrid_values[1] = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.regrid_values[1] + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.regrid_values[1] + ' when you hit Apply.'
    End
    'OD:COMBOBOX': Begin
        state.def_other_dim = widget_info(state.od_combobox, /combobox_gettext)
        state.statusbar -> update, 'OTHER_DIM = '+state.def_other_dim
    End
    'DATAGAP:DG': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Ge 0) Then Begin
                state.datagap_value = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.datagap_value + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.datagap_value + ' when you hit Apply.'
    End
    'SST:MASKSET': Begin
        state.remove_mask = event.select
        If(event.select Eq 1) Then state.statusbar -> update, 'Remove Mask Option Set' $
        Else state.statusbar -> update, 'Remove Mask Option Not Set'
    End
    'SST:MASKREMOVE': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Ge 0 And test_value Le 1.0) Then Begin
                state.mask_value = temp_string
                state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
               '.  Will be reset to ' + state.mask_value + ' when you hit Apply.'
        Endif Else state.statusbar -> update, uval+' Invalid Input: '+temp_string+ $
           '.  Will be reset to ' + state.mask_value + ' when you hit Apply.'
    End
    'SP:SET': Begin
        state.spvalue = event.select
        If(event.select Eq 1) Then state.statusbar -> update, 'SunPulse Removal Option Set' $
        Else state.statusbar -> update, 'SunPulse Removal Option Not Set'
    End
    'SP:COMBOBOX': Begin
        state.def_sp = widget_info(state.sp_combobox, /combobox_gettext)
        state.statusbar -> update, 'SUNPULSE REMOVAL METHOD= '+state.def_sp
    End
    'LIM:VALUE': Begin
        widget_control, event.id, get_value = temp_string
        If(is_numeric(temp_string)) Then Begin
            test_value = float(temp_string)
            If(test_value Ge 0) Then begin 
              state.limvalue = temp_string
              state.statusbar -> update, uval+' = '+temp_string
            Endif Else state.statusbar -> update, 'Invalid Input'+ $
               '.  Will be reset to ' + state.limvalue + ' when you hit Apply.'
        Endif Else state.statusbar -> update, 'Invalid Input'+ $
           '.  Will be reset to ' + state.limvalue + ' when you hit Apply.'
    End
    'FILL:COMBOBOX': Begin
        state.def_fill = widget_info(state.fill_combobox, /combobox_gettext)
        state.statusbar -> update, 'FILLIN_METHOD = '+state.def_fill
    End
    'APPLY':Begin

        apply = 1b              ;this is here to be checked if the catch is triggered

;Check angle inputs for any errors
        ang_err_mess = thm_ui_part_getspec_angcheck(state)
        if ~array_equal(ang_err_mess, '') then begin
          state.statusbar->Update, ang_err_mess
          state.info.historyWin->Update, ang_err_mess
          widget_control, event.top, set_uvalue = state, /no_copy
          return
        endif
;Check start and end times are not in the future
        trange = time_double(state.trange_values)
        if (trange[0] gt systime(/sec)) AND (trange[1] gt systime(/sec)) then begin
          state.statusbar->Update, "Error: Start and end times are later than today's date. "
          widget_control, event.top, set_uvalue = state, /no_copy
          return
        endif
;Find active data, get datatypes, run thm_part_getspec separately for
;each variable
        x = state.info.loadeddata
        h = state.info.historywin
        active_v = x -> getactive(/parent)
        If(is_string(active_v) Eq 0) Then Begin
            state.statusbar -> update, 'No Active Data Set, No Action Taken'
            widget_control, event.top, set_uvalue = state, /no_copy
            Return
        Endif
;thm_part_getspec runs only on SST and ESA data
        sst_vars = where(strmatch(active_v, 'th?_ps*') Eq 1, nsst)
        esa_vars = where(strmatch(active_v, 'th?_pe*') Eq 1, nesa)
        vars = ''
        If(nsst Gt 0) Then vars = [vars, active_v[sst_vars]]
        If(nesa Gt 0) Then vars = [vars, active_v[esa_vars]]
        If(n_elements(vars) Eq 1) Then Begin
            state.statusbar -> update, 'No Active ESA or SST Data Set'
            widget_control, event.top, set_uvalue = state, /no_copy
            Return
        Endif Else vars = vars[1:*]
;for easier variable tracking, run thm_part_getspec in a loop for each
;unique probe_datatype combination
        sv = strmid(vars, 0, 8)
        usv = uniq(sv)
        sv = sv[usv]
;define these variables outside of the loop, for easy testing, and
;record all choices in the history window.
        history_arr = ['INPUT PARAMETERS FOR PART_GETSPEC:', $
                       'TIME RANGE: '+state.trange_values[0]+', '+state.trange_values[1], $
                       'START ANGLE: '+state.start_angle_value, $
                       'SUFFIX: '+state.suffix, 'ANGLE: '+state.def_angle, $
                       'ENERGY RANGE: '+state.erange_values[0]+', '+state.erange_values[1], $
                       'ENERGY SPECTROGRAM OPTION: '+strcompress(string(state.energy_set)), $
                       'REGRID VALUES: '+state.regrid_values[0]+', '+state.regrid_values[1], $
                       'OTHER_DIM: '+state.def_other_dim, $
                       'NORMALIZE OPTION: '+strcompress(string(state.normalize)), $
                       'DATAGAP VALUE: '+state.datagap_value, $
                       'SST MASK REMOVAL OPTION: '+strcompress(string(state.remove_mask)), $
                       'SST MASK VALUE: '+state.mask_value, $
                       'SST SUNPULSE REMOVAL OPTION: '+strcompress(string(state.spvalue)), $
                       'SST SUNPULSE REMOVAL METHOD: '+state.def_sp, $
                       'SST SUNPULSE REMOVAL FILLIN METHOD: '+state.def_fill, $
                       'SST SUNPULSE REMOVAL TOLERANCE: '+state.limvalue]
        For j = 0, 3 Do history_arr = [history_arr, state.angle_options[j]+' LIMITS:'+$
                                       state.angle_values[0, j]+', '+state.angle_values[1, j]]

        ; reset angle widgets to remove invalid inputs...has to be a better way
        id=widget_info(state.master, find_by_uname='ANGLE:PHI:MIN')
        widget_control, id, set_value=state.angle_values[0, 0]
        id=widget_info(state.master, find_by_uname='ANGLE:PHI:MAX')
        widget_control, id, set_value=state.angle_values[1, 0]
        id=widget_info(state.master, find_by_uname='ANGLE:THETA:MIN')
        widget_control, id, set_value=state.angle_values[0, 1]
        id=widget_info(state.master, find_by_uname='ANGLE:THETA:MAX')
        widget_control, id, set_value=state.angle_values[1, 1]        
        id=widget_info(state.master, find_by_uname='ANGLE:PA:MIN')
        widget_control, id, set_value=state.angle_values[0, 2]
        id=widget_info(state.master, find_by_uname='ANGLE:PA:MAX')
        widget_control, id, set_value=state.angle_values[1, 2]
        id=widget_info(state.master, find_by_uname='ANGLE:GYRO:MIN')
        widget_control, id, set_value=state.angle_values[0, 3]
        id=widget_info(state.master, find_by_uname='ANGLE:GYRO:MAX')
        widget_control, id, set_value=state.angle_values[1, 3]
        id=widget_info(state.master, find_by_uname='ANGLE:START')
        widget_control, id, set_value=state.start_angle_value
        id=widget_info(state.master, find_by_uname='ENERGY:MIN')
        widget_control, id, set_value=state.erange_values[0]
        id=widget_info(state.master, find_by_uname='ENERGY:MAX')
        widget_control, id, set_value=state.erange_values[1]
        id=widget_info(state.master, find_by_uname='REGRID:M')
        widget_control, id, set_value=state.regrid_values[0]
        id=widget_info(state.master, find_by_uname='REGRID:N')
        widget_control, id, set_value=state.regrid_values[1]        
        id=widget_info(state.master, find_by_uname='DATAGAP:DG')
        widget_control, id, set_value=state.datagap_value
        id=widget_info(state.master, find_by_uname='SST:MASKREMOVE')
        widget_control, id, set_value=state.mask_value
        id=widget_info(state.master, find_by_uname='LIM:VALUE')
        widget_control, id, set_value=state.limvalue


        h -> update, history_arr
        trange = time_double(state.trange_values)
        start_angle = float(state.start_angle_value)
        suffix = state.suffix
        angle = strlowcase(state.def_angle)
        phi = float(state.angle_values[*, 0])
        theta = float(state.angle_values[*, 1])
        pitch = float(state.angle_values[*, 2])
        gyro = float(state.angle_values[*, 3])
        erange = float(state.erange_values)
        energy = state.energy_set
        regrid = long(state.regrid_values)
        other_dim = strlowcase(state.def_other_dim)
        normalize = state.normalize
        datagap = float(state.datagap_value)
;Keep track of new created variables
        new_vars = ''
;Keep track of tplot data here
        tnames_in = tnames()
        For j = 0, n_elements(sv)-1 Do Begin
;thm_part_getspec accepts 'probe' and 'data_type' keywords,
;assuming that there aren't non-standard variable names
            probe = strmid(sv[j], 2, 1)
            dtype = strmid(sv[j], 4, 4)
            h -> update, 'THM_PART_GETSPEC: Processing Probe: '+probe+',  Datatype: '+dtype
            state.statusbar -> update, 'Processing Probe: '+probe+',  Datatype: '+dtype
;SST masking is an issue
            If(strmid(dtype, 1, 1) Eq 's') Then Begin
                If(state.remove_mask Eq 0) Then mask_remove = 0 $
                Else mask_remove = float(state.mask_value)
;Sun pulse cleanup?
                If(state.spvalue Eq 0) Then Begin
                    method_sunpulse_clean = 0
                    limit_sunpulse_clean = 0
                Endif Else Begin
                    method_sunpulse_clean = strlowcase(state.def_sp)
                    limit_sunpulse_clean = float(state.limvalue)
                Endelse
                fillin_method = strlowcase(state.def_fill)
            Endif Else Begin
                mask_remove = 0
                method_sunpulse_clean = 0
                limit_sunpulse_clean = 0
                fillin_method = 0
            Endelse
            thm_part_getspec, probe = probe, data_type = dtype, trange = trange, $
              start_angle = start_angle, suffix = suffix, angle = angle, $
              phi = phi, theta = theta, pitch = pitch, gyro = gyro, $
              erange = erange, energy = energy, regrid = regrid, $
              other_dim = other_dim, normalize = normalize, datagap = datagap, $
              mask_remove = mask_remove, method_sunpulse_clean = method_sunpulse_clean, $
              limit_sunpulse_clean = limit_sunpulse_clean, fillin_method = fillin_method, $
              /no_dataload
;Now find the new variables
            new_spec = tnames(sv[j]+'_an_eflux'+'_'+angle+suffix)
            If(is_string(new_spec)) Then Begin
;copy a tvar object for this datatype
                tvo = x -> gettvarobject(vars[usv[j]])
                new_tvo = tvo -> copy()
                new_tvo -> setproperty, name = new_spec
                dummy = x -> addtvarobject(new_tvo, added_name = new_spec)
                If(dummy) Then Begin
                    state.statusbar -> update, 'Added Variable: '+new_spec
                    h -> update, 'Added Variable: '+new_spec
                    new_vars = [new_vars, new_spec]
                Endif Else Begin
                    state.statusbar -> update, 'Failed to Add Variable: '+new_spec+'  What''s the deal with that?'
                    h -> update, 'Failed to Add Variable: '+new_spec+'  What''s the deal with that?'
                Endelse
            Endif
            If(energy Gt 0) Then Begin
                new_espec = tnames(sv[j]+'_en_eflux'+suffix)
                If(is_string(new_espec)) Then Begin
;copy a tvar object for this datatype
                    tvo = x -> gettvarobject(vars[usv[j]])
                    new_tvo = tvo -> copy()
                    new_tvo -> setproperty, name = new_espec
                    dummy = x -> addtvarobject(new_tvo, added_name = new_espec)
                    If(dummy) Then Begin
                        state.statusbar -> update, 'Added Variable: '+new_espec
                        h -> update, 'Added Variable: '+new_espec
                        new_vars = [new_vars, new_espec]
                    Endif Else Begin
                        state.statusbar -> update, 'Failed to Add Variable: '+new_espec+'  What''s the deal with that?'
                        h -> update, 'Failed to Add Variable: '+new_espec+'  What''s the deal with that?'
                    Endelse
                Endif
            Endif
        Endfor
;If there are any new variables, set them to 'active'
        If(n_elements(new_vars) Gt 1) Then Begin
            new_vars = new_vars[1:*]
            x -> clearallactive
            For j = 0, n_elements(new_vars)-1 Do x -> setactive, new_vars[j]
;If possible, then update the active data list
            thm_ui_dproc_reset_act_data, state.dproc_id
        Endif Else Begin
            state.statusbar -> update, 'No new particle spectrum variables were created. Check Time, Angle, and Energy Ranges. '
            h -> update, 'No new particle spectrum variables were created. Check Time, Angle, and Energy Ranges. '
        Endelse
;delete any new tplot variables created here
        delete_tplot_vars:      ;this label has a goto from the catch
        tnames_out = tnames()
        If(is_string(tnames_out)) Then Begin
            If(is_string(tnames_in)) Then Begin
                For j = 0, n_elements(tnames_out)-1 Do Begin
                    xx = where(tnames_in Eq tnames_out[j], nxx)
                    If(nxx Eq 0) Then store_data, tnames_out[j], /delete
                Endfor
            Endif Else del_data, '*' ;no variables when we started, so delete all
        Endif
        ;update the data tree
        state.data_tree->update
;All done
    End
Endcase

widget_control, event.top, set_uvalue = state, /no_copy
Return
End


;widget set up
Pro thm_ui_part_getspec_options, dproc_id,data_tree

widget_control, /hourglass

If(widget_valid(dproc_id)) Then Begin
    widget_control, dproc_id, get_uval = dproc_state, /no_copy
    ext_statusbar = dproc_state.statusbar
    info = dproc_state.info
    widget_control, dproc_id, set_uval = dproc_state, /no_copy
    group_leader = dproc_id
Endif Else Begin
    group_leader = 0b
    info = 0
    ext_statusbar = -1
Endelse

master = widget_base(/col, title = 'THEMIS: Get Particle Spectra ', $
                     group_leader = group_leader, /floating, /tlb_kill_request_events)
;Warning
txtbase = widget_base(master, /col, /align_left)
txt = ['Welcome to the world of THM_PART GETSPEC. ', $
       'THM_PART_GETSPEC Operates on Level 1 SST and ESA data only.   !!WARNING!!   IT DOES NOT OPERATE ON THE VARIABLE(S) IN THE ACTIVE', $
       'DATA WINDOW. THE ACTIVE DATA VARIABLES ARE ONLY USED TO TELL THM_PART_GETSPEC WHICH PROBE(S) AND DATATYPE(S) SHOULD', $
       'BE USED TO CALCULATE ANGLE/ENERGY SPECTRA. For each datatype, (e.g., peif, peef, psif, psef, etc...) all of the L1 data must be loaded. ', $
       'For ESA data, this is done automatically when a datatype is chosen from the Load THEMIS Data widget. For SST you need to click on all L1 variable choices ', $
       'for each given data type when loading. Be sure that all of the appropriate data is loaded. For pitch angle and gyrovelocity distributions, FGS and STATE data ', $
       'for the appropriate probes are also necessary. These will be automatically loaded if they have not been loaded already.']
For j = 0, n_elements(txt)-1 Do wj = widget_label(txtbase, value = txt[j], /align_left)

m0 = widget_base(master, /row)
m1 = widget_base(m0, /col)
m2 = widget_base(m0, /col)
;sub-structure
suffix_base = widget_base(m1, /row, /frame)
angle_base = widget_base(m1, /col, /frame) ;base widget for angle info
m3 = widget_base(m1, /row)
energy_base = widget_base(m3, /col, /frame, /align_center) ;base widget for energy info
time_base = widget_base(m3, /col, /frame, /align_center) ;base widget for time range
advanced_base = widget_base(m2, /col, /frame) ;base widget for other stuff
buttonbase = widget_base(master, /row)

;suffix
suffixlabel = widget_label(suffix_base, value = 'Suffix for new variable names')
suffix = '_part_spec'
suffixid = widget_text(suffix_base, value = suffix, xsiz = 22, ysiz = 1, $
                       uvalue = 'SUFFIX', /editable, /all_events)

;Angle stuff
alabel = widget_label(angle_base, value = 'Choose Angular distribution ')
angle_options = ['PHI', 'THETA', 'PA', 'GYRO']
def_angle = 'PHI'
angle_description = ['degrees relative to probe-sun direction in spin plane', $
                     'degrees relative to spin plane/azimuth for pitch angle', $
                     'pitch angle, degrees relative to magnetic field', $
                     'gyrovelocity']        ;need to figure out gyrovelocity
angle_combobox = widget_combobox(angle_base, value = angle_options+': '+$
                                 angle_description, uvalue = 'ANGLE:COMBOBOX')

nangles = n_elements(angle_options)
angle_values = strarr(2, nangles)
angle_values[*, 0] = ['0', '360']
angle_values[*, 1] = ['-90', '90']
angle_values[*, 2] = ['0', '180']
angle_values[*, 3] = ['0', '360']

angle_values_base = widget_base(angle_base, /col)
angle_values_label = widget_label(angle_values_base, value = 'Angular Limits')
For j = 0, nangles-1 Do Begin
    uvalj = 'ANGLE:'+angle_options[j]
    basej = widget_base(angle_values_base, /row)
    basejlabel = widget_label(basej, value = angle_options[j], xsiz = 120)
    basejminbase = widget_base(basej, /row)
    basejminlabel = widget_label(basejminbase, value = 'Min')
    basejmin = widget_text(basejminbase, value = angle_values[0, j], xsiz = 6, ysiz = 1, $
                           uvalue = uvalj+':MIN', uname = uvalj+':MIN', /editable, /all_events)
    basejmaxbase = widget_base(basej, /row)
    basejmaxlabel = widget_label(basejmaxbase, value = 'Max')
    basejmax = widget_text(basejmaxbase, value = angle_values[1, j], xsiz = 6, ysiz = 1, $
                           uvalue = uvalj+':MAX', uname = uvalj+':MAX', /editable, /all_events)
Endfor

start_angle_value = '0'
start_angle_base = widget_base(angle_values_base, /row)
start_angle_label = widget_label(start_angle_base, value = 'Start Angle (Phi only)', xsiz = 150)
start_angle_id = widget_text(start_angle_base, value = start_angle_value, xsiz = 6, ysiz = 1, $
                             uvalue = 'ANGLE:START', uname='ANGLE:START', /editable, /all_events)

;Buttons for energy spectra
erange_values = ['0', '1e7']
erbase = widget_base(energy_base, /col)
erlabel = widget_label(erbase, value = 'Energy Range (eV)')
erminmaxbase = widget_base(erbase, /row)
erminbase = widget_base(erminmaxbase, /row)
erminlabel = widget_label(erminbase, value = 'Min ')
ermin = widget_text(erminbase, value = erange_values[0], xsiz = 6, ysiz = 1, $
                    uvalue = 'ENERGY:MIN', uname='ENERGY:MIN', /editable, /all_events)
ermaxbase = widget_base(erminmaxbase, /row)
ermaxlabel = widget_label(ermaxbase, value = 'Max ')
ermax = widget_text(ermaxbase, value = erange_values[1], xsiz = 6, ysiz = 1, $
                    uvalue = 'ENERGY:MAX', uname='ENERGY:MAX', /editable, /all_events)
energy_set = 0b
ebuttonbase = widget_base(energy_base, /col, /nonexclusive)
ebutton = widget_button(ebuttonbase, value = 'Return Energy Spectrum', uvalue = 'ENERGY:SET', $
                        tooltip='Output tplot variables containing the energy spectrum of ' + $ 
                        'the energy/angle ranges input above')

;time range selection
@tplot_com.pro
str_element,tplot_vars,'options.trange_full',trange_full
if (n_elements(trange_full) ne 2) OR (tplot_vars.options.trange_full[0] ge $
    tplot_vars.options.trange_full[1]) then begin
  etxt = 'SST and/or ESA data needs to be loaded before using Get Particle Spectra.'
  ok = dialog_message(etxt,title='No Data Loaded', /center, information=1)
  return
endif else trange_values = time_string(timerange(/current))

trangelabel = widget_label(time_base, value = 'Time Range ')
trminbase = widget_base(time_base, /row)
trminlabel = widget_label(trminbase, value = 'Start Time ')
trmin = widget_text(trminbase, value = trange_values[0], xsiz = 26, ysiz = 1, $
                    uvalue = 'TIME:MIN', /editable, /all_events)
trmaxbase = widget_base(time_base, /row)
trmaxlabel = widget_label(trmaxbase, value = ' End Time ')
trmax = widget_text(trmaxbase, value = trange_values[1], xsiz = 26, ysiz = 1, $
                    uvalue = 'TIME:MAX', /editable, /all_events)
;advanced stuff
normbase = widget_base(advanced_base, /col, /nonexclusive)
normalize = 0b
nbutton = widget_button(normbase, value = 'Normalize Spectrum', uvalue = 'NORM:SET', $
                        tooltip='Normalize the flux for each time sample to 0-1')

rgbase = widget_base(advanced_base, /row)
rglabelbase = widget_base(rgbase, /col)
rglabel1 = widget_label(rglabelbase, /align_left, value = 'Regrid = [m,n] for [phi, theta] for pitch and gyro')
rglabel2 = widget_label(rglabelbase, /align_left, value = 'Use powers of 2, or m divisible into 360, n divisible into 180')
regrid_values = ['16', '8']
rgmnbase = widget_base(rgbase, /row)
rgmbase = widget_base(rgmnbase, /row)
rgmlabel = widget_label(rgmbase, value = 'M ')
rgm = widget_text(rgmbase, value = regrid_values[0], xsiz = 6, ysiz = 1, $
                  uvalue = 'REGRID:M', uname='REGRID:M', /editable, /all_events)
rgnbase = widget_base(rgmnbase, /row)
rgnlabel = widget_label(rgnbase, value = 'N ')
rgn = widget_text(rgnbase, value = regrid_values[1], xsiz = 6, ysiz = 1, $
                  uvalue = 'REGRID:N', uname='REGRID:N', /editable, /all_events)
odbase = widget_base(advanced_base, /row)
odlabelbase = widget_base(odbase, /col)
odlabel1 = widget_label(odlabelbase, /align_left, value = 'Other_dim = keyword for the second axis for conversion to FAC')
od_options = ['MPHIGEO',  'PHIGEO', 'MPHISM', 'PHISM', 'MRGEO', 'RGEO', 'XGSE', 'YGSM']
def_other_dim = 'MPHIGEO'
od_combobox = widget_combobox(odbase, value = od_options, uvalue = 'OD:COMBOBOX')

dgbase = widget_base(advanced_base, /row)
dglabelbase = widget_base(dgbase, /col)
dglabel1 = widget_label(dglabelbase, /align_left, value = 'Maximum gap in seconds over which to interpolate plot ')
datagap_value = '0'
dgvaluebase = widget_base(dgbase, /row)
dgvaluelabel = widget_label(dgvaluebase, value = 'GAP')
dg = widget_text(dgvaluebase, xsiz = 8, ysiz = 1, value = datagap_value, $
                 uvalue = 'DATAGAP:DG', uname = 'DATAGAP:DG', /editable, /all_events)

;SST stuff
maskbase = widget_base(advanced_base, /col)
maskbuttonbase = widget_base(maskbase, /col, /nonexclusive)
remove_mask = 0b
maskbutton = widget_button(maskbuttonbase, value = 'Remove SST mask', uvalue = 'SST:MASKSET', $
                           tooltip='Remove on-board SST mask')
maskvaluebase = widget_base(maskbase, /row)
maskvaluelabelbase = widget_base(maskvaluebase, /col)
maskvaluelabel1 = widget_label(maskvaluelabelbase, /align_left, value = 'Mask_remove (proportion of values that must be zero')
maskvaluelabel2 = widget_label(maskvaluelabelbase, /align_left, value = 'at all energies to determine that a mask is present)')
mask_value = '0.99'
maskremove = widget_text(maskvaluebase, value = mask_value, xsiz = 6, ysiz = 1, $
                         uvalue = 'SST:MASKREMOVE', uname = 'SST:MASKREMOVE', /editable, $
                         /all_events)

spbase = widget_base(advanced_base, /col)
spbuttonbase = widget_base(spbase, /col, /nonexclusive)
spbutton = widget_button(spbuttonbase, value = 'Remove SST Sun Contamination', uvalue = 'SP:SET', $
                         tooltip='Removes sunpulse contamination of SST data')
spvalue = 0b
spbase1 = widget_base(spbase, /row)
splabelbase = widget_base(spbase1, /col)
splabel1 = widget_label(splabelbase, value = 'Sunpulse cleaning method for SST sun contamination removal')
sp_options = ['SPIN_FIT', 'MEDIAN']
def_sp = 'SPIN_FIT'
sp_combobox = widget_combobox(spbase1, value = sp_options, uvalue = 'SP:COMBOBOX')
limbase = widget_base(spbase, /row)
limlabelbase = widget_base(limbase, /col)
limlabel1 = widget_label(limlabelbase, value = 'The standard deviation tolerance used by sunpulse cleaning')
limvalue = '2.0'
limtxt = widget_text(limbase, value = limvalue, xsiz = 6, ysiz = 1, $
                     uvalue = 'LIM:VALUE', uname = 'LIM:VALUE', /editable, /all_events)
fillbase = widget_base(spbase, /row)
filllabelbase = widget_base(fillbase, /col)
filllabel1 = widget_label(filllabelbase, value = 'Fillin method, to fill points that are removed')
def_fill = 'INTERPOLATION'
fill_options = ['INTERPOLATION', 'SPIN_FIT']
fill_combobox = widget_combobox(fillbase, value = fill_options, uvalue = 'FILL:COMBOBOX')
apply_button = Widget_Button(buttonBase, Value = '    Apply    ', XSize = 85, $
                             UValue = 'APPLY', tooltip='Create particle spectra from active data')
ok_button = Widget_Button(buttonBase, Value = '   Done    ', XSize = 85, UValue = 'OK', $
                          tooltip='Close Particle Spectra Options')
; Create Status Bar Object
statusBar = Obj_New('THM_UI_MESSAGE_BAR', Value = 'Status information is displayed here.', $
                    buttonbase, Xsize=112, YSize=1)

state = {master:master, info:info, dproc_id:dproc_id, angle_combobox:angle_combobox, $
         def_angle:def_angle, angle_options:angle_options, angle_values:angle_values, $
         energy_set:energy_set, erange_values:erange_values, start_angle_value:start_angle_value, $
         trange_values:trange_values, regrid_values:regrid_values, od_combobox:od_combobox, $
         def_other_dim:def_other_dim, normalize:normalize, datagap_value:datagap_value, $
         remove_mask:remove_mask, mask_value:mask_value, spvalue:spvalue, sp_combobox:sp_combobox, $
         def_sp:def_sp, limvalue:limvalue, fill_combobox:fill_combobox, def_fill:def_fill, $
         suffix:suffix, statusbar:statusbar, ext_statusbar:ext_statusbar,data_tree:data_tree}
widget_control, master, set_uvalue = state, /no_copy
widget_control, master, /realize
xmanager, 'thm_ui_part_getspec_options', master, /no_block

Return
End
