;+ 
;NAME: 
; thm_ui_highlight_settings__define
;
;PURPOSE:  
; highlight properties is an object that holds all parameters associated with
; theh Trace Properties Highlight panel
;
;CALLING SEQUENCE:
; highlightProperties = Obj_New("THM_UI_HIGHLIGHT_SETTINGS")
;
;INPUT:
; none
;
;KEYWORDS:
; markVertical      flag to mark vertical
; lineStyle         line style object
; symbol            symbol object 
; placement         droplistfor placement of symbol
;                     top, bottom, both top and bottom
; labelWith         0=No label, 1=X value, 2=Y value
; format            numeric format for label 
; markWhenY         flag to mark y value 
; whenYEquals       y value to mark
; markEvery         flag to mark every y value ..
; whenEveryEquals  y value to mark
; setBackground     flag to set background
; backgroundWhenY   droplist of logical operators
; backgroundYValue  value for logical operator
; backgroundColor   r,g,b color for background
;
;OUTPUT:
; highlight property object reference
;
;METHODS:
; SetProperty   procedure to set keywords 
; GetProperty   procedure to get keywords 
; GetAll        returns the entire structure
; GetPlacements returns a string array with placement options
; GetPlacement  return a string with placement option of given index
; GetOperators  returns a string array with operator options ['<','<=', ...]
; GetOperator   return a string with placement option of given index
;
;HISTORY:
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2008-12-17 14:25:17 -0800 (Wed, 17 Dec 2008) $
;$LastChangedRevision: 4233 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/templates/thm_ui_highlight_settings__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::Copy
   out = Obj_New('THM_UI_HIGHLIGHT_SETTINGS')
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out
   newLineStyle=Obj_New("THM_UI_LINE_STYLE")
   IF Obj_Valid(self.lineStyle) THEN newLineStyle=self.lineStyle->Copy() ELSE $
      newLineStyle=Obj_New()
   out->SetProperty, LineStyle=newLineStyle
   newSymbol=Obj_New("THM_UI_SYMBOL")
   IF Obj_Valid(self.symbol) THEN newSymbol=self.symbol->Copy() ELSE $
      newSymbol=Obj_New()
   out->SetProperty, Symbol=newSymbol
   RETURN, out
END ;--------------------------------------------------------------------------------



PRO THM_UI_HIGHLIGHT_SETTINGS::SetProperty,   $
              MarkVertical=markvertical,        $ ; flag to mark with vertical
              LineStyle=linestyle,              $ ; line style object
              Symbol=symbol,                    $ ; symbol object 
              Placement=placement,              $ ; droplist value of placement location
              LabelWith=labelwith,              $ ; 0=No label, 1=X value, 2=Y value
              Format=format,                    $ ; format for numeric label 
              MarkWhenY=markwheny,              $ ; flag to mark values when y  
              WhenYEquals=whenyequals,          $ ; y value to mark
              MarkEvery=markevery,              $ ; mark every value
              WhenEveryEquals=wheneveryequals,  $ ; y value to mark
              SetBackground=setbackground,      $ ; flag to set background
              BackgroundWhenY=backgroundwheny,  $ ; droplist of when y is (<, >, <=, >=)
              BackgroundYValue=backgroundyvalue,$ ; y value 
              BackgroundColor=backgroundcolor     ; rgb background color

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(linestyle) NE 0 THEN self.lineStyle = linestyle 
   IF N_Elements(symbol) NE 0 THEN self.symbol = symbol 
   IF N_Elements(placement) NE 0 THEN self.placement = placement 
   IF N_Elements(labelwith) NE 0 THEN self.labelWith = labelwith 
   IF N_Elements(format) NE 0 THEN self.format = format 
   IF N_Elements(whenyequals) NE 0 THEN self.whenYEquals = whenyequals 
   IF N_Elements(wheneveryequals) NE 0 THEN self.whenEveryEquals = wheneveryequals 
   IF N_Elements(backgroundwheny) NE 0 THEN self.backgroundWhenY = backgroundwheny 
   IF N_Elements(backgroundyvalue) NE 0 THEN self.backgroundYValue = backgroundyvalue 
   IF N_Elements(backgroundcolor) NE 0 THEN self.backgroundColor = backgroundcolor
   if n_elements(markVertical) ne 0 then self.markervertical = markvertical
   if n_elements(markwheny) ne 0 then self.markwheny = markwheny
   if n_elements(markevery) ne 0 then self.markevery = markevery
   if n_elements(setBackground) ne 0 then self.setBackground = setBackground


RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::GetOperators
RETURN, ['<', '<=', '>', '>='] 
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::GetOperator, index
   IF N_Elements(index) EQ 0 THEN RETURN, 0
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 5 THEN RETURN, 0
   operators=self->GetOperators()
RETURN, operators(index)
END ;--------------------------------------------------------------------------------




FUNCTION THM_UI_HIGHLIGHT_SETTINGS::GetPlacements
RETURN, ['Top', 'Bottom', 'Top and Bottom']
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::GetPlacement, index
   IF N_Elements(index) EQ 0 THEN RETURN, 0
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 5 THEN RETURN, 0
   placements=self->GetPlacements()
RETURN, placements(index)
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::GetAll
RETURN, self
END ;--------------------------------------------------------------------------------



PRO THM_UI_HIGHLIGHT_SETTINGS::GetProperty,   $
              MarkVertical=markvertical,        $ ; flag to mark with vertical
              LineStyle=linestyle,              $ ; line style object
              Symbol=symbol,                    $ ; symbol object
              Placement=placement,              $ ; droplist value of placement location
              LabelWith=labelwith,              $ ; 0=No label, 1=X value, 2=Y value
              Format=format,                    $ ; format for numeric label 
              MarkWhenY=markwheny,              $ ; flag to mark values when y  
              WhenYEquals=whenyequals,          $ ; y value to mark
              MarkEvery=markevery,              $ ; mark every value
              WhenEveryEquals=wheneveryequals,  $ ; y value to mark
              SetBackground=setbackground,      $ ; flag to set background
              BackgroundWhenY=backgroundwheny,  $ ; droplist of when y is (<, >, <=, >=)
              BackgroundYValue=backgroundyvalue,$ ; y value 
              BackgroundRGB=backgroundrgb,      $ ; rgb background color 
              BackgroundColor=backgroundcolor     ; name for background color

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF Arg_Present(markvertical) THEN markvertical = self.markVertical
   IF Arg_Present(linestyle) THEN linestyle = self.lineStyle
   IF Arg_Present(symbol) THEN symbol = self.symbol
   IF Arg_Present(placement) THEN placement = self.placement
   IF Arg_Present(labelwith) THEN labelwith = self.labelWith
   IF Arg_Present(format) THEN format = self.format
   IF Arg_Present(markwheny) THEN markwheny = self.markWhenY
   IF Arg_Present(whenyequals) THEN whenyequals = self.whenYEquals
   IF Arg_Present(markevery) THEN markevery = self.MarkEvery
   IF Arg_Present(wheneveryequals) THEN wheneveryequals = self.whenEveryEquals
   IF Arg_Present(setbackground) THEN setbackground = self.setBackground
   IF Arg_Present(backgroundwheny) THEN backgroundwheny = self.backgroundWhenY
   IF Arg_Present(backgroundyvalue) THEN backgroundyvalue = self.backgroundYValue
   IF Arg_Present(backgroundcolor) THEN backgroundcolor = self.backgroundColor

RETURN
END ;--------------------------------------------------------------------------------



;PRO THM_UI_HIGHLIGHT_SETTINGS::Cleanup
;   Obj_Destroy, self.lineStyle
;   Obj_Destroy, self.symbol
;END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_HIGHLIGHT_SETTINGS::Init,     $
              MarkVertical=markvertical,        $ ; flag to mark with vertical
              LineStyle=linestyle,              $ ; line style object
              Symbol=symbol,                    $ ; symbol object
              Placement=placement,              $ ; droplist value of placement location
              LabelWith=labelwith,              $ ; 0=No label, 1=X value, 2=Y value
              Format=format,                    $ ; format for numeric label 
              MarkWhenY=markwheny,              $ ; flag to mark values when y  
              WhenYEquals=whenyequals,          $ ; y value to mark
              MarkEvery=markevery,              $ ; mark every value
              WhenEveryEquals=wheneveryequals,  $ ; y value to mark
              SetBackground=setbackground,      $ ; flag to set background
              BackgroundWhenY=backgroundwheny,  $ ; droplist of when y is (<, >, <=, >=)
              BackgroundYValue=backgroundyvalue,$ ; y value 
              BackgroundColor=backgroundcolor,  $ ; background color
              Debug=debug                         ; set to one for debugging

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF
 
   ; Check that all parameters have values
   
   IF N_Elements(placement) EQ 0 THEN placement = 0
   IF N_Elements(labelwith) EQ 0 THEN labelwith = 0
   IF N_Elements(format) EQ 0 THEN format = 0
   IF N_Elements(whenyequals) EQ 0 THEN whenyequals = 0
   IF N_Elements(wheneveryequals) EQ 0 THEN wheneveryequals = 0
   IF N_Elements(backgroundwheny) EQ 0 THEN backgroundwheny = 0
   IF N_Elements(backgroundyvalue) EQ 0 THEN backgroundyvalue = 0
   IF N_Elements(backgroundcolor) EQ 0 THEN backgroundcolor = [0,0,0]
   
   IF NOT Obj_Valid(linestyle) THEN linestyle = Obj_New("THM_UI_LINE_STYLE")
   IF NOT Obj_Valid(symbol) THEN symbol = Obj_New("THM_UI_SYMBOL")

  ; Set all parameters
   
   self.markVertical = Keyword_Set(markvertical)
   self.markWhenY = Keyword_Set(markwheny)
   self.markEvery = Keyword_Set(markevery)
   self.setBackground = Keyword_Set(setbackground)
  
   self.placement = placement
   self.labelWith = labelwith
   self.format = format
   self.whenYEquals = whenyequals
   self.whenEveryEquals = wheneveryequals
   self.backgroundwheny = backgroundwheny
   self.backgroundyValue = backgroundyvalue
   self.backgroundColor = backgroundcolor
   self.lineStyle =linestyle
   self.symbol = symbol
                    
RETURN, 1
END ;--------------------------------------------------------------------------------



PRO THM_UI_HIGHLIGHT_SETTINGS__DEFINE

   struct = { THM_UI_HIGHLIGHT_SETTINGS, $

              markVertical: 0,             $ ; flag to mark vertical
              lineStyle: Obj_New(),        $ ; line style object
              symbol: Obj_New(),           $ ; symbol object
              placement: 0,                $ ; droplist value of placement location
              labelWith: 0,                $ ; 0=No label, 1=X value, 2=Y value
              format: 0,                   $ ; format for x or y values
              markWhenY: 0,                $ ; flag to mark when y equals value
              whenYEquals: 0,              $ ; y value to mark
              markEvery: 0,                $ ; flag to mark y for every value eq. to
              whenEveryEquals: 0,          $ ; y value for every
              setBackground: 0,            $ ; flag to set background
              backgroundWhenY: 0,          $ ; droplist of logic for when y is
              backgroundYValue: 0,         $ ; numerical value for when y is
              backgroundColor: [0,0,0]     $ ; name for background color
                                    
}

END