;+ 
;NAME: 
; thm_ui_spectra_settings__define
;
;PURPOSE:  
; spectra properties is an object that holds all of the settings necessary for 
; spectral data plots
;
;CALLING SEQUENCE:
; spectraSettings = Obj_New("THM_UI_SPECTRA_SETTINGS")
;
;INPUT:
; none
;
;KEYWORDS:
; dataX              string naming the x component of the spectral plot
; dataY              string naming the y component of the spectral plot
; dataz              string naming the z component of the spectral plot
;
;OUTPUT:
; spectra property object reference
;
;METHODS:
; SetProperty    procedure to set keywords 
; GetProperty    procedure to get keywords 
; GetAll         returns the entire structure
; GetPlacements  returns string array of placement options
; GetPlacement   returns name of placement option given an index
; GetColorTables returns string array of color tables 
; GetColorTable  returns name of color table given an index

;HISTORY:
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2009-01-08 14:12:55 -0800 (Thu, 08 Jan 2009) $
;$LastChangedRevision: 4365 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/templates/thm_ui_spectra_settings__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_SPECTRA_SETTINGS::Copy
   out = Obj_New("THM_UI_SPECTRA_SETTINGS")
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out
   RETURN, out
END ;--------------------------------------------------------------------------------



PRO THM_UI_SPECTRA_SETTINGS::SetProperty,    $
              dataX=dataX,                     $ ; string naming the x component of the spectral plot
              dataY=dataY,                     $ ; string naming the y component of the spectral plot
              dataZ=dataZ                        ; string naming the z component of the spectral plot
             
   
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(dataX) NE 0 THEN self.dataX = dataX
   IF N_Elements(dataY) NE 0 THEN self.dataY = dataY
   IF N_Elements(dataZ) NE 0 THEN self.dataZ = dataZ

RETURN
END ;--------------------------------------------------------------------------------


FUNCTION THM_UI_SPECTRA_SETTINGS::GetAll
RETURN, self
END ;--------------------------------------------------------------------------------



PRO THM_UI_SPECTRA_SETTINGS::GetProperty,    $
              dataX=dataX,                     $ ; string naming the x component of the spectral plot
              dataY=dataY,                     $ ; string naming the y component of the spectral plot
              dataZ=dataZ                        ; string naming the z component of the spectral plot
          
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF Arg_Present(dataX) THEN dataX = self.dataX
   IF Arg_Present(dataY) THEN dataY = self.dataY
   IF Arg_Present(dataZ) THEN dataZ = self.dataZ

RETURN
END ;--------------------------------------------------------------------------------


;this routine will update references to a data quantity
;This should be used if a name has changed while traces are
;already in existence .
;arguments should be two arrays of strings, with the arrays being of equal length
;changed will be set to 1 if a value is changed
pro thm_ui_spectra_settings::updatedatareference,oldnames,newnames,changed=changed

  compile_opt idl2

  changed=0
  
  self->getProperty,datax=datax,datay=datay,dataz=dataz
  
  idx = where(datax eq oldnames,c)
  
  if c then begin
    changed = 1
    self->setProperty,datax=newnames[idx]
  endif
  
  idx = where(datay eq oldnames,c)
  
  if c then begin
    changed = 1
    self->setProperty,datay = newnames[idx]
  endif
  
  idx = where(dataz eq oldnames,c)
  
  if c then begin
    changed = 1
    self->setProperty,dataz = newnames[idx]
  endif
  
end

FUNCTION THM_UI_SPECTRA_SETTINGS::Init,dataX=dataX,dataY=dataY,dataZ=dataZ

  if n_elements(dataX) eq 0 then dataX = ''
  if n_elements(dataY) eq 0 then dataY = ''
  if n_elements(dataZ) eq 0 then dataZ = ''
  
  self.dataX = dataX
  self.dataY = dataY
  self.dataZ = dataZ
                 
RETURN, 1
END ;--------------------------------------------------------------------------------

PRO THM_UI_SPECTRA_SETTINGS__DEFINE

   struct = { THM_UI_SPECTRA_SETTINGS,$
              dataX          : '',      $ ; string naming the x component of the spectral plot
              dataY          : '',      $ ; string naming the y component of the spectral plot
              dataZ          : '',      $ ; string naming the z component of the spectral plot
              INHERITS THM_UI_READWRITE $ ; generalized read/write methods
          
                                     
}

END
