;+ 
;NAME: 
; thm_ui_zaxis_settings__define
;
;PURPOSE:  
; zaxis_settings is an object that holds all of the settings necessary for 
; spectral data plots that are common to the entire panel
;
;CALLING SEQUENCE:
; zaxisSettings = Obj_New("THM_UI_ZAXIS_SETTINGS")
;
;INPUT:
; none
;
;KEYWORDS:

; xAxisIndex         flag to index x axis
; yAxisIndex         flag to index y axis
; colorTable         droplist value of color tables
; minRange           minimum range value 
; maxRange           maximum range value 
; fixed              flag to use fixed min/max values
; tickNum            the number of z-axis ticks
; minorTickNum       the number of minor ticks between majors
; annotationStyle    droplist value for annotation style
; annotateTextObject a text object to indicate the size,color,font,format of the annotation text 
; annotationOrientation flag indicating horizontal or vertical annotations 0=horizontal, 1=vertical
; annotateExponent   Flag indicates whether to force annotations into an exponential format.  0: default behavior, 1: Always exponential, 2: Never exponential                   
; labelTextObject a text object to indicate the size,color,font,format of the label text
; labelOrientation flag indicating horizontal/vertical text
; labelMargin in number of pts 
; scaling            0=Linear, 1=Log10, 2= Natural Log
; placement          droplist value of placement location
; margin             number of points between plot & zaxis
; showFrequencies    flag to show frequencies
; frequencyMin       minimum value for showing frequencies 
; frequencyMax       maximum value for showing frequencies 
;
;OUTPUT:
; spectra property object reference
;
;METHODS:
; SetProperty    procedure to set keywords 
; GetProperty    procedure to get keywords 
; GetAll         returns the entire structure
; GetPlacements  returns string array of placement options
; GetPlacement   returns name of placement option given an index
; GetColorTables returns string array of color tables 
; GetColorTable  returns name of color table given an index

;HISTORY:
;
;$LastChangedBy: aaflores $
;$LastChangedDate: 2009-04-15 09:27:30 -0700 (Wed, 15 Apr 2009) $
;$LastChangedRevision: 5623 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/templates/thm_ui_zaxis_settings__define.pro $
;-----------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::Copy
   out = Obj_New("THM_UI_ZAXIS_SETTINGS", /nosave)
   selfClass = Obj_Class(self)
   outClass = Obj_Class(out)
   IF selfClass NE outClass THEN BEGIN
       Print, 'Object classes not identical'
       RETURN, -1
   END
   Struct_Assign, self, out   
   newLabel=Obj_New("THM_UI_TEXT")
   IF Obj_Valid(self.labelTextObject) THEN newLabel=self.labelTextObject->Copy() ELSE $
      newLabel=Obj_New()
   out->SetProperty, LabelTextObject=newLabel
   newAnnotate=Obj_New("THM_UI_TEXT")
   IF Obj_Valid(self.annotateTextObject) THEN newAnnotate=self.annotateTextObject->Copy() ELSE $
      newAnnotate=Obj_New()
   out->SetProperty, AnnotateTextObject=newAnnotate  
   RETURN, out
END ;--------------------------------------------------------------------------------



PRO THM_UI_ZAXIS_SETTINGS::SetProperty,    $
              XAxisIndex=xaxisindex,           $ ; flag to index x axis
              YAxisIndex=yaxisindex,           $ ; flag to index y axis
              ColorTable=colortable,           $ ; droplist value of color tables
              MinRange=minrange,               $ ; minimum range value 
              MaxRange=maxrange,               $ ; maximum range value 
              fixed=fixed,                     $ ; flag to use fixed min/max values
              tickNum=tickNum,                 $ ; the number of z-axis ticks
              minorTickNum=minorTickNum,       $ ; the number of minor ticks between majors 
              AnnotationStyle=annotationstyle, $ ; droplist value for annotation style
              annotateTextObject=annotateTextObject, $ ; a text Object to indicate the size/color/font/format of the annotations
              annotationOrientation=annotationOrientation, $ ; the orientation of zaxis annotations
              annotateExponent=annotateExponent,     $ ;  Flag indicates whether to force annotations into an exponential format.  0: default behavior, 1: Always exponential, 2: Never exponential                   
              labelTextObject=labelTextObject, $ ; a text object to indicate the size/color/font/format of the labels
              labelOrientation=labelOrientation, $ ; the orientation of the z-axis labels
              labelMargin=labelMargin,         $ ; the margin between z-axis & z-axis label
              Scaling=scaling,                 $ ; 0=Linear, 1=Log10, 2 = natural log
              Placement=placement,             $ ; droplist value of placement location
              margin=margin,                   $ ; flag to indicate the number of points between the plot and the zaxis
              ShowFrequencies=showfrequencies, $ ; flag to show frequencies
              FrequencyMin=frequencymin,       $ ; minimum value for showing frequencies 
              FrequencyMax=frequencymax          ; maximum value for showing frequencies 
   
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF N_Elements(xaxisindex) NE 0 THEN self.xAxisIndex = xaxisindex  
   IF N_Elements(yaxisindex) NE 0 THEN self.yAxisIndex = yaxisindex 
   IF N_Elements(colortable) NE 0 THEN  self.colorTable = colortable 
   IF N_Elements(minrange) NE 0 THEN self.minRange = minrange 
   IF N_Elements(maxrange) NE 0 THEN self.maxRange = maxrange 
   IF N_Elements(fixed) NE 0 THEN self.fixed = fixed 
   IF n_elements(tickNum) ne 0 then self.tickNum = tickNum
   if n_elements(minorTickNum) ne 0 then self.minorTickNum = minorTickNum
   IF N_Elements(annotationstyle) NE 0 THEN self.annotationStyle = annotationstyle 
   IF N_Elements(annotateTextObject) NE 0 THEN self.annotateTextObject = annotateTextObject
   If n_elements(annotationOrientation) ne 0 then self.annotationOrientation = annotationOrientation
   if n_elements(annotateExponent) ne 0 then self.annotateExponent = annotateExponent
   if n_elements(labelTextObject) ne 0 then self.labelTextObject = labelTextObject
   if n_elements(labelOrientation) ne 0 then self.labelOrientation = labelOrientation
   if n_elements(labelMargin) ne 0 then self.labelMargin = labelMargin 
   IF N_Elements(scaling) NE 0 THEN self.scaling = scaling 
   IF N_Elements(placement) NE 0 THEN self.placement = placement
   IF n_elements(margin) ne 0 then self.margin = margin 
   IF N_Elements(showfrequencies) NE 0 THEN self.showFrequencies = showfrequencies 
   IF N_Elements(frequencymin) NE 0 THEN self.frequencyMin = frequencymin 
   IF N_Elements(frequencymax) NE 0 THEN  self.frequencyMax = frequencymax

RETURN
END ;--------------------------------------------------------------------------------


FUNCTION THM_UI_ZAXIS_SETTINGS::GetAll

  all=create_struct(name="THM_UI_ZAXIS_SETTINGS")
  struct_assign,self,all
  
RETURN, all
END ;--------------------------------------------------------------------------------



PRO thm_ui_zaxis_settings::setAll,stct

  struct_assign,stct,self

END ;--------------------------------------------------------------------------------



PRO THM_UI_ZAXIS_SETTINGS::Save

  copy = self->copy()
  if ptr_valid(self.origsettings) then begin
    ptr_free,self.origsettings
  endif
  self.origsettings = ptr_new(copy->getall()) 
  
RETURN    
END ;--------------------------------------------------------------------------------



PRO THM_UI_ZAXIS_SETTINGS::Reset

   if ptr_valid(self.origsettings) then begin
    self->setall,*self.origsettings
    self->Save
   endif

END ;--------------------------------------------------------------------------------


FUNCTION THM_UI_ZAXIS_SETTINGS::GetPlacements
RETURN, ['Top', 'Bottom', 'Left', 'Right', 'Do Not Show ColorBar']
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::GetPlacement, index
   IF N_Elements(index) EQ 0 THEN RETURN, 0
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 5 THEN RETURN, 0
   placements = self->GetPlacements()
RETURN, placements(index)
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::GetColorTables
RETURN, ['Rainbow', 'Cool', 'Hot', 'Copper', 'Extreme Hot-Cold', 'Gray','Themis']
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::GetColorTableNumber, index
   IF N_Elements(index) EQ 0 THEN index = self.colorTable
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 6 THEN RETURN, 0
   RETURN, index
END ;--------------------------------------------------------------------------------




FUNCTION THM_UI_ZAXIS_SETTINGS::GetColorTable, index
   IF N_Elements(index) EQ 0 THEN RETURN, 0
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 5 THEN RETURN, 0
   colorTables = self->GetColorTables()
RETURN, colorTables(index)
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::GetAnnotations
return, [ '1234 (rounded down)', '1.', '1.2', '1.23', '1.234', '1.1234', $
            '1.01234', '1.001234','1.0001234','1.00001234','1.000001234']
;RETURN, ['d(1234)', 'f0(1234)', 'f1(123.4)', 'f2(12.34)', 'f3(1.234)', 'f4(0.1234)', $
;   'f5(0.01234)', 'f6(0.001234)', 'e0(123e4)', 'e1(12.3e4)', 'e2(1.23e4)', $
;   'e3(0.123e4)', 'e4(0.01234)', 'e5(0.001234)', 'e6(0.0001234)']
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::GetAnnotation, index
   IF N_Elements(index) EQ 0 THEN RETURN, 0
   IF NOT Is_Numeric(index) THEN RETURN, 0
   IF index LT 0 OR index GT 5 THEN RETURN, 0
   annotations = self->GetAnnotations()
RETURN, annotations(index)
END ;--------------------------------------------------------------------------------



PRO THM_UI_ZAXIS_SETTINGS::GetProperty,    $
              XAxisIndex=xaxisindex,           $ ; flag to index x axis
              YAxisIndex=yaxisindex,           $ ; flag to index y axis
              ColorTable=colortable,           $ ; droplist value of color tables
              MinRange=minrange,               $ ; minimum range value 
              MaxRange=maxrange,               $ ; maximum range value 
              fixed=fixed,                     $ ; flag to use fixed min/max values
              tickNum=tickNum,                 $ ; the number of ticks on the z axis
              minorTickNum=minorTickNum,       $ ; the number of minor ticks between major ticks
              AnnotationStyle=annotationstyle, $ ; droplist value for annotation style 
              annotateTextObject=annotateTextObject, $ ; a text object to indicate the size/color/font/format of annotations
              annotationOrientation=annotationOrientation, $ ; flag for the orientation of the annotations(0: horizontal,1:vertical)
              annotateExponent=annotateExponent,     $ ;  Flag indicates whether to force annotations into an exponential format.  0: default behavior, 1: Always exponential, 2: Never exponential                   
              labelTextObject=labelTextObject, $ ; a text object to indicate the label text/color/font/format/size
              labelOrientation=labelOrientation, $ ; flag for the orientation of the label (0:horizontal,1:vertical)
              labelMargin=labelMargin,         $ ; the margin of the label in points from the zaxis to the label
              Scaling=scaling,                 $ ; 0=Linear, 1=Log10, 2 = Natural Log
              Placement=placement,             $ ; droplist value of placement location
              margin=margin,                   $ ; number of points between plot & zaxis
              ShowFrequencies=showfrequencies, $ ; flag to show frequencies
              FrequencyMin=frequencymin,       $ ; minimum value for showing frequencies 
              FrequencyMax=frequencymax          ; maximum value for showing frequencies 
   
   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF

   IF Arg_Present(xaxisindex) THEN xaxisindex = self.xAxisIndex
   IF Arg_Present(yaxisindex) THEN yaxisindex = self.yAxisIndex
   IF Arg_Present(colortable) THEN colortable = self.colorTable
   IF Arg_Present(minrange) THEN minrange = self.minRange
   IF Arg_Present(maxrange) THEN maxrange = self.maxRange
   IF Arg_Present(fixed) THEN fixed = self.fixed
   IF arg_present(ticknum) then tickNum = self.tickNum
   if arg_present(minorTickNum) then minorTickNum = self.minorTickNum
   IF Arg_Present(annotationstyle) THEN annotationstyle = self.annotationStyle
   IF arg_present(annotateTextObject) then annotateTextObject = self.annotateTextObject
   if arg_present(annotationOrientation) then annotationOrientation = self.annotationOrientation
   if arg_present(annotateExponent) then annotateExponent = self.annotateExponent
   if arg_present(labelTextObject) then labelTextObject = self.labelTextObject
   if arg_present(labelOrientation) then labelOrientation = self.labelOrientation
   if arg_present(labelMargin) then labelMargin = self.labelMargin
   IF Arg_Present(scaling) THEN scaling = self.scaling
   IF Arg_Present(placement) THEN placement = self.placement
   IF arg_present(margin) then margin = self.margin
   IF Arg_Present(showfrequencies) THEN showfrequencies = self.showFrequencies
   IF Arg_Present(frequencymin) THEN frequencymin = self.frequencyMin
   IF Arg_Present(frequencymax) THEN frequencymax = self.frequencyMax

RETURN
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_ZAXIS_SETTINGS::Init, $
                                  xAxisIndex=xAxisIndex, $
                                  yAxisIndex=yAxisIndex, $
                                  colorTable=colorTable,$
                                  minRange=minRange,$
                                  maxRange=maxRange,$
                                  fixed=fixed,$
                                  tickNum=tickNum,$
                                  minorTickNum=minorTickNum,$
                                  annotationStyle=annotationStyle,$
                                  annotateTextObject=annotateTextObject,$
                                  annotationOrientation=annotationOrientation,$
                                  annotateExponent=annotateExponent,     $ ;  Flag indicates whether to force annotations into an exponential format.  0: default behavior, 1: Always exponential, 2: Never exponential                   
                                  labelTextObject=labelTextObject,$
                                  labelOrientation=labelOrientation,$
                                  labelMargin=labelMargin,$
                                  scaling=scaling,$
                                  placement=placement,$
                                  margin=margin,$
                                  showFrequencies=showFrequencies,$
                                  frequencyMin=frequencyMin,$
                                  frequencyMax=frequencyMax, $                      
                                  nosave=nosave
                                  
  if n_elements(xAxisIndex) eq 0 then xAxisIndex = 0
  if n_elements(yAxisIndex) eq 0 then yAxisIndex = 0
  if n_elements(colorTable) eq 0 then colorTable = 0
  if n_elements(minRange) eq 0 then minRange = 1D-99
  if n_elements(maxRange) eq 0 then maxRange = 1.0D
  if n_elements(fixed) eq 0 then fixed = 0
  if n_elements(tickNum) eq 0 then tickNum = 0
  if n_elements(minorTickNum) eq 0 then minorTickNum = 0
  if n_elements(annotationStyle) eq 0 then annotationStyle = 0
  if n_elements(annotateTextObject) eq 0 then annotateTextObject = obj_new()
  if n_elements(annotationOrientation) eq 0 then annotationOrientation = 0
  if n_elements(annotateExponent) eq 0 then annotateExponent = 0
  if n_elements(labelTextObject) eq 0 then labelTextObject = obj_new()
  if n_elements(labelOrientation) eq 0 then labelOrientation = 0
  if n_elements(labelMargin) eq 0 then labelMargin = 0
  if n_elements(scaling) eq 0 then scaling = 0
  if n_elements(placement) eq 0 then placement = 0
  if n_elements(margin) eq 0 then margin = 0
  if n_elements(showFrequencies) eq 0 then showFrequencies = 0
  if n_elements(frequencyMin) eq 0 then frequencyMin = 0
  if n_elements(frequencyMax) eq 0 then frequencyMax = 0
  
  self.xAxisIndex = xAxisIndex
  self.yAxisIndex = yAxisIndex
  self.colorTable = colorTable
  self.minRange = minRange
  self.maxRange = maxRange
  self.fixed = fixed
  self.tickNum = tickNum
  self.minorTickNum = minorTickNum
  self.annotationStyle = annotationStyle
  self.annotateTextObject = annotateTextObject
  self.annotationOrientation = annotationOrientation
  self.annotateExponent = annotateExponent
  self.labelTextObject = labelTextObject
  self.labelOrientation = labelOrientation
  self.labelMargin = labelMargin
  self.scaling = scaling
  self.placement = placement
  self.margin = margin
  self.showFrequencies = showFrequencies
  self.frequencyMin = frequencyMin
  self.frequencyMax = frequencyMax

  if n_elements(nosave) EQ 0 then begin      
      self->save
  endif 
                
RETURN, 1
END ;--------------------------------------------------------------------------------

PRO THM_UI_ZAXIS_SETTINGS__DEFINE

   struct = { THM_UI_ZAXIS_SETTINGS,$
              xAxisIndex     : 0,       $ ; flag to index x axis
              yAxisIndex     : 0,       $ ; flag to index y axis
              colorTable     : 0,       $ ; droplist value of color tables
              minRange       : 0.0D,    $ ; minimum range value 
              maxRange       : 0.0D,    $ ; maximum range value 
              fixed          : 0,       $ ; flag to indicate that fixed range is being used
              tickNum        : 0,       $ ; the number of ticks on the z-axis
              minorTickNum   : 0,       $ ; the number of minor ticks between major ticks
              annotationStyle: 0,       $ ; droplist value for annotation style
              annotateTextObject:Obj_new(), $ ;text object for style of zaxis annotations
              annotationOrientation: 0, $ ; flag indicating horizontal or vertical annotation orientation
              annotateExponent:0 ,      $ ; Flag indicates whether to force annotations into an exponential format.  0: default behavior, 1: Always exponential, 2: Never exponential                   
              labelTextObject: Obj_new(), $ ; a text object indicating the text/color/font/format/size of the label
              labelOrientation: 0,      $ ; flag indicating horizontal or vertical label orientation
              labelMargin: 0,           $ ; the margin between the label and the axis in points 
              scaling        : 0,       $ ; 0=Linear, 1=Log10, 2 = Natural Log
              placement      : 0,       $ ; droplist value of placement location
              margin         : 0,       $ ; margin of spacing between plot and zaxis in pts
              showFrequencies: 0,       $ ; flag to show frequencies
              frequencyMin   : 0,       $ ; minimum value for showing frequencies 
              frequencyMax   : 0,       $ ; maximum value for showing frequencies 
              origsettings: ptr_New(),  $  ;pointer to original settings in case of reset
              INHERITS THM_UI_READWRITE $ ; generalized read/write methods
                                     
}

END
