;+ 
;NAME:
; thm_ui_test_axis_settings
;
;PURPOSE:
; test script for panel settings object
;
;CALLING SEQUENCE:
; thm_ui_test_axis_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_axis_settings.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_axis_settings

    ;initializations

    Print, '**********  Starting axis setting tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myGS = Obj_New("THM_UI_AXIS_SETTINGS")
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 1: Did not initialize'
  
    ; Test  2: Error Test Get for all     
    myGS->GetProperty, $
      RangeOption=rangeoption,               $ ; range options (auto, floating, fixed)
      Scaling=scaling,                       $ ; axis scaling (linear, log10, natural)
      FixedMin=fixedmin,                     $ ; if fixed, minimum value of range 
      FixedMax=fixedmax,                     $ ; if fixed, maximum value of range
      EqualXYScaling=equalxyscaling,         $ ; flag set if equal X & Y axis scaled 
      IsTimeAxis=istimeaxis,                 $ ; flag set if axis is time based
      MinTime=mintime,                       $ ; if fixed, minimum time (object)
      MaxTime=maxtime,                       $ ; if fixed, maximum time (object)
      RangeMargin=rangemargin,               $ ; if auto - range margin
      BoundScaling=boundscaling,             $ ; flag set if auto scaling is bound
      MinRange=minrange,                     $ ; min range value if fixed scaling
      MaxRange=maxrange,                     $ ; max range value if fixed scaling
      FloatingSpan=floatingspan,             $ ; value of span if floating
      FloatingCenter=floatingcenter,         $ ; flag set if float center
      MajorTickEvery=majortickevery,         $ ; display major ticks every 
      MinorTickEvery=minortickevery,         $ ; display major ticks every 
      MajorTickUnits=majortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MinorTickUnits=minortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MajorTickAuto=majortickauto,           $ ; set to automatically figure out major ticks
      MinorTickAuto=minortickauto,           $ ; set to automatically figure out minior ticks
      NumMajorTicks=nummajorticks,           $ ; number of major ticks
      NumMinorTicks=numminorticks,           $ ; number of major ticks
      FirstTickAt=firsttickat,               $ ; value where first tick should be
      FirstTickUnits=firsttickunits,         $ ; first tick unit (sec, hr, min, day, none)
      TickStyle=tickstyle,                   $ ; style (inside, outside, both)
      LineThickness=linethickness,           $ ; thickness of tick mark
      BottomPlacement=bottomplacement,       $ ; flag set if ticks should be on bottom axis
      TopPlacement=topplacement,             $ ; flag set if ticks should be on top axis
      MajorLength=majorlength,               $ ; length of major tick
      MinorLength=minorlength,               $ ; length of minor tick
      LineAtZero=lineatzero,                 $ ; flag set if line is drawn at zero
      Date=date,                             $ ; time object for annotation                                   
      AnnotateAxis=annotateaxis,             $ ; flag set to annotate along axis                         
      PlaceAnnotation=placeannotation,       $ ; placement of annotation (bottom or top)
      AnnotateMajorTicks=annotatemajorticks, $ ; set flag if major ticks are annotated
      AnnotateMajorEvery=annotatemajorevery, $ ; value where annotation of major ticks occur
      AnnotateMajorUnits=annotatemajorunits, $ ; units for major tick value (sec, min, hr, day)
      FirstAnnotation=firstannotation,       $ ; value where annotation of first major tick occurs
      FirstAnnotateUnits=firstannotateunits, $ ; units for major tick value (sec, min, hr, day)
      AnnotateFirstTick=annotatefirsttick,   $ ; set flag to annotate first tick  
      AnnotateStyle=annotatestyle,           $ ; format style of tick (h:m, doy, time, etc....)
      XGrid=xgrid,                           $ ; grid object for x axis 
      YGrid=ygrid,                           $ ; grid object for y axis 
      FrameStyle=frameStyle,                 $ ; line style for panel frame 
      Label=label,                           $ ; axis label (text object) 
      Orientation=orientation,               $ ; orientation of labels 0=Horizontal, 1=Vertical
      Margin=margin                            ; number of points for label margins

   IF rangeoption NE 0 THEN Print, 'Test Error 1a: invalid property returned'
   IF scaling NE 0 THEN Print, 'Test Error 1b: invalid property returned'
   IF fixedmin NE 0.0 THEN Print, 'Test Error 1c: invalid property returned'
   IF fixedmax NE 0.0 THEN Print, 'Test Error 1d: invalid property returned'
   IF equalxyscaling NE 0 THEN Print, 'Test Error 1e: invalid property returned'
   IF istimeaxis NE 1 THEN Print,'Test Error 1f: invalid property returned'
   IF NOT Obj_Valid(mintime) THEN Print, 'Test Error 1g: invalid property returned'
   IF NOT Obj_Valid(maxtime) THEN Print, 'Test Error 1h: invalid property returned'
   IF rangemargin NE 5 THEN Print, 'Test Error 1i: invalid property returned'
   IF boundscaling NE 0 THEN Print,'Test Error 1j: invalid property returned'
   IF minrange NE 0.0 THEN Print, 'Test Error 1k: invalid property returned'
   IF maxrange NE 0.0 THEN Print, 'Test Error 1l: invalid property returned'
   IF floatingspan NE 20 THEN Print, 'Test Error 1m: invalid property returned'
   IF floatingcenter NE 0 THEN Print, 'Test Error 1n: invalid property returned'
   IF majortickevery NE 0 THEN Print,'Test Error 1o: invalid property returned'
   IF minortickevery NE 0 THEN Print, 'Test Error 1p: invalid property returned'
   IF majortickunits NE 0 THEN Print,'Test Error 1q: invalid property returned'
   IF minortickunits NE 0 THEN Print, 'Test Error 1r: invalid property returned'
   IF majortickauto NE 0 THEN Print,'Test Error 1s: invalid property returned'
   IF minortickauto NE 0 THEN Print,'Test Error 1t: invalid property returned'
   IF nummajorticks NE 0 THEN Print, 'Test Error 1u: invalid property returned'
   IF numminorticks NE 0 THEN Print,'Test Error 1v: invalid property returne'
   IF firsttickat NE 0 THEN Print, 'Test Error 1w: invalid property returned'
   IF firsttickunits NE 0 THEN Print, 'Test Error 1x: invalid property returned'
   IF tickstyle NE 0 THEN Print, 'Test Error 1y: invalid property returned'
   IF linethickness NE 0 THEN Print,'Test Error 1z: invalid property returned'
   IF bottomplacement NE 0 THEN Print, 'Test Error 1aa: invalid property returned'
   IF topplacement NE 0 THEN Print, 'Test Error 1bb: invalid property returned'
   IF majorlength NE 0 THEN Print,'Test Error 1cc: invalid property returned'
   IF minorlength NE 0 THEN Print,'Test Error 1dd: invalid property returned'
   IF lineatzero NE 0 THEN Print,'Test Error 1ee: invalid property returned'
   IF NOT Obj_Valid(date) THEN Print,'Test Error 1ff: invalid property returned'
   IF annotateaxis NE 0 THEN Print,'Test Error 1gg: invalid property returned'
   IF placeannotation NE 0 THEN Print,'Test Error 1hh: invalid property returned'
   IF annotatemajorticks NE 0 THEN Print,'Test Error 1ii: invalid property returned'
   IF annotatemajorevery NE 0 THEN Print,'Test Error 1jj: invalid property returned'
   IF annotatemajorunits NE 0 THEN Print,'Test Error 1kk: invalid property returned'
   IF firstannotation NE 0 THEN Print,'Test Error 1ll: invalid property returned'
   IF firstannotateunits NE 0 THEN Print,'Test Error 1mm: invalid property returned'
   IF annotatefirsttick NE 0 THEN Print,'Test Error 1nn: invalid property returned'
   IF annotatestyle NE 0 THEN Print,'Test Error 1oo: invalid property returned'
   IF NOT Obj_Valid(xgrid) THEN Print,'Test Error 1pp: invalid property returned'
   IF NOT Obj_Valid(ygrid) THEN Print,'Test Error 1qq: invalid property returned'
   IF NOT Obj_Valid(framestyle) THEN Print,'Test Error 1rr: invalid property returned'
   IF NOT Obj_Valid(label) THEN Print,'Test Error 1ss: invalid property returned'
   IF orientation NE 0 THEN Print,'Test Error 1tt: invalid property returned'
   IF margin NE 0 THEN Print,'Test Error 1uu: invalid property returned'

     ; Test  2: Error Test for initializing with parameters  
    mint=Obj_New("THM_UI_TIME_RANGE", StartTime='2008-05-05/00:00:00.0')
    maxt=Obj_New("THM_UI_TIME_RANGE", EndTime='2008-05-05/00:00:00.0')
    date=Obj_New("THM_UI_TIME_RANGE", StartTime='2008-09-13/00:00:00.0')
    fs=Obj_New("THM_UI_LINE_STYLE", Color=[0,255,0])
    xg=Obj_New("THM_UI_GRID_SETTINGS", StyleX=fs)
    yg=Obj_New("THM_UI_GRID_SETTINGS", StyleY=fs)
    l=Obj_New("THM_UI_TEXT", Value='Hi', Color=[255,0,0])
    myGS = Obj_New("THM_UI_AXIS_SETTINGS", $
      RangeOption=1,               $ ; range options (auto, floating, fixed)
      Scaling=1,                       $ ; axis scaling (linear, log10, natural)
      FixedMin=1.0,                     $ ; if fixed, minimum value of range 
      FixedMax=1.0,                     $ ; if fixed, maximum value of range
      EqualXYScaling=1,         $ ; flag set if equal X & Y axis scaled 
      IsTimeAxis=0,                 $ ; flag set if axis is time based
      MinTime=mint,                       $ ; if fixed, minimum time (object)
      MaxTime=maxt,                       $ ; if fixed, maximum time (object)
      RangeMargin=1,               $ ; if auto - range margin
      BoundScaling=1,             $ ; flag set if auto scaling is bound
      MinRange=1,                     $ ; min range value if fixed scaling
      MaxRange=1,                     $ ; max range value if fixed scaling
      FloatingSpan=1,             $ ; value of span if floating
      FloatingCenter=1,         $ ; flag set if float center
      MajorTickEvery=1,         $ ; display major ticks every 
      MinorTickEvery=1,         $ ; display major ticks every 
      MajorTickUnits=1,         $ ; major tick units (sec, hr, min, day, none)
      MinorTickUnits=1,         $ ; major tick units (sec, hr, min, day, none)
      MajorTickAuto=1,           $ ; set to automatically figure out major ticks
      MinorTickAuto=1,           $ ; set to automatically figure out minior ticks
      NumMajorTicks=1,           $ ; number of major ticks
      NumMinorTicks=1,           $ ; number of major ticks
      FirstTickAt=1,               $ ; value where first tick should be
      FirstTickUnits=1,         $ ; first tick unit (sec, hr, min, day, none)
      TickStyle=1,                   $ ; style (inside, outside, both)
      LineThickness=1,           $ ; thickness of tick mark
      BottomPlacement=1,       $ ; flag set if ticks should be on bottom axis
      TopPlacement=1,             $ ; flag set if ticks should be on top axis
      MajorLength=1.0,               $ ; length of major tick
      MinorLength=1.0,               $ ; length of minor tick
      LineAtZero=1,                 $ ; flag set if line is drawn at zero
      Date=date,                             $ ; time object for annotation                                   
      AnnotateAxis=1,             $ ; flag set to annotate along axis                         
      PlaceAnnotation=1,       $ ; placement of annotation (bottom or top)
      AnnotateMajorTicks=1, $ ; set flag if major ticks are annotated
      AnnotateMajorEvery=1, $ ; value where annotation of major ticks occur
      AnnotateMajorUnits=1, $ ; units for major tick value (sec, min, hr, day)
      FirstAnnotation=1,       $ ; value where annotation of first major tick occurs
      FirstAnnotateUnits=1, $ ; units for major tick value (sec, min, hr, day)
      AnnotateFirstTick=1,   $ ; set flag to annotate first tick  
      AnnotateStyle=1,           $ ; format style of tick (h:m, doy, time, etc....)
      XGrid=xg,                           $ ; grid object for x axis 
      YGrid=yg,                           $ ; grid object for x axis 
      FrameStyle=fs,                 $ ; line style for panel frame 
      Label=l,                           $ ; axis label (text object) 
      Orientation=1,               $ ; orientation of labels 0=Horizontal, 1=Vertical
      Margin=1)                            ; number of points for label margins
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 1: Did not initialize'
    myGS->GetProperty, $
      RangeOption=rangeoption,               $ ; range options (auto, floating, fixed)
      Scaling=scaling,                       $ ; axis scaling (linear, log10, natural)
      FixedMin=fixedmin,                     $ ; if fixed, minimum value of range 
      FixedMax=fixedmax,                     $ ; if fixed, maximum value of range
      EqualXYScaling=equalxyscaling,         $ ; flag set if equal X & Y axis scaled 
      IsTimeAxis=istimeaxis,                 $ ; flag set if axis is time based
      MinTime=mintime,                       $ ; if fixed, minimum time (object)
      MaxTime=maxtime,                       $ ; if fixed, maximum time (object)
      RangeMargin=rangemargin,               $ ; if auto - range margin
      BoundScaling=boundscaling,             $ ; flag set if auto scaling is bound
      MinRange=minrange,                     $ ; min range value if fixed scaling
      MaxRange=maxrange,                     $ ; max range value if fixed scaling
      FloatingSpan=floatingspan,             $ ; value of span if floating
      FloatingCenter=floatingcenter,         $ ; flag set if float center
      MajorTickEvery=majortickevery,         $ ; display major ticks every 
      MinorTickEvery=minortickevery,         $ ; display major ticks every 
      MajorTickUnits=majortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MinorTickUnits=minortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MajorTickAuto=majortickauto,           $ ; set to automatically figure out major ticks
      MinorTickAuto=minortickauto,           $ ; set to automatically figure out minior ticks
      NumMajorTicks=nummajorticks,           $ ; number of major ticks
      NumMinorTicks=numminorticks,           $ ; number of major ticks
      FirstTickAt=firsttickat,               $ ; value where first tick should be
      FirstTickUnits=firsttickunits,         $ ; first tick unit (sec, hr, min, day, none)
      TickStyle=tickstyle,                   $ ; style (inside, outside, both)
      LineThickness=linethickness,           $ ; thickness of tick mark
      BottomPlacement=bottomplacement,       $ ; flag set if ticks should be on bottom axis
      TopPlacement=topplacement,             $ ; flag set if ticks should be on top axis
      MajorLength=majorlength,               $ ; length of major tick
      MinorLength=minorlength,               $ ; length of minor tick
      LineAtZero=lineatzero,                 $ ; flag set if line is drawn at zero
      Date=date,                             $ ; time object for annotation                                   
      AnnotateAxis=annotateaxis,             $ ; flag set to annotate along axis                         
      PlaceAnnotation=placeannotation,       $ ; placement of annotation (bottom or top)
      AnnotateMajorTicks=annotatemajorticks, $ ; set flag if major ticks are annotated
      AnnotateMajorEvery=annotatemajorevery, $ ; value where annotation of major ticks occur
      AnnotateMajorUnits=annotatemajorunits, $ ; units for major tick value (sec, min, hr, day)
      FirstAnnotation=firstannotation,       $ ; value where annotation of first major tick occurs
      FirstAnnotateUnits=firstannotateunits, $ ; units for major tick value (sec, min, hr, day)
      AnnotateFirstTick=annotatefirsttick,   $ ; set flag to annotate first tick  
      AnnotateStyle=annotatestyle,           $ ; format style of tick (h:m, doy, time, etc....)
      XGrid=xgrid,                           $ ; grid object for x axis 
      YGrid=ygrid,                           $ ; grid object for x axis 
      FrameStyle=frameStyle,                 $ ; line style for panel frame 
      Label=label,                           $ ; axis label (text object) 
      Orientation=orientation,               $ ; orientation of labels 0=Horizontal, 1=Vertical
      Margin=margin                            ; number of points for label margins

   IF rangeoption NE 1 THEN Print, 'Test Error 2a: invalid property returned'
   IF scaling NE 1 THEN Print, 'Test Error 2b: invalid property returned'
   IF fixedmin NE 1.0 THEN Print, 'Test Error 2c: invalid property returned'
   IF fixedmax NE 1.0 THEN Print, 'Test Error 2d: invalid property returned'
   IF equalxyscaling NE 1 THEN Print, 'Test Error 2e: invalid property returned'
   IF istimeaxis NE 0 THEN Print,'Test Error 2f: invalid property returned'
   IF NOT Obj_Valid(mintime) THEN Print, 'Test Error 2g: invalid property returned'
   IF NOT Obj_Valid(maxtime) THEN Print, 'Test Error 2h: invalid property returned'
   IF rangemargin NE 1 THEN Print, 'Test Error 2i: invalid property returned'
   IF boundscaling NE 1 THEN Print,'Test Error 2j: invalid property returned'
   IF minrange NE 1.0 THEN Print, 'Test Error 2k: invalid property returned'
   IF maxrange NE 1.0 THEN Print, 'Test Error 2l: invalid property returned'
   IF floatingspan NE 1 THEN Print, 'Test Error 2m: invalid property returned'
   IF floatingcenter NE 1 THEN Print, 'Test Error 2n: invalid property returned'
   IF majortickevery NE 1 THEN Print,'Test Error 2o: invalid property returned'
   IF minortickevery NE 1 THEN Print, 'Test Error 2p: invalid property returned'
   IF majortickunits NE 1 THEN Print,'Test Error 2q: invalid property returned'
   IF minortickunits NE 1 THEN Print, 'Test Error 2r: invalid property returned'
   IF majortickauto NE 1 THEN Print,'Test Error 2s: invalid property returned'
   IF minortickauto NE 1 THEN Print,'Test Error 2t: invalid property returned'
   IF nummajorticks NE 1 THEN Print, 'Test Error 2u: invalid property returned'
   IF numminorticks NE 1 THEN Print,'Test Error 2v: invalid property returne'
   IF firsttickat NE 1 THEN Print, 'Test Error 2w: invalid property returned'
   IF firsttickunits NE 1 THEN Print, 'Test Error 2x: invalid property returned'
   IF tickstyle NE 1 THEN Print, 'Test Error 2y: invalid property returned'
   IF linethickness NE 1 THEN Print,'Test Error 2z: invalid property returned'
   IF bottomplacement NE 1 THEN Print, 'Test Error 2aa: invalid property returned'
   IF topplacement NE 1 THEN Print, 'Test Error 2bb: invalid property returned'
   IF majorlength NE 1 THEN Print,'Test Error 2cc: invalid property returned'
   IF minorlength NE 1 THEN Print,'Test Error 2dd: invalid property returned'
   IF lineatzero NE 1 THEN Print,'Test Error 2ee: invalid property returned'
   IF NOT Obj_Valid(date) THEN Print,'Test Error 2ff: invalid property returned'
   IF annotateaxis NE 1 THEN Print,'Test Error 2gg: invalid property returned'
   IF placeannotation NE 1 THEN Print,'Test Error 2hh: invalid property returned'
   IF annotatemajorticks NE 1 THEN Print,'Test Error 2ii: invalid property returned'
   IF annotatemajorevery NE 1 THEN Print,'Test Error 2jj: invalid property returned'
   IF annotatemajorunits NE 1 THEN Print,'Test Error 2kk: invalid property returned'
   IF firstannotation NE 1 THEN Print,'Test Error 2ll: invalid property returned'
   IF firstannotateunits NE 1 THEN Print,'Test Error 2mm: invalid property returned'
   IF annotatefirsttick NE 1 THEN Print,'Test Error 2nn: invalid property returned'
   IF annotatestyle NE 1 THEN Print,'Test Error 2oo: invalid property returned'
   IF NOT Obj_Valid(xgrid) THEN Print,'Test Error 2pp: invalid property returned'
   IF NOT Obj_Valid(ygrid) THEN Print,'Test Error 2qq: invalid property returned'
   IF NOT Obj_Valid(framestyle) THEN Print,'Test Error 2rr: invalid property returned'
   IF NOT Obj_Valid(label) THEN Print,'Test Error 2ss: invalid property returned'
   IF orientation NE 1 THEN Print,'Test Error 2tt: invalid property returned'
   IF margin NE 1 THEN Print,'Test Error 2uu: invalid property returned'
   mt = minTime->GetStartTime(/String)
   IF mt NE '2008-05-05/00:00:00.0' THEN Print,'Test Error 2vv: invalid property returned'
   mt = maxTime->GetEndTime(/String)
   IF mt NE '2008-05-05/00:00:00.0' THEN Print,'Test Error 2ww: invalid property returned'
   mt = date->GetStartTime(/String)
   IF mt NE '2008-09-13/00:00:00.0' THEN Print,'Test Error 2xx: invalid property returned'
   label->GetProperty, Value=v, Color=c 
   IF c[0] NE 255 THEN Print, 'Test Error 2yy: Invalid color returned'
   IF v NE 'Hi' THEN Print, 'Test Error 2zz: Invalid color returned'
   xgrid->GetProperty, StyleX=sx
   sx->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 2aaa: Invalid color returned'
   ygrid->GetProperty, StyleY=sx
   sx->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 2bbb: Invalid color returned'
   framestyle->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 2ccc: Invalid color returned'

    ; Test  3: Error Test for set properties  
    myGS->SetProperty, $
      RangeOption=2,               $ ; range options (auto, floating, fixed)
      Scaling=2,                       $ ; axis scaling (linear, log10, natural)
      FixedMin=2.0,                     $ ; if fixed, minimum value of range 
      FixedMax=2.0,                     $ ; if fixed, maximum value of range
      EqualXYScaling=2,         $ ; flag set if equal X & Y axis scaled 
      IsTimeAxis=2,                 $ ; flag set if axis is time based
      MinTime=mint,                       $ ; if fixed, minimum time (object)
      MaxTime=maxt,                       $ ; if fixed, maximum time (object)
      RangeMargin=2,               $ ; if auto - range margin
      BoundScaling=2,             $ ; flag set if auto scaling is bound
      MinRange=2,                     $ ; min range value if fixed scaling
      MaxRange=2,                     $ ; max range value if fixed scaling
      FloatingSpan=2,             $ ; value of span if floating
      FloatingCenter=2,         $ ; flag set if float center
      MajorTickEvery=2,         $ ; display major ticks every 
      MinorTickEvery=2,         $ ; display major ticks every 
      MajorTickUnits=2,         $ ; major tick units (sec, hr, min, day, none)
      MinorTickUnits=2,         $ ; major tick units (sec, hr, min, day, none)
      MajorTickAuto=2,           $ ; set to automatically figure out major ticks
      MinorTickAuto=2,           $ ; set to automatically figure out minior ticks
      NumMajorTicks=2,           $ ; number of major ticks
      NumMinorTicks=2,           $ ; number of major ticks
      FirstTickAt=2,               $ ; value where first tick should be
      FirstTickUnits=2,         $ ; first tick unit (sec, hr, min, day, none)
      TickStyle=2,                   $ ; style (inside, outside, both)
      LineThickness=2,           $ ; thickness of tick mark
      BottomPlacement=2,       $ ; flag set if ticks should be on bottom axis
      TopPlacement=2,             $ ; flag set if ticks should be on top axis
      MajorLength=2.0,               $ ; length of major tick
      MinorLength=2.0,               $ ; length of minor tick
      LineAtZero=2,                 $ ; flag set if line is drawn at zero
      Date=date,                             $ ; time object for annotation                                   
      AnnotateAxis=2,             $ ; flag set to annotate along axis                         
      PlaceAnnotation=2,       $ ; placement of annotation (bottom or top)
      AnnotateMajorTicks=2, $ ; set flag if major ticks are annotated
      AnnotateMajorEvery=2, $ ; value where annotation of major ticks occur
      AnnotateMajorUnits=2, $ ; units for major tick value (sec, min, hr, day)
      FirstAnnotation=2,       $ ; value where annotation of first major tick occurs
      FirstAnnotateUnits=2, $ ; units for major tick value (sec, min, hr, day)
      AnnotateFirstTick=2,   $ ; set flag to annotate first tick  
      AnnotateStyle=2,           $ ; format style of tick (h:m, doy, time, etc....)
      XGrid=xg,                           $ ; grid object for x axis 
      YGrid=yg,                           $ ; grid object for x axis 
      FrameStyle=fs,                 $ ; line style for panel frame 
      Label=l,                           $ ; axis label (text object) 
      Orientation=2,               $ ; orientation of labels 0=Horizontal, 1=Vertical
      Margin=2                            ; number of points for label margins
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 1: Did not initialize'
    myGS->GetProperty, $
      RangeOption=rangeoption,               $ ; range options (auto, floating, fixed)
      Scaling=scaling,                       $ ; axis scaling (linear, log10, natural)
      FixedMin=fixedmin,                     $ ; if fixed, minimum value of range 
      FixedMax=fixedmax,                     $ ; if fixed, maximum value of range
      EqualXYScaling=equalxyscaling,         $ ; flag set if equal X & Y axis scaled 
      IsTimeAxis=istimeaxis,                 $ ; flag set if axis is time based
      MinTime=mintime,                       $ ; if fixed, minimum time (object)
      MaxTime=maxtime,                       $ ; if fixed, maximum time (object)
      RangeMargin=rangemargin,               $ ; if auto - range margin
      BoundScaling=boundscaling,             $ ; flag set if auto scaling is bound
      MinRange=minrange,                     $ ; min range value if fixed scaling
      MaxRange=maxrange,                     $ ; max range value if fixed scaling
      FloatingSpan=floatingspan,             $ ; value of span if floating
      FloatingCenter=floatingcenter,         $ ; flag set if float center
      MajorTickEvery=majortickevery,         $ ; display major ticks every 
      MinorTickEvery=minortickevery,         $ ; display major ticks every 
      MajorTickUnits=majortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MinorTickUnits=minortickunits,         $ ; major tick units (sec, hr, min, day, none)
      MajorTickAuto=majortickauto,           $ ; set to automatically figure out major ticks
      MinorTickAuto=minortickauto,           $ ; set to automatically figure out minior ticks
      NumMajorTicks=nummajorticks,           $ ; number of major ticks
      NumMinorTicks=numminorticks,           $ ; number of major ticks
      FirstTickAt=firsttickat,               $ ; value where first tick should be
      FirstTickUnits=firsttickunits,         $ ; first tick unit (sec, hr, min, day, none)
      TickStyle=tickstyle,                   $ ; style (inside, outside, both)
      LineThickness=linethickness,           $ ; thickness of tick mark
      BottomPlacement=bottomplacement,       $ ; flag set if ticks should be on bottom axis
      TopPlacement=topplacement,             $ ; flag set if ticks should be on top axis
      MajorLength=majorlength,               $ ; length of major tick
      MinorLength=minorlength,               $ ; length of minor tick
      LineAtZero=lineatzero,                 $ ; flag set if line is drawn at zero
      Date=date,                             $ ; time object for annotation                                   
      AnnotateAxis=annotateaxis,             $ ; flag set to annotate along axis                         
      PlaceAnnotation=placeannotation,       $ ; placement of annotation (bottom or top)
      AnnotateMajorTicks=annotatemajorticks, $ ; set flag if major ticks are annotated
      AnnotateMajorEvery=annotatemajorevery, $ ; value where annotation of major ticks occur
      AnnotateMajorUnits=annotatemajorunits, $ ; units for major tick value (sec, min, hr, day)
      FirstAnnotation=firstannotation,       $ ; value where annotation of first major tick occurs
      FirstAnnotateUnits=firstannotateunits, $ ; units for major tick value (sec, min, hr, day)
      AnnotateFirstTick=annotatefirsttick,   $ ; set flag to annotate first tick  
      AnnotateStyle=annotatestyle,           $ ; format style of tick (h:m, doy, time, etc....)
      XGrid=xgrid,                           $ ; grid object for x axis 
      YGrid=ygrid,                           $ ; grid object for x axis 
      FrameStyle=frameStyle,                 $ ; line style for panel frame 
      Label=label,                           $ ; axis label (text object) 
      Orientation=orientation,               $ ; orientation of labels 0=Horizontal, 1=Vertical
      Margin=margin                            ; number of points for label margins
      
   IF rangeoption NE 2 THEN Print, 'Test Error 3a: invalid property returned'
   IF scaling NE 2 THEN Print, 'Test Error 3b: invalid property returned'
   IF fixedmin NE 2.0 THEN Print, 'Test Error 3c: invalid property returned'
   IF fixedmax NE 2.0 THEN Print, 'Test Error 3d: invalid property returned'
   IF equalxyscaling NE 2 THEN Print, 'Test Error 3e: invalid property returned'
   IF istimeaxis NE 2 THEN Print,'Test Error 3f: invalid property returned'
   IF NOT Obj_Valid(mintime) THEN Print, 'Test Error 3g: invalid property returned'
   IF NOT Obj_Valid(maxtime) THEN Print, 'Test Error 3h: invalid property returned'
   IF rangemargin NE 2 THEN Print, 'Test Error 3i: invalid property returned'
   IF boundscaling NE 2 THEN Print,'Test Error 3j: invalid property returned'
   IF minrange NE 2.0 THEN Print, 'Test Error 3k: invalid property returned'
   IF maxrange NE 2.0 THEN Print, 'Test Error 3l: invalid property returned'
   IF floatingspan NE 2 THEN Print, 'Test Error 3m: invalid property returned'
   IF floatingcenter NE 2 THEN Print, 'Test Error 3n: invalid property returned'
   IF majortickevery NE 2 THEN Print,'Test Error 3o: invalid property returned'
   IF minortickevery NE 2 THEN Print, 'Test Error 3p: invalid property returned'
   IF majortickunits NE 2 THEN Print,'Test Error 3q: invalid property returned'
   IF minortickunits NE 2 THEN Print, 'Test Error 3r: invalid property returned'
   IF majortickauto NE 2 THEN Print,'Test Error 3s: invalid property returned'
   IF minortickauto NE 2 THEN Print,'Test Error 3t: invalid property returned'
   IF nummajorticks NE 2 THEN Print, 'Test Error 3u: invalid property returned'
   IF numminorticks NE 2 THEN Print,'Test Error 3v: invalid property returne'
   IF firsttickat NE 2 THEN Print, 'Test Error 3w: invalid property returned'
   IF firsttickunits NE 2 THEN Print, 'Test Error 3x: invalid property returned'
   IF tickstyle NE 2 THEN Print, 'Test Error 3y: invalid property returned'
   IF linethickness NE 2 THEN Print,'Test Error 3z: invalid property returned'
   IF bottomplacement NE 2 THEN Print, 'Test Error 3aa: invalid property returned'
   IF topplacement NE 2 THEN Print, 'Test Error 3bb: invalid property returned'
   IF majorlength NE 2 THEN Print,'Test Error 3cc: invalid property returned'
   IF minorlength NE 2 THEN Print,'Test Error 3dd: invalid property returned'
   IF lineatzero NE 2 THEN Print,'Test Error 3ee: invalid property returned'
   IF NOT Obj_Valid(date) THEN Print,'Test Error 3ff: invalid property returned'
   IF annotateaxis NE 2 THEN Print,'Test Error 3gg: invalid property returned'
   IF placeannotation NE 2 THEN Print,'Test Error 3hh: invalid property returned'
   IF annotatemajorticks NE 2 THEN Print,'Test Error 3ii: invalid property returned'
   IF annotatemajorevery NE 2 THEN Print,'Test Error 3jj: invalid property returned'
   IF annotatemajorunits NE 2 THEN Print,'Test Error 3kk: invalid property returned'
   IF firstannotation NE 2 THEN Print,'Test Error 3ll: invalid property returned'
   IF firstannotateunits NE 2 THEN Print,'Test Error 3mm: invalid property returned'
   IF annotatefirsttick NE 2 THEN Print,'Test Error 3nn: invalid property returned'
   IF annotatestyle NE 2 THEN Print,'Test Error 3oo: invalid property returned'
   IF NOT Obj_Valid(xgrid) THEN Print,'Test Error 3pp: invalid property returned'
   IF NOT Obj_Valid(ygrid) THEN Print,'Test Error 3qq: invalid property returned'
   IF NOT Obj_Valid(framestyle) THEN Print,'Test Error 3rr: invalid property returned'
   IF NOT Obj_Valid(label) THEN Print,'Test Error 3ss: invalid property returned'
   IF orientation NE 2 THEN Print,'Test Error 3tt: invalid property returned'
   IF margin NE 2 THEN Print,'Test Error 3uu: invalid property returned'
   mt = minTime->GetStartTime(/String)
   IF mt NE '2008-05-05/00:00:00.0' THEN Print,'Test Error 3vv: invalid property returned'
   mt = maxTime->GetEndTime(/String)
   IF mt NE '2008-05-05/00:00:00.0' THEN Print,'Test Error 3ww: invalid property returned'
   mt = date->GetStartTime(/String)
   IF mt NE '2008-09-13/00:00:00.0' THEN Print,'Test Error 3xx: invalid property returned'
   label->GetProperty, Value=v, Color=c 
   IF c[0] NE 255 THEN Print, 'Test Error 3yy: Invalid color returned'
   IF v NE 'Hi' THEN Print, 'Test Error 3zz: Invalid color returned'
   xgrid->GetProperty, StyleX=sx
   sx->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 3aaa: Invalid color returned'
   ygrid->GetProperty, StyleY=sx
   sx->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 3bbb: Invalid color returned'
   framestyle->GetProperty, Color=c
   IF c[1] NE 255 THEN Print, 'Test Error 3ccc: Invalid color returned'

    Print, '**********  Done with axis settings test  ***************'
    Obj_Destroy, myGS

END
