;+
;NAME:
; thm_ui_test_data
;
;PURPOSE:
; test script for data object
;
;CALLING SEQUENCE:
; thm_ui_test_data
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_data.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_data

    ;initializations

    Print, '**********  Starting Initialization tests  ***************'


    ; test that invalid parameters are handled gracefully
    ; Test 1: No name or groupid provided
    myData = Obj_New("THM_UI_DATA")
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 1: Did not give required init input'

    ; Init Test 2: No groupid given
    myData = Obj_New("THM_UI_DATA", 'tha_fge')
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 2: Did not give required init groupID'
    ; Init Test 3: No name given
    myData = Obj_New("THM_UI_DATA", 1)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 3: Did not give required init name'
    myData = Obj_New("THM_UI_DATA", 1, 12)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 3: Did not give required init name'

     ; Init Test 4: Invalid init parent ID value given
    myData = Obj_New("THM_UI_DATA", 'tha_fge', 12, 'a')
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 4: Invalid group ID'

        ; Init Test 5: Invalid init file name given, group name
    myData = Obj_New("THM_UI_DATA",'tha_fge', 12, 1, FileName=1)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 5: Invalid group name'
    myData = Obj_New("THM_UI_DATA",'tha_fge', 12, 1, GroupName=1)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 5: Invalid file name'

    ; test that valid parameters are handled correctly at initialization

    ; Init Test 6: Valid values given
    myData = Obj_New("THM_UI_DATA", 'tha_fge', 12, 1)
    IF NOT Obj_Valid(myData) THEN Print, 'Error Init Test 6: Valid init failed'

    ; Init Test 7: Valid values given plus other optional valid params
    myData = Obj_New("THM_UI_DATA", 'tha_fge_x',12, 1, GroupName='tha_fge', $
      fileName='test.dat')
    IF NOT Obj_Valid(myData) THEN Print, 'Error Init Test 7: Valid init failed'


    Print, '**********  End of Initialization tests  ***************'
    Print, ' '
    Print, '**********  Start of GetProperty tests  ***************'

        ; GetProperty Test 1: default values used for pid and filename

    myData = Obj_New("THM_UI_DATA",'tha_fge', 12, 1)
    myData->GetProperty, Name=n, ID=id, GroupID=gid, GroupName=gn, FileName=fn
    IF n NE 'tha_fge' THEN Print, 'Error Get Test 1a: Invalid name returned'
    IF id NE 12 THEN Print, 'Error Get Test 1b: Invalid id returned'
    IF gid NE 1 THEN Print, 'Error Get Test 1c: Invalid groupid returned'
    IF gn NE n THEN Print, 'Error Get Test 1d: Invalid groupname returned'
    IF fn NE ' ' THEN Print, 'Error Get Test 1e: Invalid filename returned'

        ; GetProperty Test 2: pid, filename are provided

    myData = Obj_New("THM_UI_DATA", 'tha_fge_x', 12, 1, GroupName='tha_fge',  FileName='test.dat')
    myData->GetProperty, Name=n, id=id, GroupID=gid, GroupName=gn, fileName=fn
    IF n NE 'tha_fge_x' THEN Print, 'Error Get Test 2a: Invalid name returned'
    IF id NE 12 THEN Print, 'Error Get Test 2b: Invalid id returned'
    IF gid NE 1 THEN Print, 'Error Get Test 2c: Invalid groupid returned'
    IF gn NE 'tha_fge' THEN Print, 'Error Get Test 2d: Invalid group name returned'
    IF fn NE 'test.dat' THEN Print, 'Error Get Test 2e: Invalid filename returned'

    Print, '**********  End of GetProperty tests  ***************'
    Print, ' '

    Obj_Destroy, myData
END
