;+ 
;NAME:
; thm_ui_test_highlight_settings
;
;PURPOSE:
; test script for highlight settings object
;
;CALLING SEQUENCE:
; thm_ui_test_highlight_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_highlight_settings.pro $
;
;-----------------------------------------------------------------------------------
; markVertical      flag to mark vertical
; lineStyle         line style object
; symbol            symbol object 
; placement         droplistfor placement of symbol
;                     top, bottom, both top and bottom
; labelWith         0=No label, 1=X value, 2=Y value
; format            numeric format for label 
; markWhenY         flag to mark y value 
; whenYEquals       y value to mark
; markEvery         flag to mark every y value ..
; whenEveryEquals  y value to mark
; setBackground     flag to set background
; backgroundWhenY   droplist of logical operators
; backgroundYValue  value for logical operator
; backgroundColor   r,g,b color for background
;
;METHODS:
; SetProperty   procedure to set keywords 
; GetProperty   procedure to get keywords 
; GetAll        returns the entire structure
; GetPlacements returns a string array with placement options
; GetPlacement  return a string with placement option of given index
; GetOperators  returns a string array with operator options ['<','<=', ...]
; GetOperator   return a string with placement option of given index

Pro thm_ui_test_highlight_settings

    ;initializations

    Print, '********** Starting highlight settings tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myLS = Obj_New('THM_UI_HIGHLIGHT_SETTINGS')
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 1: Did not initialize'
   
    ; Test 1: Test Get for all     
    myLS->GetProperty, MarkVertical=mv, LineStyle=ls, Symbol=s, Placement=p, $
      LabelWith=lw, Format=f,  MarkWhenY=my, WhenYEquals=wy, MarkEvery=me, $
      WhenEveryEquals=we, SetBackground=sb, BackgroundWhenY=swy, BackgroundYValue=bv, $ 
      BackgroundColor=bc
    IF mv NE 0 THEN Print, 'Error Test 1a: Did not return mark vertical'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 1b: Did not return line style'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 1c: Did not return symbol'
    IF p NE 0 THEN Print, 'Error Test 1d: Did not return correct placement'
    IF lw NE 0 THEN Print, 'Error Test 1e: Did not return correct label with'
    IF f NE 0 THEN Print, 'Error Test 1f: Did not return format'
    IF my NE 0 THEN Print, 'Error Test 1g: Did not return mark when y'
    IF wy NE 0 THEN Print, 'Error Test 1h: Did not return when y equals'
    IF me NE 0 THEN Print, 'Error Test 1i: Did not return mark every'
    IF we NE 0 THEN Print, 'Error Test 1j: Did not return when every equals'
    IF sb NE 0 THEN Print, 'Error Test 1k: Did not return set background'
    IF swy NE 0 THEN Print, 'Error Test 1l: Did not return background when y'
    IF bv NE 0 THEN Print, 'Error Test 1l: Did not return background y value'
    IF bc[2] NE 0 THEN Print, 'Error Test 1l: Did not return background color'
    IF f NE 0 THEN Print, 'Error Test 1l: Did not return format'

     ; Test 2: Initialize with parameters 
    blueL=Obj_New('THM_UI_LINE_STYLE', Color=[0,0,255])
    redS=Obj_New('THM_UI_SYMBOL', Color=[255,0,0])
    myLS = Obj_New("THM_UI_HIGHLIGHT_SETTINGS", /MarkVertical, LineStyle=blueL, $
      Symbol=redS, Placement=3, LabelWith=1, Format=1,  /MarkWhenY, WhenYEquals=1, $
      /MarkEvery, WhenEveryEquals=5, /SetBackground, BackgroundWhenY=1, $
      BackgroundYValue=6, BackgroundColor=[0,255,0])
    myLS->GetProperty, MarkVertical=mv, LineStyle=ls, Symbol=s, Placement=p, $
      LabelWith=lw, Format=f,  MarkWhenY=my, WhenYEquals=wy, MarkEvery=me, $
      WhenEveryEquals=we, SetBackground=sb, BackgroundWhenY=swy, BackgroundYValue=bv, $
      BackgroundColor=bc
    IF mv NE 1 THEN Print, 'Error Test 2a: Did not return mark vertical'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 2b: Did not return line style'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 2c: Did not return symbol'
    IF p NE 3 THEN Print, 'Error Test 2d: Did not return correct placement'
    IF lw NE 1 THEN Print, 'Error Test 2e: Did not return correct label with'
    IF f NE 1 THEN Print, 'Error Test 2f: Did not return format'
    IF my NE 1 THEN Print, 'Error Test 2g: Did not return mark when y'
    IF wy NE 1 THEN Print, 'Error Test 2h: Did not return when y equals'
    IF me NE 1 THEN Print, 'Error Test 2i: Did not return mark every'
    IF we NE 5 THEN Print, 'Error Test 2j: Did not return when every equals'
    IF sb NE 1 THEN Print, 'Error Test 2k: Did not return set background'
    IF swy NE 1 THEN Print, 'Error Test 2l: Did not return background when y'
    IF bv NE 6 THEN Print, 'Error Test 2l: Did not return background y value'
    IF bc[1] NE 255 THEN Print, 'Error Test 2l: Did not return background color'
    ls->GetProperty, Color=c
    IF c[2] NE 255 THEN Print, 'Error Test 2m: Did not return line style color'
    s->GetProperty, Color=c
    IF c[0] NE 255 THEN Print, 'Error Test 2n: Did not return symbol color'

    ; Test 5: Test Set for all
    purpleL=Obj_New('THM_UI_LINE_STYLE', Color=[255,0,0])
    yellowS=Obj_New('THM_UI_SYMBOL', Color=[0,0,255])
    
    myLS->SetProperty, MarkVertical=0, LineStyle=purpleL, Symbol=yellowS, Placement=2, $
      LabelWith=0, Format=0,  MarkWhenY=0, WhenYEquals=0, MarkEvery=0, $
        WhenEveryEquals=6, SetBackground=0, BackgroundWhenY=0, BackgroundYValue=7, $ 
        BackgroundColor=[0,255,0]
    myLS->GetProperty, MarkVertical=mv, LineStyle=ls, Symbol=s, Placement=p, $
      LabelWith=lw, Format=f,  MarkWhenY=my, WhenYEquals=wy, MarkEvery=me, $
      WhenEveryEquals=we, SetBackground=sb, BackgroundWhenY=swy, BackgroundYValue=bv, $ 
      BackgroundColor=bc
    IF mv NE 0 THEN Print, 'Error Test 5a: Did not return mark vertical'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 5b: Did not return line style'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 5c: Did not return symbol'
    IF p NE 2 THEN Print, 'Error Test 5d: Did not return correct placement'
    IF lw NE 0 THEN Print, 'Error Test 5e: Did not return correct label with'
    IF f NE 0 THEN Print, 'Error Test 5f: Did not return format'
    IF my NE 0 THEN Print, 'Error Test 5g: Did not return mark when y'
    IF wy NE 0 THEN Print, 'Error Test 5h: Did not return when y equals'
    IF me NE 0 THEN Print, 'Error Test 5i: Did not return mark every'
    IF we NE 6 THEN Print, 'Error Test 5j: Did not return when every equals'
    IF sb NE 0 THEN Print, 'Error Test 5k: Did not return set background'
    IF swy NE 0 THEN Print, 'Error Test 5l: Did not return background when y'
    IF bv NE 7 THEN Print, 'Error Test 5m: Did not return background y value'
    IF bc[1] NE 255 THEN Print, 'Error Test 5n: Did not return background color'
    IF Obj_Valid(ls) THEN ls->GetProperty, Color=c
    IF c[0] NE 255 THEN Print, 'Error Test 5o: Did not return line style color'
    IF Obj_Valid(s) THEN s->GetProperty, Color=c
    IF c[2] NE 255 THEN Print, 'Error Test 5p: Did not return symbol color'

    ; Test 6: Test Get Operators     
    o=myLS->GetOperators()
    IF N_Elements(o) NE 4 THEN Print, 'Error Test 6: Did not return right num elements of color tables'
    Print, 'Operators = ', o

    ; Test 7: Test Get Placements     
    pls=myLS->GetPlacements()
    IF N_Elements(pls) NE 3 THEN Print, 'Error Test 7: Wrong number elements for placements'
    Print, 'Placements = ', pls

    ; Test 8: Test Get Operator
    name=myLS->GetOperator(3)
    IF name NE '>=' THEN Print, 'Error Test 8: Wrong operator name'
    ; Test 8: Test Get Operator
    name=myLS->GetOperator()
    IF name NE 0 THEN Print, 'Error Test 8: Wrong operator name'
    ; Test 8: Test Get Operator
    name=myLS->GetOperator(12)
    IF name NE 0 THEN Print, 'Error Test 8: Wrong operator name'
    ; Test 8: Test Get Operator
    name=myLS->GetOperator('a')
    IF name NE 0 THEN Print, 'Error Test 8: Wrong operator name'

    ; Test 9: Test Get placement
    name=myLS->GetPlacement(1)
    IF name NE 'Bottom' THEN Print, 'Error Test 9: Wrong placement name'
    ; Test 9: Test Get placement
    name=myLS->GetPlacement()
    IF name NE 0 THEN Print, 'Error Test 9: Wrong placement name'
    ; Test 9: Test Get placement
    name=myLS->GetPlacement(12)
    IF name NE 0 THEN Print, 'Error Test 9: Wrong placement name'
    ; Test 9: Test Get placement
    name=myLS->GetPlacement('a')
    IF name NE 0 THEN Print, 'Error Test 9: Wrong placement name'

    ; Test 10: Test Get All
    obj=myLS->GetAll()
    IF NOT Obj_Valid(obj) THEN Print, 'Error Test 10: Invalid object from obj'

   Print, '********** Starting highlight settings tests  ***************'

Obj_Destroy, blueL
Obj_Destroy, redS
Obj_Destroy, purpleL
Obj_Destroy, yellowS
Obj_Destroy, myLS
    
END

