;+ 
;NAME:
; thm_ui_test_line_settings
;
;PURPOSE:
; test script for line settings object
;
;CALLING SEQUENCE:
; thm_ui_test_line_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_line_settings.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_line_settings

    ;initializations

    Print, '**********  Starting line settings tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myLS = Obj_New("THM_UI_LINE_SETTINGS")
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 1: Did not initialize'
   
    ; Test 1: Test Get for all     
    myLS->GetProperty, MirrorLine=ml, LineStyle=ls, DrawBetweenPts=dbp, SeparatedBy=sb, $
        SeparatedUnits=su, Symbol=s, PlotPoints=pp, EveryOther=eo, PositiveEndPt=pep, $
        NegativeEndPt=nep, PositiveEndRel=pepr, NegativeEndRel=nepr, BarLine=bl, $
        MarkSymbol=ms
    IF ml NE 0 THEN Print, 'Error Test 1a: Did not return mirrorline state'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 1b: Did not return line style object'
    IF dbp NE 0 THEN Print, 'Error Test 1c: Did not return draw between state'
    IF sb NE 0 THEN Print, 'Error Test 1d: Did not return correct separated by value'
    IF su NE 0 THEN Print, 'Error Test 1e: Did not return correct separated units'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 1f: Did not return symbol object'
    IF pp NE 0 THEN Print, 'Error Test 1g: Did not return correct plot points'
    IF eo NE 1 THEN Print, 'Error Test 1h: Did not return correct every other value'
    IF pep NE '' THEN Print, 'Error Test 1i: Did not return correct pos. end pt.'
    IF nep NE '' THEN Print, 'Error Test 1j: Did not return correct neg. end pt.'
    IF nepr NE 0 THEN Print, 'Error Test 1k: Did not return correct neg. end pt. rel.'
    IF pepr NE 0 THEN Print, 'Error Test 1l: Did not return correct pos. end pt. rel.'
    IF NOT Obj_Valid(bl) THEN Print, 'Error Test 1m: Did not return bar line object'
    IF NOT Obj_Valid(ms) THEN Print, 'Error Test 1n: Did not return marker symbol'

    
    ; Test 2: Test Get for all for symbol object     
    s->GetProperty, Name=n, ID=id, Show=sf, Color=c, Fill=f, Size=si
    IF n NE 'Plus Sign' THEN Print, 'Error Test 23: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 2: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 2: Did not return correct show flag'
    IF c NE 'black' THEN Print, 'Error Test 2: Did not return correct color'
    IF f NE 0 THEN Print, 'Error Test 2: Did not return correct fill flag'
    IF si NE 2 THEN Print, 'Error Test 2: Did not return size'
 
    ; Test 3: Test Get for all for line style object     
    ls->GetProperty, Name=n, ID=id, Show=sf, Color=c, Thickness=t, Symbol=s
    IF n NE 'Solid' THEN Print, 'Error Test 3: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 3: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 3: Did not return correct show flag'
    IF c NE 'black' THEN Print, 'Error Test 3: Did not return correct color'
    IF t NE 1 THEN Print, 'Error Test 3: Did not return correct thickness'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 3: Did not return symbol'

    ; Test 4: Test Get for all for bar line object     
    bl->GetProperty, Name=n, ID=id, Show=sf, Color=c, Thickness=t, Symbol=s
    IF n NE 'Solid' THEN Print, 'Error Test 4: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 4: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 4: Did not return correct show flag'
    IF c NE 'black' THEN Print, 'Error Test 4: Did not return correct color'
    IF t NE 1 THEN Print, 'Error Test 4: Did not return correct thickness'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 4: Did not return symbol'

   ; Test 5: Test Get for all for end mark symbol object     
    ms->GetProperty, Name=n, ID=id, Show=sf, Color=c, Fill=f, Size=si
    IF n NE 'Plus Sign' THEN Print, 'Error Test 5: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 5: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 5: Did not return correct show flag'
    IF c NE 'black' THEN Print, 'Error Test 5: Did not return correct color'
    IF f NE 0 THEN Print, 'Error Test 5: Did not return correct fill flag'
    IF si NE 2 THEN Print, 'Error Test 5: Did not return size'

    Obj_Destroy, s
    Obj_Destroy, ms
    Obj_Destroy, ls
    Obj_Destroy, bl
    Obj_Destroy, myLS
  
    blueL = Obj_New("THM_UI_LINE_STYLE", Color='blue')
    greenL = Obj_New("THM_UI_LINE_STYLE", Color='green')
    redS = Obj_New("THM_UI_SYMBOL", Color='red')
    yellowS = Obj_New("THM_UI_SYMBOL", Color='yellow')

    ; Test 6: Initialize with parameters    
    myLS = Obj_New("THM_UI_LINE_SETTINGS", MirrorLine=1, LineStyle=blueL, SeparatedBy=10, $
                   Symbol=redS, BarLine=greenL, MarkSymbol=yellowS)
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 6: Did not initialize'
    myLS->GetProperty, MirrorLine=ml, LineStyle=ls, DrawBetweenPts=dbp, SeparatedBy=sb, $
        SeparatedUnits=su, Symbol=s, PlotPoints=pp, EveryOther=eo, PositiveEndPt=pep, $
        NegativeEndPt=nep, PositiveEndRel=pepr, NegativeEndRel=nepr, BarLine=bl, $
        MarkSymbol=ms
    IF ml NE 1 THEN Print, 'Error Test 6: Init with parameters, mirror'
    IF sb NE 10 THEN Print, 'Error Test 6: Init with parameters, separated'
    ls->GetProperty, Color=c
    IF c NE 'blue' THEN Print, 'Error Test 6: Init with parameters, line style'
    bl->GetProperty, Color=c
    IF c NE 'green' THEN Print, 'Error Test 6: Init with parameters, bar line'
    s->GetProperty, Color=c
    IF c NE 'red' THEN Print, 'Error Test 6: Init with parameters, symbol'
    ms->GetProperty, Color=c
    IF c NE 'yellow' THEN Print, 'Error Test 6: Init with parameters, mark symbol'
  
    Obj_Destroy, s
    Obj_Destroy, ms
    Obj_Destroy, ls
    Obj_Destroy, bl
    Obj_Destroy, myLS

    ; Test 7: Test Set Line Settings   
    myLS = Obj_New("THM_UI_LINE_SETTINGS")
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test t: Did not initialize'
    st=''
    myLS->SetProperty, MirrorLine=1, LineStyle=blueL, DrawBetweenPts=1, SeparatedBy=2, $
        SeparatedUnits=3, Symbol=redS, PlotPoints=1, EveryOther=5, PositiveEndPt='a', $
        NegativeEndPt='b', PositiveEndRel=1, NegativeEndRel=1, BarLine=greenL, $
        MarkSymbol=yellowS
    myLS->GetProperty, MirrorLine=ml, LineStyle=ls, DrawBetweenPts=dbp, SeparatedBy=sb, $
        SeparatedUnits=su, Symbol=s, PlotPoints=pp, EveryOther=eo, PositiveEndPt=pep, $
        NegativeEndPt=nep, PositiveEndRel=pepr, NegativeEndRel=nepr, BarLine=bl, $
        MarkSymbol=ms
    IF ml NE 1 THEN Print, 'Error Test 7: Did not return mirrorline state'
    IF dbp NE 1 THEN Print, 'Error Test 7: Did not return draw between state'
    IF sb NE 2 THEN Print, 'Error Test 7: Did not return correct separated by value'
    IF su NE 3 THEN Print, 'Error Test 7: Did not return correct separated units'
    IF pp NE 1 THEN Print, 'Error Test 7: Did not return correct plot points'
    IF eo NE 5 THEN Print, 'Error Test 7: Did not return correct every other value'
    IF pep NE 'a' THEN Print, 'Error Test 7: Did not return correct pos. end pt.'
    IF nep NE 'b' THEN Print, 'Error Test 7: Did not return correct neg. end pt.'
    IF nepr NE 1 THEN Print, 'Error Test 7: Did not return correct neg. end pt. rel.'
    IF pepr NE 1 THEN Print, 'Error Test 7: Did not return correct pos. end pt. rel.'

    ; Test 8: Test GetUnits    
    units = myLS->GetUnits()
    IF N_Elements(units) NE 6 THEN Print, 'Error Test 8: Wrong number of units'
    Print, 'Units = ', units
    
    ; Test 9: Test GetUnit    
    u = myLS->GetUnit()
    IF u NE 0 THEN Print, 'Test 9: Called get unit with no index, see what happens'
    u = myLS->GetUnit(2)
    IF u NE 'hours' THEN Print, 'Test 9: Wrong unit name returned'
    u = myLS->GetUnit(9)
    IF u NE 0 THEN Print, 'Test 9: Called get unit with wrong index, see what happens'
    u = myLS->GetUnit('a')
    IF u NE 0 THEN Print, 'Test 9: Called get unit with alpha index, see what happens'

    ; Test 10 Test Get Plot Options    
    po = myLS->GetPlotOptions()
    IF N_Elements(po) NE 6 THEN Print, 'Error Test 10: Wrong number of plot options'
    
    ; Test 11: Test Get plot options    
    u = myLS->GetPlotOption()
    IF u NE 0 THEN Print, 'Test 11: Called get plot options with no index, see what happens'
    u = myLS->GetPlotOption(5)
    IF u NE 'Every' THEN Print, 'Test 11: Wrong plot option name returned'
    u = myLS->GetPlotOption(9)
    IF u NE 0 THEN Print, 'Test 11: Called get plot option with wrong index, see what happens'
    u = myLS->GetPlotOption()
    IF u NE 0 THEN Print, 'Test 11: Called get plot option with alpha index, see what happens'

    me = myLS->GetAll()
    IF NOT Obj_Valid(me) THEN Print, 'Test 12: Called get all, see what is returned'

    Print, '**********  Done with line settings test  ***************'

    Obj_Destroy, s
    Obj_Destroy, ms
    Obj_Destroy, ls
    Obj_Destroy, bl
    Obj_Destroy, blueL
    Obj_Destroy, greenL
    Obj_Destroy, redS
    Obj_Destroy, yellowS
    Obj_Destroy, myLS
    
END

