;+ 
;NAME:
; thm_ui_test_line_style
;
;PURPOSE:
; test script for line style object
;
;CALLING SEQUENCE:
; thm_ui_test_line_style
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_line_style.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_line_style

    ;initializations

    Print, '**********  Starting line style tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myLS = Obj_New("THM_UI_LINE_STYLE")
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 1: Did not initialize'
   
    ; Test 2: Test Get for all     
    myLS->GetProperty, Name=n, ID=id, Show=sf, Color=c, Thickness=t, Symbol=s
    IF n NE 'Solid' THEN Print, 'Error Test 2a: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 2b: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 2c: Did not return correct show flag'
    IF c[1] NE 0 THEN Print, 'Error Test 2e: Did not return correct color'
    IF t NE 1 THEN Print, 'Error Test 2f: Did not return correct thickness'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 2h: Did not return symbol object'

    ; Test 3: Test Get for all for symbol object     
    s->GetProperty, Name=n, ID=id, Show=sf, Color=c, Fill=f, Size=si
    IF n NE 'Plus Sign' THEN Print, 'Error Test 3a: Did not return correct name'
    IF id NE 0 THEN Print, 'Error Test 3b: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 3c: Did not return correct show flag'
    IF c[2] NE 0 THEN Print, 'Error Test 3d: Did not return correct color'
    IF f NE 0 THEN Print, 'Error Test 3e: Did not return correct fill flag'
    IF si NE 2 THEN Print, 'Error Test 3f: Did not return size'

    Obj_Destroy, myLS
    Obj_Destroy, s
  
    ; Test 4: Test Initialize with valid parameters 
    s = Obj_New("THM_UI_SYMBOL", Name='Diamond', ID=4, Color=[2,2,2], Size=5)   
    myLS = Obj_New("THM_UI_LINE_STYLE", Name='Dashed', ID=2, Show=0, Color=[3,3,3], Thickness=3, Symbol=s)
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 4: Did not initialize with parameters'

    ; Test 5: Test Get for all     
    myLS->GetProperty, Name=n, ID=id, Show=sf, Color=c, Thickness=t, Symbol=s
    IF n NE 'Dashed' THEN Print, 'Error Test 5a: Did not return correct name'
    IF id NE 2 THEN Print, 'Error Test 5b: Did not return correct id'
    IF sf NE 0 THEN Print, 'Error Test 5c: Did not return correct show flag'
    IF c[1] NE 3 THEN Print, 'Error Test 5e: Did not return correct color'
    IF t NE 3 THEN Print, 'Error Test 5f: Did not return correct thickness'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 5g: Did not return symbol object'
    
    ; Test 6: Test Get for all for symbol object     
    s->GetProperty, Name=n, ID=id, Show=sf, Color=c, Fill=f, Size=si
    IF n NE 'Diamond' THEN Print, 'Error Test 6a: Did not return correct name'
    IF id NE 4 THEN Print, 'Error Test 6b: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 6c: Did not return correct show flag'
    IF c[0] NE 2 THEN Print, 'Error Test 6d: Did not return correct color'
    IF f NE 0 THEN Print, 'Error Test 6e: Did not return correct fill flag'
    IF si NE 5 THEN Print, 'Error Test 6f: Did not return correct size'

    ; Test 7: Test Get Line Styles    
    styles = myLS->GetLineStyles()
    IF N_Elements(styles) NE 6 THEN Print, 'Error Test 7: Did not return correct number of styles'
    Print, styles
    
    ; Test 8: Test Get Line Style ID    
    sid = myLS->GetLineStyleId()
    IF sid NE 2 THEN Print, 'Error Test 8a: Did not return correct id'
    sid = myLS->GetLineStyleId(LineStyleName='Dotted')
    IF sid NE 2 THEN Print, 'Error Test 8b: Did not return correct id when given name'
    sid = myLS->GetLineStyleId(LineStyleName=4)
    IF sid NE -1 THEN Print, 'Error Test 8c: Did not correctly return -1 for error'
     
    ; Test 9: Test Get Line Style ID    
    sn = myLS->GetLineStyleName()
    IF sn NE 'Dashed' THEN Print, 'Error Test 9a: Did not return correct style name'
    sn = myLS->GetLineStyleName(LineStyleId=2)
    IF sn NE 'Dotted' THEN Print, 'Error Test 9b: Did not return correct id when given name'
    sn = myLS->GetLineStyleName(LineStyleId='a')
    IF sn NE -1 THEN Print, 'Error Test 9c: Did not correctly return -1 for error'
     
    ; Test 10: Test Set on line symbol    
    s = Obj_New("THM_UI_SYMBOL", Name='Plus Sign', ID=7, Color=[100,100,100], Size=11)   
    myLS->SetProperty, Name='Long Dashes', ID=3, Show=0, Color=[200,200,200], Thickness=10, Symbol=s
    myLS->GetProperty, Name=n, ID=id, Show=sf, Color=c, Thickness=t, Symbol=s
    IF n NE 'Long Dashes' THEN Print, 'Error Test 10a: Did not return correct name'
    IF id NE 3 THEN Print, 'Error Test 10b: Did not return correct id'
    IF sf NE 0 THEN Print, 'Error Test 10c: Did not return correct show flag'
    IF c[1] NE 200 THEN Print, 'Error Test 10d: Did not return correct color'
    IF t NE 10 THEN Print, 'Error Test 10e: Did not return correct thickness'
    IF NOT Obj_Valid(s) THEN Print, 'Error Test 10f: Did not return symbol object'

    ; Test 11: Test Set on symbol        
    s->GetProperty, Name=n, ID=id, Show=sf, Color=c, Fill=f, Size=si
    IF n NE 'Plus Sign' THEN Print, 'Error Test 11a: Did not return correct name'
    IF id NE 7 THEN Print, 'Error Test 11b: Did not return correct id'
    IF sf NE 1 THEN Print, 'Error Test 11c: Did not return correct show flag'
    IF c[2] NE 100 THEN Print, 'Error Test 11d: Did not return correct color'
    IF f NE 0 THEN Print, 'Error Test 11e: Did not return correct fill flag'
    IF si NE 11 THEN Print, 'Error Test 11f: Did not return correct size'     
    
    Print, '**********  Done with Line Style object test  ***************'
    Obj_Destroy, myLS
    Obj_Destroy, s
    
END
