;+ 
;NAME:
; thm_ui_test_loaded_data
;
;PURPOSE:
; test for loaded data object
;
;CALLING SEQUENCE:
; thm_ui_test_loaded_data
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_loaded_data.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_loaded_data

    Print, '**********  Starting tests  ***************'

    ; Test 1: load data obj created when no tplot variables available
    ldata = Obj_New("THM_UI_LOADED_DATA")
    IF NOT Obj_Valid(ldata) THEN Print, 'Error Test 1: failed to create load data obj' ELSE $
      Print, 'Test 1: Success creating load data obj with no tplot data present' 
    ; Test 2: create load data with tplot variables
    time_start='2007-3-23/00:00:00'
    time_stop='2007-3-24/00:00:00'
    t0=time_double(time_start)
   t1=time_double(time_stop) 
    dtype='fgm/fge/l1'
    probe=['a','b','c','d','e']
    tv_names = thm_ui_load_data_fn(t0, t1, dtype = dtype, probe = probe)
    ldata = Obj_New("THM_UI_LOADED_DATA")
    IF NOT Obj_Valid(ldata) THEN Print, 'Error Test 2: failed to create load data obj' ELSE $
      Print, 'Test 1: Success creating load data obj with tplot data present' 

    ; Test 2: request names w
    names = ldata->GetAll()
    Print, 'Test 2: Print Names ' 
    Help, names
    Print, names

    ; Test 3: get current group id
    gid = ldata->GetGroupID()
    Print, 'Test 3: Print Group ID (any number GT 0 is OK): ', gid 

    ; Test 4: get current data id
    did = ldata->GetDataID()
    Print, 'Test 4: Print Data ID (OK if GT 0) ', did 
    
    ; Test 5: adding another variable
    result = ldata->Add('tha_state_pos_x')
    IF result NE 1 THEN Print, 'Error Test 5: failed to add variable' ELSE $
    Print, 'Test 5a: Success adding additional variable'
    gid = ldata->GetGroupID()
    Print, 'Test 5b: Print Group ID (should be one plus the previous): ', gid 
    did = ldata->GetDataID()
    Print, 'Test 5c: Print Data ID (should be three plus the previous): ', did 
    
    ; Test6: get array of objects
    da=ldata->GetAllObjects()
    IF N_Elements(da) LE 0 THEN Print, 'Error Test 6: retrieving array of all objects' ELSE $
       Print, 'Test 6: Successfully retrieved all objects, total number is ', N_Elements(da)

stop
    ; Test 7: set active objects
    ldata->SetActive, 'thd_state_spinphase'
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 7a: No active variables returned' ELSE $
      Print, 'Test 7a: Successfully retrieved active data, names are: ', names
    ldata->SetActive, 'thd_state_spinper'
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 7b: No active variables returned' ELSE $
      Print, 'Test 7b: Successfully retrieved active data, names are: ', names

    ; Test 8: clear active objects
    ldata->ClearActive, 'thd_state_spinphase'
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 8a: No active variables returned' ELSE $
      Print, 'Test 8a: Successfully retrieved active data, names are: ', names
    ldata->ClearActive, 'thd_state_spinper'
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 8b: No active variables returned' ELSE $
      Print, 'Test 8b: Successfully retrieved active data, names are: ', names
  
    ; Test 9: clear all active objects
    ldata->SetActive, 'thd_state_spinphase'
    ldata->SetActive, 'thd_state_spinper'
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 9: No active variables returned' ELSE $
      Print, 'Test 9: Successfully retrieved active data, names are: ', names
    ldata->ClearAllActive
    names=ldata->GetActive()
    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 9: No active variables returned' ELSE $
      Print, 'Test 9: Successfully retrieved active data, names are: ', names
    
   ; Test 7: remove object (first, last, and, middle)
;    result = ldata->Remove(da[0])
;    Print, 'Loaded data names - should only be 3'
;    names = ldata->GetAllNames()
;    Print, 'Names ', names
;    result = ldata->Add(data1)
    
    ; Test 8: remove object (first, last, and, middle)
;    result = ldata->Remove(da[1])
;    Print, 'Loaded data names - should only be 3'
;    names = ldata->GetAllNames()
;    Print, names
;    result = ldata->Add(data2)
    
    ; Test 9: remove object (first, last, and, middle)
;    result = ldata->Remove(da[2])
;    Print, 'Loaded data names - should only be 3'
;    names = ldata->GetAllNames()
;    Print, names
;    result = ldata->Add(data3)
    
    ; Test 10: remove object (first, last, and, middle)    
;    result = ldata->Remove(da[3])
;    Print, 'Loaded data names - should only be 3'
;    names = ldata->GetAllNames()
;    Print, names
;    result = ldata->Add(data4)

    ; Test 11: add all
;    result = ldata->Add(data5)
;    result = ldata->Add(data6)
;    result = ldata->Add(data7)
;    result = ldata->Add(data8)
;    IF result EQ 0 THEN Print, 'Error Test 11: failed to add last 4 of 8'
;    names = ldata->GetAllNames() 
;    IF N_Elements(names) EQ 8 THEN Print, 'Error Test 11: failed to get all 8 names'
;    Print, 'all names - should be 8: ', names
    
    ; Test 12: get group id
;    names = ldata->GetGroup(2) 
;    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 12: failed to get group id'
;    Print, 'Should be group 2 names: ', names
;    
    ; Test 13: get groupid -1
;    names = ldata->GetGroup(-1) 
;    IF N_Elements(names) GT 0 THEN Print, 'Error Test 13: failed invalid group id'
;    Print, 'should be no group names: ', names
    
    ; Test 14: get active data
;    names = ldata->GetActive() 
;    IF N_Elements(names) EQ 0 THEN Print, 'Error Test 14: failed to get active data'
;    Print, 'Should be 2 active data names: ', names
    
     Print, '**********  End of tests  ***************'
    
    Obj_Destroy, ldata
END
