;+ 
;NAME:
; thm_ui_marker_data
;
;PURPOSE:
; test script for marker object
;
;CALLING SEQUENCE:
; thm_ui_test_marker
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_marker.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_marker

    ;initializations

    Print, '**********  Starting Initialization tests  ***************'

    ; Test 1: Test Initialization  
  
    myData = Obj_New("THM_UI_MARKER", 1, 2)
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 1: Did not initialize'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, PanelID=pid, TimeRange=tr, Settings=s, filename=fn
       IF n NE '' THEN Print, 'Test 1a: Did not return correct property value'
       IF id NE 1 THEN Print, 'Test 1b: Did not return correct property value'
       IF pid NE 2 THEN Print, 'Test 1c: Did not return correct property value'
       IF NOT Obj_Valid(tr) THEN Print, 'Test 1d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 1e: Did not return correct property value'
       IF fn NE '' THEN Print, 'Test 1f: Did not return correct property value'
    ENDELSE
    
    ; Init Test 2: Initialize, no parameters    
    myData = Obj_New("THM_UI_MARKER")
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 2: Did not give required ids'

    ; Init Test 3: No panel ID   
    myData = Obj_New("THM_UI_MARKER", 1)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 3: Did not give required panelid'
 
    ; Test 4: Test Init with all parameters
    st = '2008-01-02/18:09:57'
    et = '2008-01-09/00:00:00'
    start = Obj_New("THM_UI_TIME", TString=st)
    endt = Obj_New("THM_UI_TIME", TString=et)
    s=Obj_New("THM_UI_MARKER_SETTINGS", FillColor='blue')
    tr=Obj_New("THM_UI_TIME_RANGE", StartTime=start, EndTime=endt)
    myData = Obj_New("THM_UI_MARKER", 4, 12, Name='test', TimeRange=tr, Settings=s, $
      Filename='test.dat')
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 4: Did not initialize'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, PanelID=pid, TimeRange=tr, Settings=s, filename=fn
       IF n NE 'test' THEN Print, 'Test 4a: Did not return correct property value'
       IF id NE 4 THEN Print, 'Test 4b: Did not return correct property value'
       IF pid NE 12 THEN Print, 'Test 4c: Did not return correct property value'
       IF NOT Obj_Valid(tr) THEN Print, 'Test 4d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 4e: Did not return correct property value'
       IF fn NE 'test.dat' THEN Print, 'Test 4f: Did not return correct property value'
       s->GetProperty, FillColor=fc
       IF fc NE 'blue' THEN  Print, 'Test 4g: Did not return correct property value'
       tr->GetProperty, StartTime=st
       st->GetProperty, TString=ts
       IF ts NE '2008-01-02/18:09:57' THEN Print, 'Test 4h: Did not return correct property value'
    ENDELSE

    ; Test 5: Set Property
    st = '2008-09-02/18:09:57'
    et = '2008-09-09/00:00:00'
    start = Obj_New("THM_UI_TIME", TString=st)
    endt = Obj_New("THM_UI_TIME", TString=et)
    s=Obj_New("THM_UI_MARKER_SETTINGS", FillColor='green')
    tr=Obj_New("THM_UI_TIME_RANGE", StartTime=start, EndTime=endt)
    myData->SetProperty, Id=6, PanelID=3, Name='hi', TimeRange=tr, Settings=s, $
      Filename='test2.dat'
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 5: Invalid Object'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, PanelID=pid, TimeRange=tr, Settings=s, filename=fn
       IF n NE 'hi' THEN Print, 'Test 5a: Did not return correct property value'
       IF id NE 6 THEN Print, 'Test 5b: Did not return correct property value'
       IF pid NE 3 THEN Print, 'Test 5c: Did not return correct property value'
       IF NOT Obj_Valid(tr) THEN Print, 'Test 5d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 5e: Did not return correct property value'
       IF fn NE 'test2.dat' THEN Print, 'Test 5f: Did not return correct property value'
       s->GetProperty, FillColor=fc
       IF fc NE 'green' THEN  Print, 'Test 5g: Did not return correct property value'
       tr->GetProperty, StartTime=st
       st->GetProperty, TString=ts
       IF ts NE '2008-09-02/18:09:57' THEN Print, 'Test 5h: Did not return correct property value'
    ENDELSE

        
    Print, '**********  End of marker tests  ***************'
    Print, ' '

    Obj_Destroy, myData
END
