;+ 
;NAME:
; thm_ui_test_marker_settings
;
;PURPOSE:
; test script for marker settings object
;
;CALLING SEQUENCE:
; thm_ui_test_marker_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_marker_settings1.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_marker_settings1

    ;initializations

    Print, '**********  Starting marker settings tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myLS = Obj_New("THM_UI_MARKER_SETTINGS")
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 1: Did not initialize'
   
    ; Test 1: Test Get for all     
    myLS->GetProperty, Label=l, IsTime=it, StartAt=sa, EndAt=ea, VertPlacement=vp, $
        FillColor=fc, LineStyle=ls, DrawOpaque=dp
    IF l NE '' THEN Print, 'Error Test 1a: Did not return label'
    IF it NE 0 THEN Print, 'Error Test 1b: Did not return is time'
    IF sa NE '' THEN Print, 'Error Test 1c: Did not return start at'
    IF ea NE '' THEN Print, 'Error Test 1d: Did not return end at'
    IF vp NE 0 THEN Print, 'Error Test 1e: Did not return vertical placement'
    IF fc[1] NE 200 THEN Print, 'Error Test 1g: Did not return fill color'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 1f: Did not return line style object'
    IF dp NE 0 THEN Print, 'Error Test 1h: Did not return draw opaque'
    ls->GetProperty, Color=c
    IF c[0] NE 255 THEN Print, 'Error Test 1i: Did not return line style color'
    
    ; Test 3: Test Set for all for symbol object 
    blueL = Obj_New("THM_UI_LINE_STYLE", Color=[0,0,255])
    myLS->SetProperty, Label='test', IsTime=1, StartAt='hi', EndAt='there', VertPlacement=2, $
        FillColor=[0,255,0], LineStyle=blueL, DrawOpaque=1
    myLS->GetProperty, Label=l, IsTime=it, StartAt=sa, EndAt=ea, VertPlacement=vp, $
        FillColor=fc, LineStyle=ls, DrawOpaque=dp
    IF l NE 'test' THEN Print, 'Error Test 2a: Did not return label'
    IF it NE 1 THEN Print, 'Error Test 2b: Did not return is time'
    IF sa NE 'hi' THEN Print, 'Error Test 2c: Did not return start at'
    IF ea NE 'there' THEN Print, 'Error Test 2d: Did not return end at'
    IF vp NE 2 THEN Print, 'Error Test 2e: Did not return vertical placement'
    IF fc[1] NE 255 THEN Print, 'Error Test 2g: Did not return fill color'
    IF NOT Obj_Valid(ls) THEN Print, 'Error Test 2f: Did not return line style object'
    IF dp NE 1 THEN Print, 'Error Test 2h: Did not return draw opaque'
    ls->GetProperty, Color=c
    IF c[2] NE 255 THEN Print, 'Error Test 2i: Did not return line style color'
     
    names=myLS->GetPlacements()
    IF N_Elements(names) NE 7 THEN Print, '3a: Did not return right number of placements'
    Print, names

    name=myLS->GetPlacement(2)
    IF name NE 'Above Middle' THEN Print, '3a: Did not return right placement name'    
   
    Obj_Destroy, blueL
    Obj_Destroy, myLS
  
    Print, '**********  Done with marker settings test  ***************'
    
END

