;+ 
;NAME:
; thm_ui_test_page_settings
;
;PURPOSE:
; test script for page settings object
;
;CALLING SEQUENCE:
; thm_ui_test_page_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_page_settings.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_page_settings

    ;initializations

    Print, '**********  Starting page setting tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myGS = Obj_New('THM_UI_PAGE_SETTINGS')
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 1: Did not initialize'
    myGS->Save
    
    ; Test  2: Error Test Get for all     
    myGS->GetProperty, $
      Title=title,                         $ ; text object for title
      Labels=labels,                       $ ; text object for labels
      Variables=variables,                 $ ; text object for variables
      Footer=footer,                       $ ; text object for footer
      Marker=marker,                       $ ; text object for markers  
      Token=token,                         $ ; droplist values for time format
      DefMarkerTitle=defmarkertitle,       $ ; default value for marker titles
      maintainRead=maintainRead,           $ ; flag to maintain readability
      ShowTraceNames=showtracenames,       $ ; flag to display trace names
      PanelSpacing=panelspacing,           $ ; spacing between pane
      ColPerPage=colperpage,               $ ; number of columns per page
      leftLabelMargin=leftlabelmargin,     $ ; margin size for left labels
      HeightProp=heightprop,               $ ; flag to maintain prop. heights 
      GutterWidth=gutterwidth,             $ ; width size for gutter
      DisplayOnScreen=displayonscreen,     $ ; flag to display on screen
      AltTopBottom=alttopbottom,           $ ; flag to alternate top and bottom
      OffsetFirstPage=offsetfirstpage,     $ ; flag to offset first page
      Orientation=orientation,             $ ; 0=portrait, 1=landscape
      BackgroundColor=backgroundcolor,     $ ; name of background color
      LeftPrintMargin=leftprintmargin,     $ ; size of left print margin (in.)
      RightPrintMargin=rightprintmargin,   $ ; size of right print margin (in.)
      TopPrintMargin=topprintmargin,       $ ; size of top print margin (in.)
      BottomPrintMargin=bottomprintmargin, $ ; size of bottom print margin (in.)
      OverlapMajorTicks=overlapmajorticks, $ ; flag to overlap major ticks
      ShowValues=showvalues,               $ ; set this flag to show values
      CloserThanValue=closerthanvalue,     $ ; show data if closer than this
      CloserThanUnits=closerthanunits,     $ ; closer units (hr,min,sec,day)
      UseSameYRange=usesameyrange,         $ ; flag to use the same y range 
      NumMajorTicks=nummajorticks,         $ ; number of major ticks to use
      NumMinorTicks=numminorticks,         $ ; number of minor ticks to use
      SkipBlanks=skipblanks                  ; flag to skip blanks

   IF NOT Obj_Valid(title) THEN Print, 'Error Test 2a: getting properties'
   IF NOT Obj_Valid(labels) THEN Print, 'Error Test 2b: getting properties'
   IF NOT Obj_Valid(variables) THEN Print, 'Error Test 2c: getting properties'
   IF NOT Obj_Valid(footer) THEN Print, 'Error Test 2d: etting properties'
   IF NOT Obj_Valid(marker) THEN Print, 'Error Test 2e: getting properties' 
   IF token NE 0 THEN Print, 'Error Test 2f: getting properties'
   IF defmarkertitle NE '' THEN Print, 'Error Test 2h: getting properties'
   IF maintainread NE 1 THEN Print, 'Error Test 2i: getting properties'
   IF showtracenames NE 0 THEN Print, 'Error Test 2j: getting properties'
   IF panelspacing NE 14 THEN Print, 'Error Test 2k: getting properties'
   IF colperpage NE 1 THEN Print, 'Error Test 2l: getting properties'
   IF leftlabelmargin NE 0 THEN Print, 'Error Test 2m: getting properties'
   IF heightprop NE 1 THEN Print, 'Error Test 2n: getting properties'
   IF gutterwidth NE 50 THEN Print, 'Error Test 2o: getting properties'
   IF displayonscreen NE 1 THEN Print, 'Error Test 2p: getting properties'
   IF alttopbottom NE 0 THEN Print, 'Error Test 2q: getting properties'
   IF offsetfirstpage NE 0 THEN Print, 'Error Test 2r: getting properties'
   IF orientation NE 0 THEN Print, 'Error Test 2s: getting properties'
   IF backgroundcolor[0] NE 0 THEN Print, 'Error Test 2t: getting properties'
   IF leftprintmargin NE 0 THEN Print, 'Error Test 2v: getting properties'
   IF rightprintmargin NE 0 THEN Print, 'Error Test 2x: getting properties'
   IF topprintmargin NE 0 THEN Print, 'Error Test 2y: getting properties'
   IF bottomprintmargin NE 0 THEN Print, 'Error Test 2z: getting properties'
   IF overlapmajorticks NE 0 THEN Print, 'Error Test 2aa: getting properties'
   IF showvalues NE 0 THEN Print, 'Error Test 2bb: getting properties' 
   IF closerthanvalue NE 0 THEN Print, 'Error Test 2cc: getting properties' 
   IF closerthanunits NE 0 THEN Print, 'Error Test 2dd: getting properties'
   IF usesameyrange NE 1 THEN Print, 'Error Test 2ff: getting properties'
   IF nummajorticks NE 5 THEN Print, 'Error Test 2gg: getting properties'
   IF numminorticks NE 12 THEN Print, 'Error Test 2hh: getting properties'
   IF skipblanks NE 0 THEN Print, 'Error Test 2ii: getting properties'
   IF NOT Obj_Valid(title) THEN BEGIN
      Print, 'Error Test 2a: getting properties'
   ENDIF ELSE BEGIN
      title->GetProperty, Value=v, Color=c
      IF c[1] NE 0 THEN Print, 'Error Test 2jj: getting properties'
      IF v NE '%name' THEN Print, 'Error Test 2kk: getting properties'
   ENDELSE
   IF NOT Obj_Valid(labels) THEN BEGIN
      Print, 'Error Test 2a: getting properties'
   ENDIF ELSE BEGIN
      title->GetProperty, Value=v, Color=c
      IF c[2] NE 0 THEN Print, 'Error Test 2jj: getting properties'
      IF v NE '%name' THEN Print, 'Error Test 2kk: getting properties'
   ENDELSE
   IF NOT Obj_Valid(variables) THEN BEGIN
      Print, 'Error Test 2a: getting properties'
   ENDIF ELSE BEGIN
      title->GetProperty, Value=v, Color=c
      IF c[0] NE 0 THEN Print, 'Error Test 2jj: getting properties'
      IF v NE '%name' THEN Print, 'Error Test 2kk: getting properties'
   ENDELSE
   IF NOT Obj_Valid(footer) THEN BEGIN
      Print, 'Error Test 2a: getting properties'
   ENDIF ELSE BEGIN
      title->GetProperty, Value=v, Color=c
      IF c[1] NE 0 THEN Print, 'Error Test 2jj: getting properties'
      IF v NE '%name' THEN Print, 'Error Test 2kk: getting properties'
   ENDELSE
   IF NOT Obj_Valid(marker) THEN BEGIN
      Print, 'Error Test 2a: getting properties'
   ENDIF ELSE BEGIN
      title->GetProperty, Value=v, Color=c
      IF c[2] NE 0 THEN Print, 'Error Test 2jj: getting properties'
      IF v NE '%name' THEN Print, 'Error Test 2kk: getting properties'
   ENDELSE
   
      ; Test  3: Error Test Get for all  
    greenL=Obj_New('THM_UI_TEXT', Value='test')
    purpleL=Obj_New('THM_UI_TEXT', Value='hi', Color=[1,2,3])
    myGS->SetProperty, $
      Title=greenL,                         $ ; text object for title
      Labels=purpleL,                       $ ; text object for labels
      Variables=greenL,                 $ ; text object for variables
      Footer=purpleL,                       $ ; text object for footer
      Marker=greenL,                       $ ; text object for markers  
      Token=2,                         $ ; droplist values for time format
      DefMarkerTitle='test',       $ ; default value for marker titles
      maintainRead=0,           $ ; flag to maintain readability
      ShowTraceNames=1,       $ ; flag to display trace names
      PanelSpacing=12,           $ ; spacing between pane
      ColPerPage=2,               $ ; number of columns per page
      leftLabelMargin=25,     $ ; margin size for left labels
      HeightProp=0,               $ ; flag to maintain prop. heights 
      GutterWidth=15,             $ ; width size for gutter
      DisplayOnScreen=0,     $ ; flag to display on screen
      AltTopBottom=1,           $ ; flag to alternate top and bottom
      OffsetFirstPage=1,     $ ; flag to offset first page
      Orientation=1,             $ ; 0=portrait, 1=landscape
      BackgroundColor=[0,0,255],     $ ; name of background color
      LeftPrintMargin=20,     $ ; size of left print margin (in.)
      RightPrintMargin=30,   $ ; size of right print margin (in.)
      TopPrintMargin=40,       $ ; size of top print margin (in.)
      BottomPrintMargin=50, $ ; size of bottom print margin (in.)
      OverlapMajorTicks=0, $ ; flag to overlap major ticks
      ShowValues=0,               $ ; set this flag to show values
      CloserThanValue=3,     $ ; show data if closer than this
      CloserThanUnits=2,     $ ; closer units (hr,min,sec,day)
      UseSameYRange=0,         $ ; flag to use the same y range 
      NumMajorTicks=7,         $ ; number of major ticks to use
      NumMinorTicks=10,         $ ; number of minor ticks to use
      SkipBlanks=0                  ; flag to skip blanks     
    myGS->GetProperty, $
      Title=title,                         $ ; text object for title
      Labels=labels,                       $ ; text object for labels
      Variables=variables,                 $ ; text object for variables
      Footer=footer,                       $ ; text object for footer
      Marker=marker,                       $ ; text object for markers  
      Token=token,                         $ ; droplist values for time format
      DefMarkerTitle=defmarkertitle,       $ ; default value for marker titles
      maintainRead=maintainRead,           $ ; flag to maintain readability
      ShowTraceNames=showtracenames,       $ ; flag to display trace names
      PanelSpacing=panelspacing,           $ ; spacing between pane
      ColPerPage=colperpage,               $ ; number of columns per page
      leftLabelMargin=leftlabelmargin,     $ ; margin size for left labels
      HeightProp=heightprop,               $ ; flag to maintain prop. heights 
      GutterWidth=gutterwidth,             $ ; width size for gutter
      DisplayOnScreen=displayonscreen,     $ ; flag to display on screen
      AltTopBottom=alttopbottom,           $ ; flag to alternate top and bottom
      OffsetFirstPage=offsetfirstpage,     $ ; flag to offset first page
      Orientation=orientation,             $ ; 0=portrait, 1=landscape
      BackgroundColor=backgroundcolor,     $ ; name of background color
      LeftPrintMargin=leftprintmargin,     $ ; size of left print margin (in.)
      RightPrintMargin=rightprintmargin,   $ ; size of right print margin (in.)
      TopPrintMargin=topprintmargin,       $ ; size of top print margin (in.)
      BottomPrintMargin=bottomprintmargin, $ ; size of bottom print margin (in.)
      OverlapMajorTicks=overlapmajorticks, $ ; flag to overlap major ticks
      ShowValues=showvalues,               $ ; set this flag to show values
      CloserThanValue=closerthanvalue,     $ ; show data if closer than this
      CloserThanUnits=closerthanunits,     $ ; closer units (hr,min,sec,day)
      UseSameYRange=usesameyrange,         $ ; flag to use the same y range 
      NumMajorTicks=nummajorticks,         $ ; number of major ticks to use
      NumMinorTicks=numminorticks,         $ ; number of minor ticks to use
      SkipBlanks=skipblanks                  ; flag to skip blanks

   IF NOT Obj_Valid(title) THEN Print, 'Error Test 3a: getting properties'
   IF NOT Obj_Valid(labels) THEN Print, 'Error Test 3b: getting properties'
   IF NOT Obj_Valid(variables) THEN Print, 'Error Test 3c: getting properties'
   IF NOT Obj_Valid(footer) THEN Print, 'Error Test 3d: etting properties'
   IF NOT Obj_Valid(marker) THEN Print, 'Error Test 3e: getting properties' 
   IF token NE 2 THEN Print, 'Error Test 3f: getting properties'
   IF defmarkertitle NE 'test' THEN Print, 'Error Test 3h: getting properties'
   IF maintainread NE 0 THEN Print, 'Error Test 3i: getting properties'
   IF showtracenames NE 1 THEN Print, 'Error Test 3j: getting properties'
   IF panelspacing NE 12 THEN Print, 'Error Test 3k: getting properties'
   IF colperpage NE 2 THEN Print, 'Error Test 3l: getting properties'
   IF leftlabelmargin NE 25 THEN Print, 'Error Test 3m: getting properties'
   IF heightprop NE 0 THEN Print, 'Error Test 3n: getting properties'
   IF gutterwidth NE 15 THEN Print, 'Error Test 3o: getting properties'
   IF displayonscreen NE 0 THEN Print, 'Error Test 3p: getting properties'
   IF alttopbottom NE 1 THEN Print, 'Error Test 3q: getting properties'
   IF offsetfirstpage NE 1 THEN Print, 'Error Test 3r: getting properties'
   IF orientation NE 1 THEN Print, 'Error Test 3s: getting properties'
   IF backgroundcolor[2] NE 255 THEN Print, 'Error Test 3t: getting properties'
   IF leftprintmargin NE 20 THEN Print, 'Error Test 3v: getting properties'
   IF rightprintmargin NE 30 THEN Print, 'Error Test 3x: getting properties'
   IF topprintmargin NE 40 THEN Print, 'Error Test 3y: getting properties'
   IF bottomprintmargin NE 50 THEN Print, 'Error Test 3z: getting properties'
   IF overlapmajorticks NE 0 THEN Print, 'Error Test 3aa: getting properties'
   IF showvalues NE 0 THEN Print, 'Error Test 3bb: getting properties' 
   IF closerthanvalue NE 3 THEN Print, 'Error Test 3cc: getting properties' 
   IF closerthanunits NE 2 THEN Print, 'Error Test 3dd: getting properties'
   IF usesameyrange NE 0 THEN Print, 'Error Test 3ff: getting properties'
   IF nummajorticks NE 7 THEN Print, 'Error Test 3gg: getting properties'
   IF numminorticks NE 10 THEN Print, 'Error Test 3hh: getting properties'
   IF skipblanks NE 0 THEN Print, 'Error Test 3ii: getting properties'

   myGS->Reset
      myGS->GetProperty, $
      Title=title,                         $ ; text object for title
      Labels=labels,                       $ ; text object for labels
      Variables=variables,                 $ ; text object for variables
      Footer=footer,                       $ ; text object for footer
      Marker=marker,                       $ ; text object for markers  
      Token=token,                         $ ; droplist values for time format
      DefMarkerTitle=defmarkertitle,       $ ; default value for marker titles
      maintainRead=maintainRead,           $ ; flag to maintain readability
      ShowTraceNames=showtracenames,       $ ; flag to display trace names
      PanelSpacing=panelspacing,           $ ; spacing between pane
      ColPerPage=colperpage,               $ ; number of columns per page
      leftLabelMargin=leftlabelmargin,     $ ; margin size for left labels
      HeightProp=heightprop,               $ ; flag to maintain prop. heights 
      GutterWidth=gutterwidth,             $ ; width size for gutter
      DisplayOnScreen=displayonscreen,     $ ; flag to display on screen
      AltTopBottom=alttopbottom,           $ ; flag to alternate top and bottom
      OffsetFirstPage=offsetfirstpage,     $ ; flag to offset first page
      Orientation=orientation,             $ ; 0=portrait, 1=landscape
      BackgroundColor=backgroundcolor,     $ ; name of background color
      LeftPrintMargin=leftprintmargin,     $ ; size of left print margin (in.)
      RightPrintMargin=rightprintmargin,   $ ; size of right print margin (in.)
      TopPrintMargin=topprintmargin,       $ ; size of top print margin (in.)
      BottomPrintMargin=bottomprintmargin, $ ; size of bottom print margin (in.)
      OverlapMajorTicks=overlapmajorticks, $ ; flag to overlap major ticks
      ShowValues=showvalues,               $ ; set this flag to show values
      CloserThanValue=closerthanvalue,     $ ; show data if closer than this
      CloserThanUnits=closerthanunits,     $ ; closer units (hr,min,sec,day)
      UseSameYRange=usesameyrange,         $ ; flag to use the same y range 
      NumMajorTicks=nummajorticks,         $ ; number of major ticks to use
      NumMinorTicks=numminorticks,         $ ; number of minor ticks to use
      SkipBlanks=skipblanks                  ; flag to skip blanks
   IF panelspacing EQ 12 THEN Print, '*****************Error Test 3: reset' ELSE Print, 'Save/Reset OK'
   print, panelspacing

   IF NOT Obj_Valid(title) THEN BEGIN
      Print, 'Error Test 5: getting properties'
   ENDIF 
   IF NOT Obj_Valid(labels) THEN BEGIN
      Print, 'Error Test 6: getting properties'
   ENDIF ELSE BEGIN

      labels->GetProperty, Value=v, Color=c
      IF c[1] NE 0 THEN Print, 'Error Test 3kk: getting properties color'
      IF v NE '' THEN Print, 'Error Test 3kk: getting properties value'
   ENDELSE
   IF NOT Obj_Valid(variables) THEN BEGIN
      Print, 'Error Test 3a: getting properties'
   ENDIF ELSE BEGIN
      variables->GetProperty, Value=v, Color=c
      IF c[2] NE 0 THEN Print, 'Error Test 3ll: getting properties'
      IF v NE 'test' THEN Print, 'Error Test 3ll: getting properties'
   ENDELSE
   IF NOT Obj_Valid(footer) THEN BEGIN
      Print, 'Error Test 3a: getting properties'
   ENDIF ELSE BEGIN
      footer->GetProperty, Value=v, Color=c
      IF c[2] NE 0 THEN Print, 'Error Test 3mm: getting properties color'
      IF v NE '' THEN Print, 'Error Test 3mm: getting properties value'
   ENDELSE
   IF NOT Obj_Valid(marker) THEN BEGIN
      Print, 'Error Test 3a: getting properties'
   ENDIF ELSE BEGIN
      marker->GetProperty, Value=v, Color=c
      IF c[0] NE 0 THEN Print, 'Error Test 3nn: getting properties'
      IF v NE '' THEN Print, 'Error Test 3nn: getting properties'
   ENDELSE
     

    Print, '**********  Done with page test  ***************'
    Obj_Destroy, myGS

END
