;+ 
;NAME:
; thm_ui_test_panel
;
;PURPOSE:
; test script for panel object
;
;CALLING SEQUENCE:
; thm_ui_test_panel
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_panel.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_panel

    ;initializations

    Print, '**********  Starting Initialization tests  ***************'

    ; Test 1: Test Initialization  
    tnames=['tha_fge', 'thb_scm', 'thc_efi']
    nptr=Ptr_New(tnames)
    markers=[1, 2, 3, 4]
    mptr=Ptr_New(markers)
    myTime = Obj_New("THM_UI_TIME_RANGE")
    myData = Obj_New("THM_UI_PANEL", 1, 2, nptr)
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 1: Did not initialize'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, WindowID=wid, Settings=s, Data=d, $
          XAxis=xa, YAxis=ya, Tracking=t, Markers=m, IsActive=ia, $
          RowNumber=rn, ColNumber=cn, TimeRange=tr, IsTime=it
       IF n NE '' THEN Print, 'Test 1a: Did not return correct property value'
       IF id NE 1 THEN Print, 'Test 1b: Did not return correct property value'
       IF rn NE 1 THEN Print, 'Test 1bb: Did not return correct property value'
       IF cn NE 1 THEN Print, 'Test 1bbb: Did not return correct property value'
       IF wid NE 2 THEN Print, 'Test 1c: Did not return correct property value'
       IF NOT Ptr_Valid(d) THEN Print, 'Test 1d: Did not return correct property value'
       IF Ptr_Valid(m) THEN Print, 'Test 1d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 1e: Did not return correct property value'
;       IF NOT Obj_Valid(xa) THEN Print, 'Test 1f: Did not return correct property value'
;       IF NOT Obj_Valid(ya) THEN Print, 'Test 1g: Did not return correct property value'
       IF t NE 1 THEN Print, 'Test 1h: Did not return correct property value'
       IF ia NE 1 THEN Print, 'Test 1i: Did not return correct property value'
    ENDELSE

    IF Obj_Valid(tr) THEN Print, 'Error Init Test 1j: Did not return time range'
    
    ; Init Test 2: Initialize, no parameters    
    myData = Obj_New("THM_UI_PANEL")
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 2: Did not give required ids'

    ; Init Test 3: No panel ID   
    myData = Obj_New("THM_UI_PANEL", 1)
    IF Obj_Valid(myData) THEN Print, 'Error Init Test 3: Did not give required panelid'
 
    ; Test 4: Test Init with all parameters
    s=Obj_New("THM_UI_PANEL_SETTINGS", FrameColor=[0,0,255])
;    xa=Obj_New("THM_UI_axis")
;    ya=Obj_New("THM_UI_axis")
    myData = Obj_New("THM_UI_PANEL", 4, 12, nptr, Name='test', Settings=s,  $
          Tracking=0, Markers=mptr, IsActive=0, RowNumber=7, ColNumber=8, $
          TimeRange=myTime, IsTime=1)
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 4: Did not initialize'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, WindowID=wid, Settings=s, Data=d, $
          XAxis=xa, YAxis=ya, Tracking=t, Markers=m, IsActive=ia, $
          RowNumber=rn, ColNumber=cn, TimeRange=tr, IsTime=it
       IF n NE 'test' THEN Print, 'Test 4a: Did not return correct property value'
       IF id NE 4 THEN Print, 'Test 4b: Did not return correct property value'
       IF rn NE 7 THEN Print, 'Test 4bb: Did not return correct property value'
       IF cn NE 8 THEN Print, 'Test 4bbb: Did not return correct property value'
       IF wid NE 12 THEN Print, 'Test 4c: Did not return correct property value'
       IF NOT Ptr_Valid(d) THEN Print, 'Test 4d: Did not return correct property value'
       IF NOT Ptr_Valid(m) THEN Print, 'Test 4d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 4e: Did not return correct property value'
;       IF NOT Obj_Valid(xa) THEN Print, 'Test 4f: Did not return correct property value'
;       IF NOT Obj_Valid(ya) THEN Print, 'Test 4g: Did not return correct property value'
       IF t NE 0 THEN Print, 'Test 4h: Did not return correct property value'
       IF ia NE 0 THEN Print, 'Test 4i: Did not return correct property value'
       s->GetProperty, FrameColor=c
       IF c[2] NE 255 THEN Print, 'Test 4j: Did not return correct frame color'
       z=*m
       IF z[2] NE 3 THEN Print, 'Test 4k: Did not return correct marker ptr value'
       zd=*d
       IF zd[1] NE 'thb_scm' THEN Print, 'Test 4l: Did not return correct data ptr value'   
    ENDELSE

    IF NOT Obj_Valid(tr) THEN Print, 'Error Init Test 1j: Did not return time range'
    tr->GetProperty, StartTime=st, EndTime=et
    IF NOT Obj_Valid(st) THEN BEGIN
       Print, 'Error Init Test 1k: Did not return start time'
    ENDIF ELSE BEGIN
       st->GetProperty, TString=ts
       Print, 'Start Time = ', ts
    ENDELSE    
    IF it NE 1 THEN Print, 'Error Init Test 1l: Did not return time range flag'

    ; Test 5: Set Property
    s=Obj_New("THM_UI_PANEL_SETTINGS", FrameColor=[0,255,0])
    myData->SetProperty, Id=6, WindowID=3, Data=nptr, Name='hi', Settings=s,  $
          Tracking=0, Markers=mptr, IsActive=0, RowNumber=11, ColNumber=12
    IF NOT Obj_Valid(myData) THEN BEGIN
       Print, 'Error Init Test 5: Did not initialize'
    ENDIF ELSE BEGIN
       myData->GetProperty, Name=n, Id=id, WindowID=wid, Settings=s, Data=d, $
          XAxis=xa, YAxis=ya, Tracking=t, Markers=m, IsActive=ia, $
          RowNumber=rn, ColNumber=cn
       IF n NE 'hi' THEN Print, 'Test 5a: Did not return correct property value'
       IF id NE 6 THEN Print, 'Test 5b: Did not return correct property value'
       IF rn NE 11 THEN Print, 'Test 5bb: Did not return correct property value'
       IF cn NE 12 THEN Print, 'Test 5bbb: Did not return correct property value'
       IF wid NE 3 THEN Print, 'Test 5c: Did not return correct property value'
       IF NOT Ptr_Valid(d) THEN Print, 'Test 5d: Did not return correct property value'
       IF NOT Ptr_Valid(m) THEN Print, 'Test 5d: Did not return correct property value'
       IF NOT Obj_Valid(s) THEN Print, 'Test 5e: Did not return correct property value'
;       IF NOT Obj_Valid(xa) THEN Print, 'Test 5f: Did not return correct property value'
;       IF NOT Obj_Valid(ya) THEN Print, 'Test 5g: Did not return correct property value'
       IF t NE 0 THEN Print, 'Test 5h: Did not return correct property value'
       IF ia NE 0 THEN Print, 'Test 5i: Did not return correct property value'
       s->GetProperty, FrameColor=c
       IF c[1] NE 255 THEN Print, 'Test 5j: Did not return correct frame color'
       z=*m
       IF z[3] NE 4 THEN Print, 'Test 5k: Did not return correct marker ptr value'
       zd=*d
       IF zd[2] NE 'thc_efi' THEN Print, 'Test 5l: Did not return correct data ptr value'   
    ENDELSE

        
    Print, '**********  End of panel tests  ***************'
    Print, ' '

    Obj_Destroy, myData
END
