;+ 
;NAME:
; thm_ui_test_spectra_settings
;
;PURPOSE:
; test script for spectra settings object
;
;CALLING SEQUENCE:
; thm_ui_test_spectra_settings
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_spectra_settings.pro $
;
;-----------------------------------------------------------------------------------
; GetAll         returns the entire structure
; GetPlacements  returns string array of placement options
; GetPlacement   returns name of placement option given an index
; GetColorTables returns string array of color tables 
; GetColorTable  returns name of color table given an index

Pro thm_ui_test_spectra_settings

    ;initializations

    Print, '**********  Starting spectra settings tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    myLS = Obj_New("THM_UI_SPECTRA_SETTINGS")
    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 1: Did not initialize'
   
    ; Test 1: Test Get for all     
    myLS->GetProperty, XAxisIndex=xai, YAxisIndex=yai, ColorTable=ct, MinRange=mr, $
       MaxRange=mar, Guess=g, AnnotationStyle=as, Scaling=sc, Placement=p, $
       ShowFrequencies=sf, FrequencyMin=fm, FrequencyMax=fma
    IF xai NE 0 THEN Print, 'Error Test 1a: Did not return xaxis index'
    IF yai NE 0 THEN Print, 'Error Test 1b: Did not return yaxis index'
    IF ct NE 0 THEN Print, 'Error Test 1c: Did not return color table'
    IF mr NE 0 THEN Print, 'Error Test 1d: Did not return correct min range'
    IF mar NE 0 THEN Print, 'Error Test 1e: Did not return correct max range'
    IF as NE 0 THEN Print, 'Error Test 1f: Did not return annotation style'
    IF sc NE 0 THEN Print, 'Error Test 1g: Did not return correct scaling'
    IF p NE 0 THEN Print, 'Error Test 1h: Did not return correct placement'
    IF sf NE 0 THEN Print, 'Error Test 1i: Did not return correct show frequency'
    IF g NE 0 THEN Print, 'Error Test 1j: Did not return correct guess '
    IF fm NE 0 THEN Print, 'Error Test 1k: Did not return correct freq. min'
    IF fma NE 0 THEN Print, 'Error Test 1l: Did not return correct freq. max'

     ; Test 2: Initialize with parameters 
;    myLS = Obj_New("THM_UI_SPECTRA_SETTINGS", XAxisIndex=1, YAxisIndex=1, ColorTable=2, MinRange=3, $
;       MaxRange=4, Guess=1, AnnotationStyle=3, Scaling=5, Placement=3, $
;       ShowFrequencies=1, FrequencyMin=7, FrequencyMax=6)
;    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 2: Did not initialize'
   
    ; Test 2: Test Get for all     
;    Obj_Destroy, myLS
;    myLS->GetProperty, XAxisIndex=xai, YAxisIndex=yai, ColorTable=ct, MinRange=mr, $
;       MaxRange=mar, Guess=g, AnnotationStyle=as, Scaling=sc, Placement=p, $
;       ShowFrequencies=sf, FrequencyMin=fm, Frequency=fma
;    IF xai NE 1 THEN Print, 'Error Test 2a: Did not return xaxis index'
;    IF yai NE 1 THEN Print, 'Error Test 2b: Did not return yaxis index'
;    IF ct NE 2 THEN Print, 'Error Test 2c: Did not return color table'
;    IF mr NE 3 THEN Print, 'Error Test 2d: Did not return correct min range'
;    IF mar NE 4 THEN Print, 'Error Test 2e: Did not return correct max range'
;    IF as NE 1 THEN Print, 'Error Test 2f: Did not return annotation style'
;    IF sc NE 5 THEN Print, 'Error Test 2g: Did not return correct scaling'
;    IF p NE 3 THEN Print, 'Error Test 2h: Did not return correct placement'
;    IF sf NE 1 THEN Print, 'Error Test 2i: Did not return correct show frequency'
;    IF g NE 1 THEN Print, 'Error Test 2j: Did not return correct guess '
;    IF fm NE 7 THEN Print, 'Error Test 2k: Did not return correct freq. min'
;    IF fma NE 6 THEN Print, 'Error Test 2l: Did not return correct freq. max'

     ; Test 2: Initialize with parameters 
;    myLS = Obj_New("THM_UI_SPECTRA_SETTINGS", XAxisIndex=1, YAxisIndex=1, ColorTable=2, MinRange=3, $
;       MaxRange=4, Guess=1, AnnotationStyle=3, Scaling=5, Placement=3, $
;       ShowFrequencies=1, FrequencyMin=7, FrequencyMax=6)
;    IF NOT Obj_Valid(myLS) THEN Print, 'Error Init Test 2: Did not initialize'
;   
    ; Test 3: Test Get for all     
    myLS->SetProperty, XAxisIndex=1, YAxisIndex=1, ColorTable=3, MinRange=4, $
       MaxRange=5, Guess=1, AnnotationStyle=2, Scaling=6, Placement=2, $
       ShowFrequencies=1, FrequencyMin=8, FrequencyMax=9

    myLS->GetProperty, XAxisIndex=xai, YAxisIndex=yai, ColorTable=ct, MinRange=mr, $
       MaxRange=mar, Guess=g, AnnotationStyle=as, Scaling=sc, Placement=p, $
       ShowFrequencies=sf, FrequencyMin=fm, FrequencyMax=fma
    IF xai NE 1 THEN Print, 'Error Test 3a: Did not return xaxis index'
    IF yai NE 1 THEN Print, 'Error Test 3b: Did not return yaxis index'
    IF ct NE 3 THEN Print, 'Error Test 3c: Did not return color table'
    IF mr NE 4 THEN Print, 'Error Test 3d: Did not return correct min range'
    IF mar NE 5 THEN Print, 'Error Test 3e: Did not return correct max range'
    IF as NE 2 THEN Print, 'Error Test 3f: Did not return annotation style'
    IF sc NE 6 THEN Print, 'Error Test 3g: Did not return correct scaling'
    IF p NE 2 THEN Print, 'Error Test 3h: Did not return correct placement'
    IF sf NE 1 THEN Print, 'Error Test 3i: Did not return correct show frequency'
    IF g NE 1 THEN Print, 'Error Test 3j: Did not return correct guess '
    IF fm NE 8 THEN Print, 'Error Test 3k: Did not return correct freq. min'
    IF fma NE 9 THEN Print, 'Error Test 3l: Did not return correct freq. max'

    ; Test 4: Test Get Color Tables     
    ct=myLS->GetColorTables()
    IF N_Elements(ct) NE 6 THEN Print, 'Error Test 4: Did not return right num elements of color tables'
    Print, 'Tables = ', ct

    ; Test 5: Test Get Placements     
    pls=myLS->GetPlacements()
    IF N_Elements(pls) NE 4 THEN Print, 'Error Test 5: Wrong number elements for placements'
    Print, 'Placements = ', pls
    
    ; Test 6: Test Get Color Table
    name=myLS->GetColorTable(3)
    IF name NE 'Copper' THEN Print, 'Error Test 6: Wrong color table name'

    ; Test 7: Test Get Color Table
    name=myLS->GetPlacement(3)
    IF name NE 'Right' THEN Print, 'Error Test 7: Wrong placement name'

    ; Test 8: Test Get  Annotation list
    names=myLS->GetAnnotations()
    IF N_Elements(names) NE 15 THEN Print, 'Error Test 8: Wrong number annotations'

    ; Test 9: Test Get Annotation
    name=myLS->GetAnnotation(3)
    IF name NE 'Right' THEN Print, 'Error Test 9: Wrong placement name'

    ; Test 10: Test Get All
    obj=myLS->GetAll()
    IF NOT Obj_Valid(obj) THEN Print, 'Error Test 8: Invalid object from obj�

    Print, '**********  Starting spectra settings tests  ***************'


Obj_Destroy, myLS
    
END

