;+ 
;NAME:
; thm_ui_test_text
;
;PURPOSE:
; test script for text
;
;CALLING SEQUENCE:
; thm_ui_test_symbol
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_text.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_text

    ;initializations

    Print, '**********  Starting symbol tests  ***************'

    ; Test 1: Initialize with no parameters, all defaults    
    s = Obj_New("THM_UI_TEXT")   
    IF NOT Obj_Valid(s) THEN Print, 'Error Init Test 1: Did not initialize'

    ; Test 2: Test Get for all for symbol object     
    s->GetProperty, Value=v, font=f, format=fmt, Color=c, Size=si, Thickness=t, Show=sf
    IF v NE '' THEN Print, 'Error Test 2: Did not return correct name'
    IF f NE 0 THEN Print, 'Error Test 2: Did not return correct font'
    IF fmt NE 0 THEN Print, 'Error Test 2: Did not return correct format'
    IF c[1] NE 0 THEN Print, 'Error Test 2: Did not return correct color'
    IF t NE 1 THEN Print, 'Error Test 2: Did not return correct thickness'
    IF si NE 1 THEN Print, 'Error Test 2: Did not return size'
    IF sf NE 1 THEN Print, 'Error Test 2: Did not return correct show flag'
    
    Obj_Destroy, s
  
    ; Test 3: Test Initialize with valid parameters 
    s = Obj_New("THM_UI_TEXT", Value='Text', Font=1, Format=2, Color=[20,20,20], Size=3, Thickness=4, Show=0)   
    IF NOT Obj_Valid(s) THEN Print, 'Error Init Test 3: Did not initialize with parameters'

     ; Test 4: Test Get for all for symbol object     
    s->GetProperty, Value=v, font=f, format=fmt, Color=c, Size=si, Thickness=t, Show=sf
    IF v NE 'Text' THEN Print, 'Error Test 4: Did not return correct name'
    IF f NE 1 THEN Print, 'Error Test 4: Did not return correct font'
    IF fmt NE 2 THEN Print, 'Error Test 4: Did not return correct format'
    IF c[2] NE 20 THEN Print, 'Error Test 4: Did not return correct color'
    IF t NE 4 THEN Print, 'Error Test 4: Did not return correct thickness'
    IF si NE 3 THEN Print, 'Error Test 4: Did not return size'
    IF sf NE 0 THEN Print, 'Error Test 4: Did not return correct show flag'
 
    ; Test 5: Test Get Fonts    
    fonts = s->GetFonts()
    IF N_Elements(fonts) NE 13 THEN Print, 'Error Test 5: Did not return correct number of fonts'
    Print, fonts
    
    ; Test 6: Test Get Formats    
    formats = s->GetFormats()
    IF N_Elements(formats) NE 5 THEN Print, 'Error Test 6: Did not return correct number of fonts'
    Print, formats


    ; Test 7: Test Get Font Index    
    sid = s->GetFontIndex()
    IF sid NE 1 THEN Print, 'Error Test 7: Did not return correct font index'
    sid = s->GetFontIndex(FontName='Helvetica')
    IF sid NE 4 THEN Print, 'Error Test 7: Did not return correct font index when given name'
    sid = s->GetFontIndex(FontName=4)
    IF sid NE -1 THEN Print, 'Error Test 7: Did not correctly return -1 for error'
    sid = s->GetFontIndex(FontName='xyz')
    IF sid NE -1 THEN Print, 'Error Test 7: Did not correctly return -1 for error'

    ; Test8: Test Get Format Index  
    sid = s->GetFormatIndex()
    IF sid NE 2 THEN Print, 'Error Test 8: Did not return correct format index'
    sid = s->GetFormatIndex(FormatName='Superscript')
    IF sid NE 2 THEN Print, 'Error Test 8: Did not return correct format index when given name'
    sid = s->GetFormatIndex(FormatName=8)
    IF sid NE -1 THEN Print, 'Error Test 8: Did not correctly return -1 for error'
    sid = s->GetFormatIndex(FormatName='xyz')
    IF sid NE -1 THEN Print, 'Error Test 8: Did not correctly return -1 for error'

    ; Test 9: Test Get Font Index  

    sid = s->GetFont()
    IF sid NE 'AvantGarde' THEN Print, 'Error Test 9: Did not return correct font index'
    sid = s->GetFont(Index=3)
    IF sid NE 'Courier' THEN Print, 'Error Test 9: Did not return correct font index when given name'
    sid = s->GetFont(Index=34)
    IF sid NE -1 THEN Print, 'Error Test 9: Did not correctly return -1 for character error'
    sid = s->GetFont(Index='xyz')
    IF sid NE -1 THEN Print, 'Error Test 9: Did not correctly return -1 for error'

    ; Test 10: Test Get Format Index 
    sid = s->GetFormat()
    IF sid NE 'Superscript' THEN Print, 'Error Test 10: Did not return correct format index'
    sid = s->GetFormat(Index=1)
    IF sid NE 'Italic' THEN Print, 'Error Test 10: Did not return correct format index when given name'
    sid = s->GetFormat(Index=45)
    IF sid NE -1 THEN Print, 'Error Test 10: Did not correctly return -1 for error'
    sid = s->GetFormat(Index='xyz')
    IF sid NE -1 THEN Print, 'Error Test 10: Did not correctly return -1 for error'

    Print, '**********  Done with ID test  ***************'
    Obj_Destroy, s
    
END
