;+ 
;NAME:
; thm_ui_test_variable
;
;PURPOSE:
; test script for data range object
;
;CALLING SEQUENCE:
; thm_ui_test_variable
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_variable.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_variable

    ;initializations

    Print, '**********  Starting variable tests  ***************'
    ; Test 1: Initialize with no parameters, all defaults    
    myGS = Obj_New("THM_UI_VARIABLE")
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 1: Did not initialize'
    ; Test 2: Test Get for all     

    myGS->GetProperty, Data=d, Text=t, Symbol=s, Format=f, LabelMargin=lm
    IF NOT Obj_Valid(d) THEN Print,'Error Init Test 1a: Did not return data object' 
    IF NOT Obj_Valid(t) THEN Print,'Error Init Test 1b: Did not return text object' 
    IF s NE 0 THEN Print,'Error Init Test 1c: Did not return symbol' 
    IF f NE 0 THEN Print,'Error Init Test 1d: Did not return format' 
    IF lm NE 0 THEN Print,'Error Init Test 1e: Did not return label margin' 
 
    ; Test 2: Initialize with parameters
    dObj = Obj_New("THM_UI_DATA", Name='Hi', Id=1, GroupID=2, GroupName='There')
    tObj = Obj_New("THM_UI_TEXT", Value='Hello')
    myGS = Obj_New("THM_UI_VARIABLE", Data=dObj, Text=tObj, Symbol=3, $
       Format=4, LabelMargin=5)
    IF NOT Obj_Valid(myGS) THEN Print, 'Error Init Test 2: Did not initialize'
    ; Test 2: Test Get for all     
    myGS->GetProperty, Data=d, Text=t, Symbol=s, Format=f, LabelMargin=lm
    IF NOT Obj_Valid(d) THEN BEGIN
       Print,'Error Init Test 2a: Did not return data object' 
    ENDIF ELSE BEGIN
       d->GetProperty, Id=id, GroupName=gn
       IF id NE 1 THEN Print,'Error Init Test 2b: Did not return data object' 
       IF gn NE 'There' THEN Print,'Error Init Test 2c: Did not return data object' 
    ENDELSE 
    IF NOT Obj_Valid(t) THEN BEGIN
      Print,'Error Init Test 2d: Did not return text object' 
    ENDIF ELSE BEGIN
       t->GetProperty, Value=v, Color=c
       IF v NE 'Hello' THEN Print,'Error Init Test 2e: Did not return text object' 
       IF c[1] NE 0 THEN Print,'Error Init Test 2f: Did not return text object' 
    ENDELSE  
    IF s NE 3 THEN Print,'Error Init Test 2g: Did not return symbol' 
    IF f NE 4 THEN Print,'Error Init Test 2h: Did not return format' 
    IF lm NE 5 THEN Print,'Error Init Test 2i: Did not return label margin' 
    
    ; Test 3: Test for Set Data
    dObj->SetProperty, Name='Test', Id=2, GroupName='Test1'
    tObj->SetProperty, Color=[21,21,21]
    myGS->SetProperty, Data=dObj, Text=tObj, Symbol=6, Format=7, LabelMargin=8 
    myGS->GetProperty, Data=d, Text=t, Symbol=s, Format=f, LabelMargin=lm
    IF NOT Obj_Valid(d) THEN BEGIN
       Print,'Error Init Test 3a: Did not return data object' 
    ENDIF ELSE BEGIN
       d->GetProperty, Name=n, Id=id, GroupName=gn
       IF id NE 2 THEN Print,'Error Init Test 3b: Did not return data object' 
       IF n NE 'Test' THEN Print,'Error Init Test 3c: Did not return data object' 
       IF gn NE 'Test1' THEN Print,'Error Init Test 3cc: Did not return data object' 
    ENDELSE 
    IF NOT Obj_Valid(t) THEN BEGIN
      Print,'Error Init Test 3d: Did not return text object' 
    ENDIF ELSE BEGIN
       t->GetProperty, Value=v, Color=c
       IF v NE 'Hello' THEN Print,'Error Init Test 3e: Did not return text object' 
       IF c[1] NE 21 THEN Print,'Error Init Test 3f: Did not return text object' 
    ENDELSE  
    IF s NE 6 THEN Print,'Error Init Test 3g: Did not return symbol' 
    IF f NE 7 THEN Print,'Error Init Test 3h: Did not return format' 
    IF lm NE 8 THEN Print,'Error Init Test 3i: Did not return label margin' 
       
    Print, '**********  Done with variable test  ***************'
    Obj_Destroy, myGS

END
