;+ 
;NAME:
; thm_ui_test_windows
;
;PURPOSE:
; test for windows object
;
;CALLING SEQUENCE:
; thm_ui_test_windows
;
;HISTORY:
;$LastChangedBy: cgoethel $
;$LastChangedDate: 2008-10-27 08:42:40 -0700 (Mon, 27 Oct 2008) $
;$LastChangedRevision: 3818 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/tests/procedures/thm_ui_test_windows.pro $
;
;-----------------------------------------------------------------------------------

Pro thm_ui_test_windows

    Print, '**********  Starting windows tests  ***************'
    st = '2008-01-02/18:09:57'
    et = '2008-01-09/00:00:00'
    start = Obj_New("THM_UI_TIME", TString=st)
    endt = Obj_New("THM_UI_TIME", TString=et)
    tr=Obj_New("THM_UI_TIME_RANGE", StartTime=start, EndTime=endt)
    variables=['thb_state_pos', 'thd_state_vel']
    vptr=Ptr_New(variables)
    panels=[1, 2, 3, 4, 5, 6]
    pptr=Ptr_New(panels)
    setDefault=Obj_New("THM_UI_PAGE_SETTINGS")
    setBlue=Obj_New("THM_UI_PAGE_SETTINGS", BackgroundColor=[0,0,255])
    setRed=Obj_New("THM_UI_PAGE_SETTINGS", BackgroundColor=[255,0,0])
    setGreen=Obj_New("THM_UI_PAGE_SETTINGS", BackgroundColor=[0,255,0])

    ; Test 1: create panel with no input
    windows = Obj_New("THM_UI_WINDOWS")
    IF NOT Obj_Valid(windows) THEN Print, 'Error Test 1: failed to fail initialization' 
    
    ; Test 2: Add object using defaults
    result = windows->Add()
    IF result NE 1 THEN BEGIN
      Print, 'Error Test 2: failed to add a window object'
    ENDIF ELSE BEGIN
      windows->GetProperty, WindowObjs=po, Id=id
      IF N_Elements(po) LE 0 THEN Print, 'Error Test 2a: failed to return property'
      IF id NE 2 THEN Print, 'Error Test 2b: failed to return property'
      pa=*po
      pa[0]->GetProperty, Id=id, Name=n, NRows=rn, $
          NCols=cn, IsActive=ia, Variables=v, Tracking=t
      IF n NE '' THEN Print, 'Error Test 2c: failed to return property'
      IF id NE 1 THEN Print, 'Error Test 2d: failed to return property'
      IF Ptr_Valid(v) THEN Print, 'Error Test 2e: failed to return property'
      IF rn NE 2 THEN Print, 'Error Test 2g: failed to return property'
      IF cn NE 1 THEN Print, 'Error Test 2h: failed to return property'
      IF ia NE 1 THEN Print, 'Error Test 2i: failed to return property'
      IF t NE 1 THEN Print, 'Error Test 2j: failed to return property'
    ENDELSE    

    ; Test 3: Add one more object with parameters   
    result = windows->Add(Variables=vptr, $
      NRows=5, NCols=2, Name='hi', IsActive=0, Tracking=0)
    IF result NE 1 THEN BEGIN
      Print, 'Error Test 3: failed to add a panel object'
    ENDIF ELSE BEGIN
      windows->GetProperty, WindowObjs=po, Id=id
      IF NOT Ptr_Valid(po) THEN Print, 'Error Test 3a: failed to return property'
      IF id NE 3 THEN Print, 'Error Test 3b: failed to return property'
      pa=*po
      pa[1]->GetProperty, Id=id, Name=n, NRows=rn, $
          NCols=cn, IsActive=ia, Variables=v, Tracking=t, Panels=p
      IF N_Elements(pa) LE 1 THEN Print, 'Error Test 3a: failed to return property'
      IF n NE 'hi' THEN Print, 'Error Test 3c: failed to return property'
      IF id NE 2 THEN Print, 'Error Test 3d: failed to return property'
      IF NOT Ptr_Valid(v) THEN BEGIN
         Print, 'Error Test 3e: failed to return property'
      ENDIF ELSE BEGIN
         v1=*v
         IF v1[0] NE 'thb_state_pos' THEN Print, 'Error Test 3f: failed to return property'
      ENDELSE
      IF rn NE 5 THEN Print, 'Error Test 3h: failed to return property'
      IF cn NE 2 THEN Print, 'Error Test 3i: failed to return property'
      IF ia NE 0 THEN Print, 'Error Test 3j: failed to return property'
      IF t NE 0 THEN Print, 'Error Test 3k: failed to return property'
    ENDELSE    

    ; Test 4: Add several more objects     
    result = windows->Add(Variables=vptr, Settings=setBlue,$
      NRows=2, NCols=1, Name='there', IsActive=1, Tracking=0, Panels=pptr)
    result = windows->Add(Variables=vptr, Settings=setRed,$
      NRows=3, NCols=2, Name='how', IsActive=0, Tracking=1, Panels=pptr)
    result = windows->Add(Variables=vptr, Settings=setGreen,$
      NRows=4, NCols=3, Name='are', IsActive=1, Tracking=0, Panels=pptr)
    result = windows->Add(Variables=vptr, Settings=setBlue,$
      NRows=5, NCols=4, Name='you', IsActive=0, Tracking=1, Panels=pptr)
    IF result NE 1 THEN BEGIN
      Print, 'Error Test 4: failed to add a panel object'
    ENDIF ELSE BEGIN
      windows->GetProperty, WindowObjs=po, Id=id
      IF NOT Ptr_Valid(po) THEN Print, 'Error Test 4a: failed to return property'
      IF id NE 7 THEN Print, 'Error Test 4b: failed to return property'
      pa=*po
      pa[5]->GetProperty, Id=id, Name=n, NRows=rn, $
          NCols=cn, IsActive=ia, Variables=v, Tracking=t, Panels=p
      IF N_Elements(pa) LE 1 THEN Print, 'Error Test 4a: failed to return property'
      IF n NE 'you' THEN Print, 'Error Test 4c: failed to return property'
      IF id NE 6 THEN Print, 'Error Test 4d: failed to return property'
      IF NOT Ptr_Valid(v) THEN BEGIN
         Print, 'Error Test 4e: failed to return property'
      ENDIF ELSE BEGIN
         v1=*v
         IF v1[1] NE 'thd_state_vel' THEN Print, 'Error Test 4f: failed to return property'
      ENDELSE
      IF rn NE 5 THEN Print, 'Error Test 4i: failed to return property'
      IF cn NE 4 THEN Print, 'Error Test 4j: failed to return property'
      IF ia NE 0 THEN Print, 'Error Test 4k: failed to return property'
      IF t NE 1 THEN Print, 'Error Test 4l: failed to return property'
      IF NOT Ptr_Valid(p) THEN BEGIN
         Print, 'Error Test 4m: failed to return property'
      ENDIF ELSE BEGIN
         p1=*p
         IF p1[3] NE 4 THEN Print, 'Error Test 4n: failed to return property'
      ENDELSE 
    ENDELSE    
    pa[3]->GetProperty, Id=id, Name=n, NRows=rn, Settings=s,$
          NCols=cn, IsActive=ia, Variables=v, Tracking=t, Panels=p
    IF NOT Obj_Valid(s) THEN BEGIN
       Print, 'Error Test 4o: failed to return property'
    ENDIF ELSE BEGIN
       s->GetProperty, BackgroundColor=bc
       IF bc[0] NE 255 THEN Print, 'Error Test 4p: failed to return property'  
    ENDELSE
        
    pa = windows->GetActive()
    IF N_Elements(pa) NE 3 THEN Print, 'Error Test 5: failed to get active'
    
    windows->ClearActive, Id=3
    pa = windows->GetActive()
    IF N_Elements(pa) NE 2 THEN Print, 'Error Test 6: failed to get active'
    pa[0]->GetProperty, IsActive=ia, Id=id
    IF ia NE 1 THEN Print, 'Error Test 6a: failed to get active'
    IF id NE 1 THEN Print, 'Error Test 6b: failed to get active'
    pa[1]->GetProperty, IsActive=ia, Id=id
    IF ia NE 1 THEN Print, 'Error Test 6c: failed to get active'
    IF id NE 5 THEN Print, 'Error Test 6d: failed to get active'

    windows->ClearActive, Id=1
    pa = windows->GetActive()
    IF N_Elements(pa) NE 1 THEN Print, 'Error Test 7: failed to get active'
    pa[0]->GetProperty, IsActive=ia, Id=id
    IF ia NE 1 THEN Print, 'Error Test 7a: failed to get active'
    IF id NE 5 THEN Print, 'Error Test 7a: failed to get active'
        
    windows->ClearAllActive
    pa = windows->GetActive()
    IF pa NE 0 THEN Print, 'Error Test 8: failed to clear active'
    
    windows->SetActive, Id=4
    pa = windows->GetActive()
    IF N_Elements(pa) NE 1 THEN Print, 'Error Test 9: failed to set active'    
    pa[0]->GetProperty, IsActive=ia, Id=id
    IF ia NE 1 THEN Print, 'Error Test 10a: failed to get active'
    IF id NE 4 THEN Print, 'Error Test 10b: failed to get active'
    
    windows->GetProperty, WindowObjs=po, Id=id
    pa=*po
    result=windows->RemoveObject(pa[4])
    windows->GetProperty, WindowObjs=po, Id=id
    IF N_Elements(*po) NE 5 THEN Print, 'Error Test 11: failed to return property'
    pa=*po
    pa[4]->GetProperty, IsActive=ia, Id=id
    IF id NE 6 THEN Print, 'Error Test 12: failed to return property'
        
    Print, '**********  End of windows test  ***************'
    
    Obj_Destroy, windows
END
