


;An event-handling routine for the calendar control
;(Remember that in the ActiveXCal.pro file, this procedure occurs before the ActiveXCal procedure.)
PRO thm_ui_calendar_event, ev 
;The ActiveX widget is the only widget in this application that generates widget events,
;so the ID field of the event structure is guaranteed to contain the widget ID of that widget.
;Use the GET_VALUE keyword to retrieve an object reference to the IDLcomActiveX object that encapsulates the control.
   WIDGET_CONTROL, ev.ID, GET_VALUE = oCal 
;The user value of the top-level base widget is an anonymous structure that holds the widget IDs of the month, day,
;and year label widgets (see step 6 above). Retrieve the structure into a variable named state.
   WIDGET_CONTROL, ev.TOP, GET_UVALUE = state 
   WIDGET_CONTROL, ev.ID, GET_UVALUE = uval
   IF Size(uval, /Type) NE 0 THEN BEGIN
     CASE uval OF
        'HOURS': BEGIN
          widget_control, ev.id, get_value = hours
          IF hours LT 0 OR hours GE 24 THEN BEGIN
            state.hour = 0
            Widget_Control, ev.id, set_value = 0
          ENDIF ELSE BEGIN
            state.hour=hours
          ENDELSE
        END
        'MINUTES': BEGIN
          widget_control, ev.id, get_value = minutes
          IF minutes LT 0 OR minutes GE 60 THEN BEGIN
            state.minute = 0
            Widget_Control, ev.id, set_value = 0
          ENDIF ELSE BEGIN
            state.minute=minutes
          ENDELSE
        END
        'SECONDS': BEGIN
          widget_control, ev.id, get_value = seconds
          IF seconds LT 0 OR seconds GE 60 THEN BEGIN
            state.second = 0
            Widget_Control, ev.id, set_value = 0
          ENDIF ELSE BEGIN
            state.second=seconds
          ENDELSE
        END
        'OK': begin
          widget_control, ev.top, set_uvalue = state
          widget_control, state.actx, get_value=date
          date->getproperty, month=month, day=day, year=year
          if year eq 0 then begin
            ok=dialog_message('Please select a calendar date.')
            return
          endif
          datetime = time_double(strtrim(year,2)+'-'+strtrim(month,2)+'-'+strtrim(day,2) $
                 +'/'+strtrim(state.hour,2)+':'+strtrim(state.minute,2)+':'+strtrim(state.second,2))
          state.otime->setproperty, tdouble=datetime
          widget_control, ev.top, set_uvalue=state, /no_copy
          Widget_Control, ev.top, /Destroy
          return
        end
        'CANCEL': begin
          widget_control, ev.top, /Destroy
          return
        end
      ENDCASE
    ENDIF ELSE BEGIN
        ;Use the GetProperty method on the IDLcomActiveX object to retrieve
        ;the current values of the month, day, and year from the calendar control.
        ocal->GetProperty, month=month, day=day, year=year 
        ;Use WIDGET_CONTROL to set the values of the month, day, and year label widgets.
        WIDGET_CONTROL, state.month, SET_VALUE = STRTRIM(month,2) 
        WIDGET_CONTROL, state.day, SET_VALUE = STRTRIM(day,2) 
        WIDGET_CONTROL, state.year, SET_VALUE = STRTRIM(year,2) 
        ;Call HEAP_FREE to ensure that dynamic portions of the event structure are released, and end the procedure.
        HEAP_FREE, ev 
   ENDELSE
   
   widget_control, ev.top, set_uvalue=state, /no_copy
   
   Return
   
END ;--------------------------------------------------------------




PRO thm_ui_calendar, title, otime, gui_id

  if obj_valid(otime) then begin
    intime=otime->getstructure()
  endif else begin
    otime=obj_new('thm_ui_time')
  endelse

;Create a top-level base widget to hold the ActiveX control.
   wBase = WIDGET_BASE(COLUMN = 1, SCR_XSIZE = 400, $ 
      TITLE=title, /Align_Center, /modal, group_leader=gui_id) 
;Create base widgets to hold labels for the selected month, day, and year. Set the initial values of the labels.
   wDateBase = WIDGET_BASE(wBase, /ROW)
   wYearBase = WIDGET_BASE(wDateBase, /COL)
   wTimeBase = WIDGET_BASE(wDateBase, /COL)
   wSubBase = WIDGET_BASE(wYearBase, /ROW) 
   wSubTimeBase = WIDGET_BASE(wTimeBase, /COL, xpad=125)
   wVoid = WIDGET_LABEL(wSubBase, VALUE = 'Year: ') 
   wYear = WIDGET_LABEL(wSubBase, VALUE = '1999')
   wSubBase = WIDGET_BASE(wYearBase, /ROW) 
   wVoid = WIDGET_LABEL(wSubBase, value = 'Month: ') 
   wMonth = WIDGET_LABEL(wSubBase, value = 'October') 
   wSubBase = WIDGET_BASE(wYearBase, /ROW) 
   wVoid = WIDGET_LABEL(wSubBase, VALUE = 'Day: ') 
   wDay = WIDGET_LABEL(wSubBase, VALUE = '22')    
   wHour=thm_ui_spinner(wSubTimeBase, Label='Hours:       ', Increment=1, Value=intime.hour, uval='HOURS')
   wMinute=thm_ui_spinner(wSubTimeBase, Label='Minutes:    ', Increment=1, Value=intime.min, uval='MINUTES')
   wSecond=thm_ui_spinner(wSubTimeBase, Label='Seconds:  ', Increment=1, Value=intime.sec, uval='SECONDS')

;Instantiate the ActiveX Control, using the control's class ID.
   wAx=WIDGET_ACTIVEX(wBase, $ 
     '{8E27C92B-1264-101C-8A2F-040224009C02}') 
 
      
;Create the ok and cancel buttons
   buttonBase = WIDGET_BASE(wBase, /row, /align_center)
   okButton = WIDGET_BUTTON(buttonBase, value='OK', uValue='OK', xsize=75)
   cancelButton = WIDGET_BUTTON(buttonBase, value='Cancel', uValue='CANCEL', xsize=75)
   
; Realize the top-lvel base widget 
   WIDGET_CONTROL, wBase, /REALIZE 
;Set the top-level base's user value to an anonymous structure containing widget IDs of the month, day, and year label widgets.
   WIDGET_CONTROL, wBase, $ 
      SET_UVALUE = {month:wMonth, day:wDay, year:wYear, hour:0, minute:0, second:0, otime:otime, ActX:wAx} 
;Retrieve the object ID of the IDLcomActiveX object that encapsulates the ActiveX control.
   WIDGET_CONTROL, wAx, GET_VALUE = oAx 
;Initialize calendar buttons
   oAx->SetProperty, month=intime.month, day=intime.date, year=intime.year
;Use the GetProperty method to retrieve the current values of the month, day, and year from the control.
   oAx->GetProperty, month=month, day=day, year=year
;Set the values of the label widgets to reflect the current date, as reported by the ActiveX control.
   WIDGET_CONTROL, wMonth, SET_VALUE=STRTRIM(month, 2) 
   WIDGET_CONTROL, wDay, SET_VALUE=STRTRIM(day, 2) 
   WIDGET_CONTROL, wYear, SET_VALUE=STRTRIM(year, 2) 
 
   
;Call XMANAGER to manage the widget events, and end the procedure.
   XMANAGER, 'thm_ui_calendar', wBase 
   
   return
 
END ;--------------------------------------------------------------
      
